#!/usr/bin/env python
# -*- coding: utf-8 -*-
#######################################################################
#
#    YAMP - Yet Another Music Player - YampPixmaps
#    Version 3.1.2 2016-11-02
#    Coded by JohnHenry (c)2013
#    Extended by AlfredENeumann (c)2016
#    Support: www.vuplus-support.org
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#######################################################################

from YampGlobals import *

from Components.AVSwitch import AVSwitch
from Components.Pixmap import Pixmap

from enigma import ePicLoad

#
# The following class provides methods for showing coverart pictures
#
class YampCoverArtPixmap(Pixmap):
	def __init__(self):
		Pixmap.__init__(self)
		self.coverArtFilename = ""
		self.picload = ePicLoad()
		if os.path.exists("/var/lib/dpkg/status"): self.picload_conn=self.picload.PictureData.connect(self.paintCoverArtPixmapCB)
		else:	self.picload.PictureData.get().append(self.paintCoverArtPixmapCB)

	def applySkin(self, desktop, screen):
		from Tools.LoadPixmap import LoadPixmap
		self.noCoverFile = None
		if self.skinAttributes is not None:
			for (attrib, value) in self.skinAttributes:
				if attrib == "pixmap":
					self.noCoverFile = value
					break
		if self.noCoverFile is None:
			self.noCoverFile = yampDir + "images/no_coverArt.png"
		self.noCoverPixmap = LoadPixmap(self.noCoverFile)
		return Pixmap.applySkin(self, desktop, screen)

	def onShow(self):
		Pixmap.onShow(self)
		sc = AVSwitch().getFramebufferScale()
		#0=Width 1=Height 2=Aspect 3=use_cache 4=resize_type 5=Background(#AARRGGBB)
		self.picload.setPara((self.instance.size().width(), self.instance.size().height(), sc[0], sc[1], False, 1, "#00000000"))

	def paintCoverArtPixmapCB(self, picInfo=None):
		ptr = self.picload.getData()
		if ptr != None:
			LOG('YampCoverArtPixmap: paintCoverArtPixmapCB: ptr = ok', 'all')
#			self.instance.setPixmap(ptr.__deref__())
			self.instance.setPixmap(ptr)
		else:
			try:
				self.instance.setPixmap(self.noCoverPixmap)
				LOG('YampCoverArtPixmap: paintCoverArtPixmapCB: ptr = None -> noCoverPixmap', 'all')
			except:
				LOG('YampCoverArtPixmap: paintCoverArtPixmapCB: ptr = None EXCEPT', 'err')
				
	def showDefaultCover(self):
		self.coverArtFilename = self.noCoverFile
		self.instance.setPixmap(self.noCoverPixmap)
		self.copyCover()

	def showCoverArt(self, filename):
		LOG('YampCoverArtPixmap: showCoverArt: Start: filename: %s' %(filename) , 'all')
		self.coverArtFilename = filename
		self.picload.startDecode(self.coverArtFilename)
		LOG('YampCoverArtPixmap: showCoverArt: after startDecode: filename: %s' %(filename) , 'all')
		self.copyCover()
		LOG('YampCoverArtPixmap: showCoverArt: End: filename: %s' %(filename) , 'all')
		
	def getFileName(self):
#		try:
#			LOG('YampCoverArtPixmap: getFileName: self.coverArtFilename: %s' %(self.coverArtFilename) , 'err')
#		except:
#			LOG('YampCoverArtPixmap: getFileName: self.coverArtFilename: EXCEPT' , 'err')
		return self.coverArtFilename
		
	def copyCover(self):
		# copy cover to unique location for third party apps, use 8192 byte chunks
		fn = self.coverArtFilename
		if fn == "":
			fn = yampDir + "images/transparent.png"
		with open(fn, 'rb') as fsrc:
			with open("/tmp/.cover", 'wb') as fdst:
				buf = fsrc.read(8192)
				while len(buf) > 0:
					fdst.write(buf)
					buf = fsrc.read(8192)

					
#
# The following class provides methods for showing screen saver slides
#
class YampScreenSaver(Pixmap):
	def __init__(self):
		Pixmap.__init__(self)
		self.picload = ePicLoad()
		if os.path.exists("/var/lib/dpkg/status"): self.picload_conn=self.picload.PictureData.connect(self.finish_decode)
		else:	self.picload.PictureData.get().append(self.finish_decode)
		self.decodeFinished = True
		self.isDreamOS = os.path.exists("/var/lib/dpkg/status")
#		self.screenSaverTimer = eTimer()
#		if self.isDreamOS: self.screenSaverTimer_conn=self.screenSaverTimer.timeout.connect(self.showScreenSaverNext)
#		else:	self.screenSaverTimer.callback.append(self.showScreenSaverNext)
		
	def setList(self, slideList, slideIndex, showImmediate):
		try:
			LOG('YampScreenSaver setList: slideIndex: %d showImmediate: %d' %(slideIndex, showImmediate), 'all')
#			LOG('YampScreenSaver setList: slideIndex: %d showImmediate: %d' %(slideIndex, showImmediate), 'err')  ###Test
#			LOG('YampScreenSaver setList: slideList: %s' %(slideList), 'err')  ###Test
		except:
			LOG('YampScreenSaver setList: EXCEPT', 'err')
		self.slideList = slideList
		self.slideIndex = slideIndex
		if self.slideIndex < 0: self.slideIndex = 0
		if self.slideIndex > len(self.slideList)-1: self.slideIndex = len(self.slideList)-1
		self.currPic = []
		self.currPicSav = []
		self.shownow = showImmediate
		self.displayIndex = slideIndex
		sc = AVSwitch().getFramebufferScale()
		#0=Width 1=Height 2=Aspect 3=use_cache 4=resize_type 5=Background(#AARRGGBB)
		self.picload.setPara((self.instance.size().width(), self.instance.size().height(), sc[0], sc[1], False, 1, "#00222222"))
		self.decodeFinished = True
		self.displayPtr = None 
		if len(self.slideList) > 0:
			self.start_decode()
		
	def start_decode(self):
#		LOG('YampScreenSaver: start_decode: decodeFinished: %d ' %(self.decodeFinished), 'err')
		self.decodeFinished = False
#		LOG('YampScreenSaver: start_decode: decodeFinished2: %d ' %(self.decodeFinished), 'err')
		self.currPic = []
		try:
			LOG('YampScreenSaver: start_decode: self.slideIndex: %d' %(self.slideIndex), 'all')
#			LOG('YampScreenSaver: start_decode: self.slideIndex: %d  slide: %s' %(self.slideIndex, self.slideList[self.slideIndex][-10:]), 'err')
		except:
			LOG('YampScreenSaver: start_decode: EXCEPT', 'err')
		try:
			self.picload.startDecode(self.slideList[self.slideIndex])
			LOG('YampScreenSaver: start_decode: END: self.slideList[self.slideIndex]: %s ' %(self.slideList[self.slideIndex]), 'all')
		except:
			LOG('YampScreenSaver: start decode END: EXCEPT: self.slideIndex: %d ' %(self.slideIndex), 'err')
#		LOG('YampScreenSaver: start_decode: End: decodeFinished: %d ' %(self.decodeFinished), 'err')
			
	def finish_decode(self, picInfo=""):
#		LOG('YampScreenSaver: finish_decode: Start decodeFinished: %d ' %(self.decodeFinished), 'err')
#		self.decodeFinished = True
		try:
			LOG('YampScreenSaver: finish_decode: self.slideIndex: %d' %(self.slideIndex), 'all')
#			LOG('\nYampScreenSaver: finish_decode: self.slideIndex: %d' %(self.slideIndex), 'err')
		except:
			LOG('YampScreenSaver finish_decode: EXCEPT', 'err')
		ptr = self.picload.getData()
#		LOG('YampScreenSaver finish_decode: len(currpic) before: %d' %(len(self.currPic)), 'err')
		if ptr != None:
			self.currPic.append(ptr)
			self.currPic.append(self.slideIndex)
			self.currPicSav = self.currPic
#			LOG('YampScreenSaver finish_decode: len(currpic) after: %d' %(len(self.currPic)), 'all')
		else:	
			self.currPic = self.currPicSav
			LOG('YampScreenSaver finish_decode: ptr=None, corrupt picture: %s' %(self.slideList[self.slideIndex]), 'err')
		try:
			self.showPic()	#show always if possible to switch to next pic even on false pic
		except:
			LOG('YampScreenSaver finish_decode: show pic: EXCEPT', 'err')
		self.decodeFinished = True
#		LOG('iPixMaps finish_decode: Ende decodeFinished: %d ' %(self.decodeFinished), 'err')

	def showPic(self):
		LOG('\nYampScreenSaver: showPic: shownow: %d len: %d currPic: %s  self.displayIndex: %d' %(self.shownow,len(self.currPic),self.slideList[self.displayIndex][-10:],self.displayIndex), 'all')
		if self.shownow and len(self.currPic):
			self.instance.setPixmap(self.currPic[0])
			self.displayPtr = self.currPic[0] 
			self.displayIndex = self.currPic[1]
#			try:
#				pass
#				LOG('YampScreenSaver: showPic: currPic: %s  self.displayIndex: %d' %(self.slideList[self.displayIndex],self.displayIndex), 'all')
#			except:
#				LOG('PixMaps showPic: currPic: EXCEPT' , 'err')
			self.shownow = False
			self.next()
			self.start_decode()

	def refreshPic(self, test = False):
#		LOG('PixMaps refreshPic: currPic: %s  self.displayIndex: %d' %(self.slideList[self.displayIndex][-10:],self.displayIndex), 'err')
#		LOG('PixMaps refreshPic: currPic: %s  self.displayIndex: %d' %(self.currPic[0],self.currPic[1]), 'err')
#		LOG('iPixMaps refreshPic: Start decodeFinished: %d ' %(self.decodeFinished), 'err')
#		retval = 0
		try:
#			while not self.decodeFinished:
#				LOG('PixMaps: refreshPic: wait for decodeFinished:','err')	
			if not self.decodeFinished: return 0 
		except: LOG('PixMaps: refreshPic: decodeFinished: EXCEPT','err')	
		if self.displayPtr != None:
			if not test: self.instance.setPixmap(self.displayPtr)
			return 1
		return 0	
#		self.shownow = False
#		self.showPic()
#		self.showPrevPic()
#		self.showPic()
			
	def next(self):
		self.slideIndex += 1
		if self.slideIndex > len(self.slideList) -1:
			self.slideIndex = 0

	def prev(self):
		self.slideIndex -= 1
		if self.slideIndex < 0:
			self.slideIndex = len(self.slideList) -1

	def showNextPic(self):
		self.shownow = True
		if len(self.currPic):
			self.showPic()
		else:
			self.start_decode()

	def showPrevPic(self):
		self.slideIndex = self.displayIndex
		self.prev()
		self.shownow = True
		self.start_decode()
		
	def getCurrentSlide(self):
		if len(self.slideList):
			return self.slideList[self.displayIndex]
		else:
			return ''

	def removeCurrentSlide(self):		
		if len(self.slideList) > 1:
			index=self.displayIndex	
			self.showNextPic()
			self.slideList.pop(index)

	def removeSlide(self, slidepathname):
		try:
			if len(self.slideList) > 1:
				self.showNextPic()
				self.slideList.remove(slidepathname)
		except:		
			LOG('PixMaps removeSlide: EXCEPT' , 'err')
			
	def getLen(self):
		return len(self.slideList)
			
	def __len__(self):
		return len(self.slideList)
