


# -*- coding: utf-8 -*-
#######################################################################
#
#    YAMP - Yet Another Music Player - Video Special
#    Version 3.2.1 2017-06-19
#    Coded by AlfredENeumann (c)2016-2017
#    Support: www.vuplus-support.org
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    Thanks to the authors of Merlin Music Player and OpenPli Media
#    Player for ideas and code snippets
#
#######################################################################

from YampGlobals import *

from Components.Label import Label
from Components.Pixmap import Pixmap, MultiPixmap
from Components.Sources.Boolean import Boolean

#from timer import TimerEntry
from YampPixmaps import YampCoverArtPixmap
from YampBoxDisplay import *


class YampVideoTitleV32(Screen):
	def __init__(self, session, parent, autoStartTime=0):
		Screen.__init__(self, session, parent = parent)
		try:
			with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampVideoTitle.xml"), 'r') as f:
				self.skin = f.read()
				self.skinOpenError = False
		except:
			LOG('YampVideoTitle: open xml: EXCEPT', 'err')
			self.skinOpenError = True
			self.close()
			return
			
		self.parent = parent
		self.autoStartTime = autoStartTime
		
		self.isDreamOS = os.path.exists("/var/lib/dpkg/status")
		self.updateTimer = eTimer()
		if self.isDreamOS: self.updateTimer_conn=self.updateTimer.timeout.connect(self.updateInfoCyclic)
		else: self.updateTimer.callback.append(self.updateInfoCyclic)
		self.startupTimer = eTimer()
		if self.isDreamOS: self.startupTimer_conn=self.startupTimer.timeout.connect(self.startupTimerTimeout)
		else: self.startupTimer.callback.append(self.startupTimerTimeout)

		self["songtitle"] = Label("")
		self["artist"] = Label("")
		self["album"] = Label ("")
		self["year"] = Label("")
		self["genre"] = Label("")
		self["line1"] = Label(".")
		self["line2"] = Label(".")
		self["coverArt"] = YampCoverArtPixmap()
		self["nextsong"] = Label (_("Next Song"))
		self["nextsongtitle"] = Label ("")
		self["bitrate"] = Label("")
		self["songTextsBackground"] = Boolean(True)

		self["lyricsLine"] = Label("")
		self["lyricsLineBackground"] = Boolean(False)
		
		self["actions"] = ActionMap(["YampActions", "YampOtherActions"], 
		{
			"exit": self.keyClose,
			"red": self.keyClose,
			"green": self.keyCloseOk,
			"ok": self.keyCloseOk,
			"up": self.keyCloseUp,
			"down": self.keyCloseDown,
			"moveTop": self.keyCloseMoveTop,
			"moveEnd": self.keyCloseMoveEnd,
			"menu": self.keyCloseMenu,				
			"info": self.keyCloseInfo,
			"prevTitle": self.keyPrevious,
			"nextTitle": self.keyNext,
			"Pvr" : self.keyPvr,
			"Text" : self.keyText,
			"play": self.keyPlay,
			"pause": self.keyPause,
			"stop": self.keyStop,
		}, -2)
		self["seekactions"] = ActionMap(["YampSeekSubActions"], 
		{
			"key1": self.key1,
			"key3": self.key3,
			"key4": self.key4,
			"key6": self.key6,
			"key7": self.key7,
			"key9": self.key9,
		}, -2)
		
		self.LcdText=''

		try:
			LOG('YampVideoTitle: init: autoStartTime: %d' %(autoStartTime), 'spe')
			LOG('YampVideoTitle: init: autoStartTime: %d' %(self.autoStartTime), 'spe')
		except:
			LOG('YampVideoTitle: init: parameters: EXCEPT', 'err')

		LOG('YampVideoTitle: init: IsVideo: %d' %(self.parent.currentIsVideo), 'spe')
		self.actTitle = self.actArtist = self.actAlbum = ''
		self.onLayoutFinish.append(self.layoutFinished)
		self.onClose.append(self.cleanup)
		self.parent.videoScreenActive = True

	def layoutFinished(self):
		LOG('YampVideoTitle: layoutFinished: autoStartTime: %d  isVideo: %d' %(self.autoStartTime, self.parent.currentIsVideo), 'spe')
		self.updateSongInfo()
		if self.parent.lyricsLineShow or self.parent.lyricsLineShowBig:
			self["lyricsLine"].show()
			self["lyricsLineBackground"].setBoolean(True)
		self.updateTimer.start(300,False)
		if self.autoStartTime > 0: self.startupTimer.start(self.autoStartTime * 1000,True)
		
	def startupTimerTimeout(self):
		LOG('YampVideoTitle: startupTimerTimeout: close: IsVideo: %d' %(self.parent.currentIsVideo), 'spe')
		self.parent.videoTitleScreenClose = True
		if self.parent.currentIsVideo:
			if self.parent.lyricsLineShow or self.parent.lyricsLineShowBig:
				self.parent.videoLyricsScreenOpen = True
		else:
#			self.startupTimer.stop()
			self.updateTimer.stop()
			
				
	def updateSongInfo(self):
#		LOG('YampVideoTitle: updateSongInfo: IsVideo: %d' %(self.parent.currentIsVideo), 'spe')
		if not self.parent.currentIsVideo:
			self.updateTimer.stop()
			self.parent.videoTitleScreenClose = True
			return
		try:
			if self.parent.lyricsLineShow or self.parent.lyricsLineShowBig:
				self["lyricsLine"].setText(self.parent.displayLyricsText)
		except:
			LOG('YampVideoTitle: updateSongInfo: lyricsLine SetText: EXCEPT', 'err')
		try:
			coverArtFile, PixNumCover = self.parent.getCoverArtFile()
#			LOG('YampVideoTitle: updateSongInfo: PixNumCover: %d  coverArtFile: %s actCoverArtFile: %s' %(PixNumCover, coverArtFile, self.actCoverArtFile), 'spe')
			if PixNumCover > 0: 
##				LOG('YampVideoTitle: updateSongInfo: showCoverArtFile: %s' %(coverArtFile), 'spe')
#				LOG('YampVideoTitle: updateSongInfo: coverart.getFileName: %s  coverArtFile: %s' %(self["coverArt"].getFileName(),coverArtFile), 'err')
				self["coverArt"].showCoverArt(coverArtFile)
		except:
			LOG('YampVideoTitle: updateSongInfo: cover: EXCEPT', 'err')
			
		replaceText = self.parent.infoBarNaReplace
		newTitle = self.parent.currTitle.replace('n/a',replaceText)
		newArtist = self.parent.currArtist.replace('n/a',replaceText)
		newAlbum = self.parent.currAlbum.replace('n/a',replaceText)

		if self.actTitle != newTitle or self.actArtist != newArtist or self.actAlbum != newAlbum:
			if self.autoStartTime > 0:
				if self.startupTimer.isActive(): self.startupTimer.stop()
				self.startupTimer.start(self.autoStartTime * 1000,True)

			self.actTitle = newTitle
			self.actArtist = newArtist
			self.actAlbum = newAlbum
			self["songtitle"].setText(newTitle)
			self["artist"].setText(newArtist)
			self["album"].setText(newAlbum)
			self["year"].setText(self.parent.currYear.replace('n/a',replaceText))
			self["genre"].setText(self.parent.currGenre.replace('n/a',replaceText))
#			self["nextsongtitle"].setText(self.parent.nextsongtitle.replace('n/a',replaceText) + "  -  " + self.parent.nextsongartist.replace('n/a',replaceText) + "  -  " + self.parent.nextsongalbum.replace('n/a',replaceText))
			self["nextsongtitle"].setText(self.parent.nextSongDisplay.replace('n/a',replaceText))
			self["bitrate"].setText(self.parent.currBitRate.replace('n/a',replaceText))
#		self["line1"] = Label (".")
#		self["line2"] = Label (".")
		
	def updateInfoCyclic(self):
		self.updateSongInfo()
		self.setLcdText()

	def key1(self):
		self.parent.seekOwn(1)

	def key3(self):
		self.parent.seekOwn(3)

	def key4(self):
		self.parent.seekOwn(4)

	def key6(self):
		self.parent.seekOwn(6)

	def key7(self):
		self.parent.seekOwn(7)

	def key9(self):
		self.parent.seekOwn(9)

	def keyPrevious(self):
		self.parent.seekOwn(11)

	def keyNext(self):
		self.parent.seekOwn(12)

	def keyPvr(self):
		self.parent.videoTitleScreenClose = True
		if self.parent.lyricsLineShow  or self.parent.lyricsLineShowBig:
			self.parent.videoLyricsScreenOpen = True
#		self.close()

	def keyText(self):
		try: 
			self.parent.lyricsLineShow = not self.parent.lyricsLineShow	#!!!noch Restzeit autostart
			self.parent.lyricsLineShowBig = False
			self.parent.displayHideLyricsLine()
#			LOG('YampVideoTitle: keyText: startupTimer Active: %d' %(self.startupTimer.isActive()), 'err')
			self["lyricsLineBackground"].setBoolean(self.parent.lyricsLineShow)
			if self.parent.lyricsLineShow or self.parent.lyricsLineShowBig: self["lyricsLine"].show()
			else:	self["lyricsLine"].hide()
		except: LOG('YampVideoTitle: keyText: EXCEPT', 'err') 
		
	def keyClose(self):
		try:
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyClose: EXCEPT', 'err')

	def keyCloseOk(self):
		try:
			self.parent.ok()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseOk: EXCEPT', 'err')

	def keyCloseUp(self):
		try:
#			LOG('YampVideoTitle: keyCloseUp:', 'spe')
			self.parent.moveup()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseUp: EXCEPT', 'err')
		
	def keyCloseDown(self):
		try:
			self.parent.movedown()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseDown: EXCEPT', 'err')
			
	def keyCloseMoveTop(self):
		try:
			self.parent.skipToListbegin()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseMoveTop: EXCEPT', 'err')
			
	def keyCloseMoveEnd(self):
		try:
			self.parent.skipToListend()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseMoveEnd: EXCEPT', 'err')

	def keyCloseMenu(self):
		try:
#			LOG('YampVideoTitle: keyCloseMenu: start', 'spe')
			self.parent.menuRequest = True
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseMenu: EXCEPT', 'err')

	def keyCloseInfo(self):
		try:
#			LOG('YampVideoTitle: keyCloseInfo: start', 'spe')
			self.hide()
			self.parent.showLyrics()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyCloseInfo: EXCEPT', 'err')

	def keyPlay(self):
		try:
			self.parent.play()
		except:	LOG('YampVideoTitle: keyPlay: EXCEPT', 'err')

	def keyPause(self):
		try:
			self.parent.pause()
		except:	LOG('YampVideoTitle: keyPause: EXCEPT', 'err')

	def keyStop(self):
		try:
			self.parent.stopEntry()
			self.parent.videoTitleScreenClose = True
		except:	LOG('YampVideoTitle: keyStop: EXCEPT', 'err')
		
	def cleanup(self):
		LOG('YampVideoTitle: cleanup', 'spe')
		self.updateTimer.stop()
		self.startupTimer.stop()
		del self.updateTimer
		del self.startupTimer
		self.parent.videoScreenActive = False
		self.parent.videoTitleScreenWaitForClosed = False
		
	def setLcdText(self):
		if config.plugins.yampmusicplayer.yampLCD.value == 'off': return
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			self.updateLCDText(self.actTitle + ' - ' + self.actArtist + ' - ' + self.actAlbum,1)
		else:	
			self.updateLCDText(self.actTitle, 1)
			self.updateLCDText(self.actArtist, 2)
			self.updateLCDText(self.actAlbum, 3)

	def updateLCDText(self, text, line):
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			try:
				self.LcdText = text
#				LOG('YampVideoTitle: updateLCDText: RollingLCD: LcdText: %s' %(self.LcdText), 'err') 
			except:	pass
		else:	
			self.summaries.setCover()
#			LOG('YampVideoTitle: updateLCDText: normalLCD: LcdText: %s' %(text), 'err') 
			self.summaries.setText(text,line)

	def getLcdText(self):	#for LCD Running Text
		if self.skinOpenError == True: return('')
#		LOG('YampVideoTitle: getLcdText: LcdText: %s' %(self.LcdText), 'all')
		return(self.LcdText)
		
	def createSummary(self):
		lcdMode = config.plugins.yampmusicplayer.yampLCD.value
		if lcdMode == 'off': return
		if lcdMode == 'rolling':
			return YampLCDRunningScreenV31
		else:
			return YampLCDScreenV31

	def lockShow(self):
		pass

	def unlockShow(self):
		pass

	
class YampVideoLyricsV32(Screen):
	def __init__(self, session, parent):
		Screen.__init__(self, session, parent = parent)
		try:
			with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampVideoLyrics.xml"), 'r') as f:
				self.skin = f.read()
				self.skinOpenError = False
		except:
			LOG('YampVideoLyrics: open xml: EXCEPT', 'err')
			self.skinOpenError = True
			self.close()
			return

		self.parent = parent
		
		self.isDreamOS = os.path.exists("/var/lib/dpkg/status")
		self.updateTimer = eTimer()
		if self.isDreamOS: self.updateTimer_conn=self.updateTimer.timeout.connect(self.updateInfoCyclic)
		else: self.updateTimer.callback.append(self.updateInfoCyclic)

		self["lyricsLine"] = Label("")
		self["lyricsLineBackground"] = Boolean(True)

		self["actions"] = ActionMap(["YampActions", "YampOtherActions"], 
		{
			"exit": self.keyClose,
			"red": self.keyClose,
			"green": self.keyCloseOk,
			"ok": self.keyCloseOk,
			"up": self.keyCloseUp,
			"down": self.keyCloseDown,
			"moveTop": self.keyCloseMoveTop,
			"moveEnd": self.keyCloseMoveEnd,
			"menu": self.keyCloseMenu,				
			"info": self.keyCloseInfo,
			"prevTitle": self.keyPrevious,
			"nextTitle": self.keyNext,
			"Pvr" : self.keyPvr,
			"Text" : self.keyText,
			"play": self.keyPlay,
			"pause": self.keyPause,
			"stop": self.keyStop,
		}, -2)
		self["seekactions"] = ActionMap(["YampSeekSubActions"], 
		{
			"key1": self.key1,
			"key3": self.key3,
			"key4": self.key4,
			"key6": self.key6,
			"key7": self.key7,
			"key9": self.key9,
		}, -2)
		
		self.LcdText=''

#		LOG('YampVideoLyrics: init: IsVideo: %d' %(self.parent.currentIsVideo), 'err')
		self.actTitle = self.actArtist = self.actAlbum = ''
		self.onLayoutFinish.append(self.layoutFinished)
		self.onClose.append(self.cleanup)
		self.parent.videoScreenActive = True

	def layoutFinished(self):
		self.updateSongInfo()
		self.updateTimer.start(300,False)
		
	def updateSongInfo(self):
#		LOG('YampVideoLyrics: updateSongInfo: IsVideo: %d' %(self.parent.currentIsVideo), 'spe')
		if not self.parent.currentIsVideo:
			LOG('YampVideoLyrics: updateSongInfo: close: IsVideo: %d' %(self.parent.currentIsVideo), 'spe')
			self.updateTimer.stop()
			self.parent.videoLyricsScreenClose = True
			return
		try:
			if self.parent.lyricsLineShow or self.parent.lyricsLineShowBig:
				self["lyricsLine"].setText(self.parent.displayLyricsText)
		except:
			LOG('YampVideoLyrics: updateSongInfo: lyricsLine SetText: EXCEPT', 'err')

		replaceText = self.parent.infoBarNaReplace
		newTitle = self.parent.currTitle.replace('n/a',replaceText)
		newArtist = self.parent.currArtist.replace('n/a',replaceText)
		newAlbum = self.parent.currAlbum.replace('n/a',replaceText)

		if self.actTitle != newTitle or self.actArtist != newArtist or self.actAlbum != newAlbum:
			self.actTitle = newTitle
			self.actArtist = newArtist
			self.actAlbum = newAlbum
		
	def updateInfoCyclic(self):
		self.updateSongInfo()
		self.setLcdText()

	def key1(self):
		self.parent.seekOwn(1)

	def key3(self):
		self.parent.seekOwn(3)

	def key4(self):
		self.parent.seekOwn(4)

	def key6(self):
		self.parent.seekOwn(6)

	def key7(self):
		self.parent.seekOwn(7)

	def key9(self):
		self.parent.seekOwn(9)

	def keyPrevious(self):
		self.parent.seekOwn(11)

	def keyNext(self):
		self.parent.seekOwn(12)

	def keyPvr(self):
		try: 
#			self.parent.openVideoTitleLyricsScreen()
			self.parent.videoLyricsScreenClose = True
			self.parent.videoTitleScreenOpen = True
		except:
			LOG('YampVideoLyrics: keyPvr: EXCEPT', 'err')

	def keyText(self):
		try: 
			self.parent.lyricsLineShow = False
			self.parent.lyricsLineShowBig = False
			self.parent.displayHideLyricsLine()
			self.parent.videoLyricsScreenClose = True
		except: LOG('YampVideoLyrics: keyText: EXCEPT', 'err') 
		
	def keyClose(self):
		try:
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyClose: EXCEPT', 'err')

	def keyCloseOk(self):
		try:
			self.parent.ok()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseOk: EXCEPT', 'err')

	def keyCloseUp(self):
		try:
#			LOG('YampVideoLyrics: keyCloseUp:', 'spe')
			self.parent.moveup()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseUp: EXCEPT', 'err')
		
	def keyCloseDown(self):
		try:
			self.parent.movedown()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseDown: EXCEPT', 'err')
			
	def keyCloseMoveTop(self):
		try:
			self.parent.skipToListbegin()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseMoveTop: EXCEPT', 'err')
			
	def keyCloseMoveEnd(self):
		try:
			self.parent.skipToListend()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseMoveEnd: EXCEPT', 'err')

	def keyCloseMenu(self):
		try:
			self.parent.menuRequest = True
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseMenu: EXCEPT', 'err')

	def keyCloseInfo(self):
		try:
#			LOG('YampVideoLyrics: keyCloseInfo: start', 'spe')
			self.hide()
			self.parent.showLyrics()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyCloseInfo: EXCEPT', 'err')

	def keyPlay(self):
		try:
			self.parent.play()
		except:	LOG('YampVideoLyrics: keyPlay: EXCEPT', 'err')

	def keyPause(self):
		try:
			self.parent.pause()
		except:	LOG('YampVideoLyrics: keyPause: EXCEPT', 'err')

	def keyStop(self):
		try:
			self.parent.stopEntry()
			self.parent.videoLyricsScreenClose = True
		except:	LOG('YampVideoLyrics: keyStop: EXCEPT', 'err')
		
	def cleanup(self):
		LOG('YampVideoLyrics: cleanup', 'spe')
		self.updateTimer.stop()
		del self.updateTimer
		self.parent.videoScreenActive = False
		self.parent.videoLyricsScreenWaitForClosed = False

	def setLcdText(self):
		if config.plugins.yampmusicplayer.yampLCD.value == 'off': return
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			self.updateLCDText(self.actTitle + ' - ' + self.actArtist + ' - ' + self.actAlbum,1)
		else:	
			self.updateLCDText(self.actTitle, 1)
			self.updateLCDText(self.actArtist, 2)
			self.updateLCDText(self.actAlbum, 3)

	def updateLCDText(self, text, line):
		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			try:
				self.LcdText = text
#				LOG('YampVideoLyrics: updateLCDText: RollingLCD: LcdText: %s' %(self.LcdText), 'err') 
			except:	pass
		else:	
			self.summaries.setCover()
#			LOG('YampVideoLyrics: updateLCDText: normalLCD: LcdText: %s' %(text), 'err') 
			self.summaries.setText(text,line)

	def getLcdText(self):	#for LCD Running Text
		if self.skinOpenError == True: return('')
#		LOG('YampVideoLyrics: getLcdText: LcdText: %s' %(self.LcdText), 'all')
		return(self.LcdText)
		
	def createSummary(self):
		lcdMode = config.plugins.yampmusicplayer.yampLCD.value
		if lcdMode == 'off': return
		if lcdMode == 'rolling':
			return YampLCDRunningScreenV31
		else:
			return YampLCDScreenV31

	def lockShow(self):
		pass

	def unlockShow(self):
		pass

