﻿# -*- coding: utf-8 -*-
#######################################################################
#
#    YAMP - Yet Another Music Player
#    Version 3.2.2.1 2022-03-04
#    Coded by JohnHenry (c)2013 (up to V2.6.5)
#    Extended by AlfredENeumann (c)2016-2022
#    Support: www.vuplus-support.org, board.newnigma2.to
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    Thanks to the authors of Merlin Music Player and OpenPli Media
#    Player for ideas and code snippets
#
#######################################################################
#spe:		Coversuche
#spe2:	Lyricssuche Genius

from YampGlobals import *

from Components.GUIComponent import GUIComponent
from Components.MultiContent import MultiContentEntryText

#from Components import Renderer
#from Components import ConditionalWidget

from enigma import eListboxPythonMultiContent, eListbox, gFont, RT_VALIGN_CENTER, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER
from enigma import eServiceCenter, eServiceReference, getDesktop, iServiceInformation, iPlayableService
from enigma import ePythonMessagePump

from Components.FileList import FileList
from Components.Label import Label
from Components.Pixmap import Pixmap, MultiPixmap
from Components.ScrollLabel import ScrollLabel
from Components.ServiceEventTracker import ServiceEventTracker, InfoBarBase

from datetime import timedelta as datetime_timedelta, datetime as datetime_datetime

import mutagen
from mutagen.apev2 import APEv2
from mutagen.easyid3 import EasyID3
from mutagen.easymp4 import EasyMP4
from mutagen.flac import Picture, FLAC, StreamInfo
from mutagen.id3 import ID3
from mutagen.mp3 import MP3
from mutagen.oggvorbis import OggVorbis

#import os
import shutil
import random

from Screens.ChoiceBox import ChoiceBox
from Screens.InfoBarGenerics import InfoBarSeek, InfoBarNotifications
from Screens.MessageBox import MessageBox
from Screens.HelpMenu import HelpableScreen
from Screens.VirtualKeyBoard import VirtualKeyBoard

from ServiceReference import ServiceReference

import sqlite3

from threading import Thread, Lock

from time import time, strftime
from timer import TimerEntry

from Tools.BoundFunction import boundFunction
#from Tools.Directories import fileExists, resolveFilename, SCOPE_CURRENT_SKIN, SCOPE_CONFIG

##from twisted.web import client
from twisted.web.client import downloadPage, getPage

from urllib import quote, quote_plus, unquote

from xml.etree.cElementTree import fromstring as xml_fromstring
import xml.etree.ElementTree as xmlET

from Components.Sources.StaticText import StaticText   #!!!!!neu
from Tools.Directories import *

#from Components.Sources import Boolean
from Components.Sources.Boolean import Boolean
from skin import parseColor

# our own modules
from __init__ import _
from Titlecase import titlecase
from YampConfig import YampConfigScreenV31
from YampDatabaseList import YampDatabaseList, DblistStackEntry, DblistEntryComponent
from YampFileList import YampFileList
from YampLyrics import YampLyricsScreenV32
from YampPixmaps import YampCoverArtPixmap, YampScreenSaver
from YampPlayList import YampPlayList
from YampPlaylistParsers import YampParserE2pls, YampParserM3u, YampParserPls
from YampBoxDisplay import *
from YampVideoSpecial import YampVideoTitleV32, YampVideoLyricsV32

from Components.VideoWindow import VideoWindow

FANARTDLTIME = DOWNLOADBASETIME * FANARTDLTIMEMUL
FANARTDLTIMEPERSONAL = DOWNLOADBASETIME * FANARTDLTIMEPERSONALMUL

INFOBARTEXTLIST = ('n/a','   ',' - ',' . ')

#
# Configuration Entries
#
config.plugins.yampmusicplayer = ConfigSubsection()

#General Adjustments
if getDesktop(0).size().width() > 1280:	
	config.plugins.yampmusicplayer.yampSkin = ConfigSelection(default = "fhd", choices = [
			("default265", _("Standard skin V2.6.5")),
			("transparent", _("Transparent skin V2.6.5")),
			("default", _("Standard skin")),
			("custom", _("Custom skin")),
			("fhd", _("Full HD standard skin")),
			("fhdCustom", _("Custom Full HD skin")),
			])
else:
	config.plugins.yampmusicplayer.yampSkin = ConfigSelection(default = "default", choices = [
			("default265", _("Standard skin V2.6.5")),
			("transparent", _("Transparent skin V2.6.5")),
			("default", _("Standard skin")),
			("custom", _("Custom skin")),
			])
config.plugins.yampmusicplayer.databasePath = ConfigDirectory(default = "/media/hdd/")
config.plugins.yampmusicplayer.filebrowserPath = ConfigDirectory(default = "/media/hdd/")
config.plugins.yampmusicplayer.saveLastFilebrowserPath = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.playlistPath = ConfigDirectory(default = "/media/hdd/")
config.plugins.yampmusicplayer.useSingleLyricsPath = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.lyricsPath = ConfigDirectory(default = "/media/hdd/")
config.plugins.yampmusicplayer.screenSaverSlidePath = ConfigDirectory(default = "/media/hdd/")
config.plugins.yampmusicplayer.screenSaverArtworkPath = ConfigDirectory(default = "/media/hdd/Yamp/ArtistPics/")

#Databaselist / Playlist
config.plugins.yampmusicplayer.fileListTitleLayout = ConfigSelection(default = "titleOnly", choices = [
		("titleOnly", _("Title Only")),
		("pathOnly", _("Path Only")),
		("titleAndPath", _("Title and Path")),
		])
config.plugins.yampmusicplayer.newLayoutDbList = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.titleOnlyOnceInSelection = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.capitalizeTitleAndArtist = ConfigYesNo(default = True)
			
config.plugins.yampmusicplayer.playlistLayout = ConfigSelection(default = "titart", choices = [
		("fn", _("Filename without Extension")),
		("fnext", _("Filename with Extension")),
		("tit", _("Title")),
		("titart", _("Title and Artist")),
		("titart2", _("Artist and Title")),
		])
config.plugins.yampmusicplayer.displayUnknown = ConfigSelection(default = "0", choices = [
		("0", _("no change: <n/a>")),
		("1", _("3 x blank: <   >")),
		("2", _("blank - minus - blank: < - >")),
		("3", _("blank - dot - blank: < . >")),
		])
config.plugins.yampmusicplayer.savePlaylistOnExit = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.repeatPlaylistAtEnd = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.wrapMovePlaylist = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.startImmediate = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.gapCorrection = ConfigInteger(0,limits = (0, 50))
config.plugins.yampmusicplayer.displayNext = ConfigSelection(default = "0", choices = [
		("0", _("Title - Artist - Album")),
		("1", _("Title - Album - Artist")),
		("2", _("Artist - Title - Album")),
		("3", _("Artist - Album - Title")),
		("4", _("Title - Artist")),
		("5", _("Title - Album")),
		("6", _("Artist - Title")),
		("7", _("Artist - Album")),
		("8", _("Title")),
		])

#Cover Search 
config.plugins.yampmusicplayer.prioCover1 = ConfigSelection(default = "coverID3", choices = [
		("0", _("off")),
		("coverID3", _("search in ID3-tags (mp3, flac)")),
		("coverDirTitle", _("search for <title>.<ext> in title-directory")),
		("coverAlbum", _("search for albumcover.<ext> in title-directory")),
		("coverDirAny", _("search for any picture in title-directory")),
		("coverGoogle", _("search at google online")),
		])
config.plugins.yampmusicplayer.prioCover2 = ConfigSelection(default = "coverDirTitle", choices = [
		("0", _("off")),
		("coverID3", _("search in ID3-tags (mp3, flac)")),
		("coverDirTitle", _("search for <title>.<ext> in title-directory")),
		("coverAlbum", _("search for \"albumcover.<ext>\" in title-directory")),
		("coverDirAny", _("search for any picture in title-directory")),
		("coverGoogle", _("search at google online")),
		])
config.plugins.yampmusicplayer.prioCover3 = ConfigSelection(default = "coverAlbum", choices = [
		("0", _("off")),
		("coverID3", _("search in ID3-tags (mp3, flac)")),
		("coverDirTitle", _("search for <title>.<ext> in title-directory")),
		("coverAlbum", _("search for \"albumcover.<ext>\" in title-directory")),
		("coverDirAny", _("search for any picture in title-directory")),
		("coverGoogle", _("search at google online")),
		])
config.plugins.yampmusicplayer.prioCover4 = ConfigSelection(default = "0", choices = [
		("0", _("off")),
		("coverID3", _("search in ID3-tags (mp3, flac)")),
		("coverDirTitle", _("search for <title>.<ext> in title-directory")),
		("coverAlbum", _("search for \"albumcover.<ext>\" in title-directory")),
		("coverDirAny", _("search for any picture in title-directory")),
		("coverGoogle", _("search at google online")),
		])
config.plugins.yampmusicplayer.prioCover5 = ConfigSelection(default = "coverGoogle", choices = [
		("0", _("off")),
		("coverID3", _("search in ID3-tags (mp3, flac)")),
		("coverDirTitle", _("search for <title>.<ext> in title-directory")),
		("coverAlbum", _("search for \"albumcover.<ext>\" in title-directory")),
		("coverDirAny", _("search for any picture in title-directory")),
		("coverGoogle", _("search at google online")),
		])
config.plugins.yampmusicplayer.searchGoogleCoverVideo = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.saveCover = ConfigSelection(default = "0", choices = [
		("0", _("off")),
		("titleNoPicture", _("save <title>.<ext> if no picture in song dir")),
		("titleNoTitle", _("save <title>.<ext> if not existing in song dir")),
		("titleAlways", _("save <title>.<ext>, overwrite existing")),
		("albumNoAlbum", _("save albumcover.<ext> if not existing")),
		("albumNoAlways", _("save albumcover.<ext>, overwrite existing")),
		])


#Lyrics Search 
config.plugins.yampmusicplayer.prioLyrics1 = ConfigSelection(default = "lyricsID3", choices = [
		("0", _("off")),
		("lyricsID3", _("search in ID3-tags (mp3, flac)")),
		("lyricsFile", _("search in lyrics-directory")),
		("lyricsChart", _("search at chartlyrics.com online")),
		("lyricsGenius", _("search at genius.com online")),
		])
config.plugins.yampmusicplayer.prioLyrics2 = ConfigSelection(default = "lyricsFile", choices = [
		("0", _("off")),
		("lyricsID3", _("search in ID3-tags (mp3, flac)")),
		("lyricsFile", _("search in lyrics-directory")),
		("lyricsChart", _("search at chartlyrics.com online")),
		("lyricsGenius", _("search at genius.com online")),
		])
config.plugins.yampmusicplayer.prioLyrics3 = ConfigSelection(default = "lyricsChart", choices = [
		("0", _("off")),
		("lyricsID3", _("search in ID3-tags (mp3, flac)")),
		("lyricsFile", _("search in lyrics-directory")),
		("lyricsChart", _("search at chartlyrics.com online")),
		("lyricsGenius", _("search at genius.com online")),
		])
config.plugins.yampmusicplayer.prioLyrics4 = ConfigSelection(default = "lyricsGenius", choices = [
		("0", _("off")),
		("lyricsID3", _("search in ID3-tags (mp3, flac)")),
		("lyricsFile", _("search in lyrics-directory")),
		("lyricsChart", _("search at chartlyrics.com online")),
		("lyricsGenius", _("search at genius.com online")),
		])
config.plugins.yampmusicplayer.showLyricsOnVideo = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.autoSaveLyrics = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.karaokeMaxTime = ConfigInteger(10,limits = (0, 600))

#Fanart Search 
config.plugins.yampmusicplayer.searchFanart = ConfigSelection(default = "newArtists", choices = [
		("0", _("off")),
		("newArtists", _("search only for new artists")),
		("always", _("search always")),
		])
config.plugins.yampmusicplayer.fanarttvPersonal = ConfigText (default = '1234567890abcdef7890123456789012', fixed_size = True, visible_width = 33)			 #personal api key
config.plugins.yampmusicplayer.fanarttvPersonal.setUseableChars('0123456789abcdef')
config.plugins.yampmusicplayer.searchMbidMode = ConfigSelection(default = "standard", choices = [
		("standard", _("standard like on muscicbrainz website")),
		("special", _("special, use if standard is not working")),
		])
config.plugins.yampmusicplayer.searchMbidMinScore = ConfigInteger(50,limits = (1, 100))
		
#Screensaver			
config.plugins.yampmusicplayer.screenSaverMode = ConfigSelection(default = "artwork", choices = [
		("blank", _("Blank screen")),
		("standard", _("Standard Screensaver")),
		("custom", _("Custom Screensaver")),
		("artwork", _("Artwork as Screensaver")),
		])
config.plugins.yampmusicplayer.screenSaverBg = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.resetScreenSaverBg = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.screenSaverSubDirs = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.screenSaverWait = ConfigInteger(10,limits = (0, 600))
config.plugins.yampmusicplayer.screenSaverNextSlide = ConfigInteger(10,limits = (5, 60))
config.plugins.yampmusicplayer.screenSaverSort = ConfigSelection(default = "shuffle", choices = [
		("nosort", _("not sorting")),
		("sortA", _("sorting A->Z")),
		("sortZ", _("sorting Z->A")),
		("shuffle", _("shuffling")),
		])
config.plugins.yampmusicplayer.screenSaverLeft = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.manKeyScreensaver = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.maxTimeSSoffMan = ConfigInteger(600,limits = (0, 1200))
config.plugins.yampmusicplayer.showTextBg = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.showClockSS = ConfigYesNo(default = False)

#Operation
config.plugins.yampmusicplayer.yampRemote = ConfigSelection(default = "special", choices = [
		("default", _("as in V2.6.5")),
		("special", _("Standard")),
		])
config.plugins.yampmusicplayer.newExitKey = ConfigYesNo(default = True)
config.plugins.yampmusicplayer.commonPlayPause = ConfigYesNo(default = False)
config.plugins.yampmusicplayer.yampConfirmExit = ConfigYesNo (default = True)

#Lyrics
config.plugins.yampmusicplayer.lyricsMinLinesScroll = ConfigInteger(20,limits = (0, 100))
config.plugins.yampmusicplayer.lyricsScrollLine= ConfigInteger(7,limits = (1, 100))
config.plugins.yampmusicplayer.lyricsOffsetTime = ConfigInteger(1000,limits = (0, 5000))
config.plugins.yampmusicplayer.lyricsPlayOffsetTime = ConfigInteger(1000,limits = (-10000, 10000))

#Miscellaneous		
try:
	if os.path.exists('/usr/lib/enigma2/python/Components/Renderer/RunningText.py') or os.path.exists('/usr/lib/enigma2/python/Components/Renderer/RunningText.pyo'):
		config.plugins.yampmusicplayer.yampLCD = ConfigSelection(default = "threelines", choices = [
				("off", _("off")),
				("oneline", _("1 line display")),
				("threelines", _("3 lines display")),
				("rolling", _("Rolling Text (graphical)")),
				("cover", _("display cover (graphical)")),
				("cover1", _("display cover (graphical) + 1 line text")),
				("cover3", _("display cover (graphical) + 3 lines text")),
				])
	else:
		config.plugins.yampmusicplayer.yampLCD = ConfigSelection(default = "threelines", choices = [
				("off", _("off")),
				("oneline", _("1 line display")),
				("threelines", _("3 lines display")),
				("cover", _("display cover (graphical)")),
				("cover1", _("display cover (graphical) + 1 line text")),
				("cover3", _("display cover (graphical) + 3 lines text")),
				])
	config.plugins.yampmusicplayer.lcdCoverSize = ConfigInteger(60,limits = (10, 2000))
	config.plugins.yampmusicplayer.lcdCoverColor = ConfigSelection(default = "bw", choices = [
			("bw", _("black & white")),
			("bwt", _("black & white transparent")),
			("color", _("color")),
			("colort", _("color transparent")),
			])
except:
	pass
config.plugins.yampmusicplayer.lcdRollingFontSize = ConfigInteger(38,limits = (1, 99))
config.plugins.yampmusicplayer.displayButtonHelp = ConfigYesNo(default = False)

config.plugins.yampmusicplayer.yampInExtendedPluginlist = ConfigYesNo (default = True)
config.plugins.yampmusicplayer.yampInMainMenu = ConfigYesNo (default = False)
config.plugins.yampmusicplayer.debugPath = ConfigDirectory (default = "/media/hdd/")
config.plugins.yampmusicplayer.yampDebugMode = ConfigSelection(default = "error", choices = [
		("off", _("off")),
		("error", _("errors only")),
		("all", _("all")),
		("special", _("special")),
		("special2", _("special2")),
		])
config.plugins.yampmusicplayer.yampDebugMemory = ConfigYesNo (default = False)

config.plugins.yampmusicplayer.yampCoverLyricsTestMode = ConfigInteger(1,limits = (1, 3))

config.plugins.yampmusicplayer.insertBlankVideo = ConfigYesNo (default = False)
config.plugins.yampmusicplayer.blankVideoDelay = ConfigInteger(500,limits = (0, 6000))
config.plugins.yampmusicplayer.videoAutoFull = ConfigSelection(default = "immediate", choices = [
		("immediate", _("immediate")),
		("screensaver", _("screensaver delay")),
		("manual", _("only manual (key ok)")),
		])
config.plugins.yampmusicplayer.showTimeVideoTitle = ConfigInteger(10,limits = (0, 10000))


#	not configurable by user	
config.plugins.yampmusicplayer.startWithDatabase = ConfigYesNo(default = False) # not configurable by user
config.plugins.yampmusicplayer.shuffle = ConfigYesNo (default = False) 			  # not configurable by user
config.plugins.yampmusicplayer.lastPlaylistIndex = ConfigInteger(-1) 			    # not configurable by user
config.plugins.yampmusicplayer.lastSlide = ConfigText (default = "")			    # not configurable by user
config.plugins.yampmusicplayer.debugFile = ConfigText (default = "")				  # not configurable by user
config.plugins.yampmusicplayer.separator = ConfigSelection(default = "1", choices = [('1', '')])

config.plugins.yampmusicplayer.displayVideoPreview = ConfigYesNo(default = False) # not configurable by user

global yamp_session
yamp_session=None

#
# The YAMP main screen class - needs a HD skin (minimum 1280x720)
#
class YampScreenV31(Screen, InfoBarBase, InfoBarSeek, InfoBarNotifications, HelpableScreen):
#	@classmethod
	def __init__(self, session):
		try:
			filename1 = yampDir + 'images/coverlcd.png'
			filename2 = '/tmp/coverlcd.png'
			os.system ("cp %s %s" % (filename1, filename2))
		except:
			pass
		Screen.__init__(self, session)
		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "Yamp.xml"), 'r') as f:
			self.skin = f.read()
		InfoBarBase.__init__(self)
		InfoBarNotifications.__init__(self)
		HelpableScreen.__init__(self)

		self.screenSaverFirstCall = True		#avoid switching to next picture on first call of timer
		self.screenSaverManOff = False
		self.isVideoFullScreen = False

		self.LcdText = ""
		
		self.session = session
		self.oldService = self.session.nav.getCurrentlyPlayingServiceReference()
		self.session.nav.stopService()

		self.playlistparsers = {}	# parsers for external playlists
		self.addPlaylistParser(YampParserM3u, ".m3u")
		self.addPlaylistParser(YampParserPls, ".pls")
		self.addPlaylistParser(YampParserE2pls, ".e2pls")
		
		startingDir = config.plugins.yampmusicplayer.filebrowserPath.value
		self.filelist = YampFileList(startingDir, matchingPattern = "(?i)^.*\.(mp2|mp3|ogg|ts|wav|m3u|pls|e2pls|mpg|vob|avi|divx|m4v|mkv|mp4|m4a|dat|flac|mov|m2ts)", showDirectories = True, showFiles = True, useServiceRef = True, enableWrapAround = True, additionalExtensions = "4098:m3u 4098:e2pls 4098:pls")
		try:
			startingDir2 = '/usr/lib/enigma2/python/Plugins/Extensions/YampMusicPlayer/images/'
			self.filelistint = YampFileList(startingDir2, matchingPattern = "(?i)^.*\.(mp2|mp3|ogg|ts|wav|m3u|pls|e2pls|mpg|vob|avi|divx|m4v|mkv|mp4|m4a|dat|flac|mov|m2ts)", showDirectories = False, showFiles = True, useServiceRef = True, enableWrapAround = False, additionalExtensions = "4098:m3u 4098:e2pls 4098:pls")
		except:
			pass
		self["filelist"] = self.filelist
###		self["Vorschau"] = VideoWindow(decoder = 1, fb_width=428, fb_height=399)
###		self["Video"] = VideoWindow(decoder = 0, fb_width=desktopSize.width(), fb_height=desktopSize.height())
		self.dblist = YampDatabaseList()
		self["dblist"] = self.dblist
		self["title"] = Label("YAMP Music Player  " + YAMPVERSION)
		self["leftContentTitle"] = Label ("")
		self["rightContentTitle"] = Label (_("P l a y l i s t"))
		self.playlist = YampPlayList()
		self["playlist"] = self.playlist
		self["songtitle"] = Label ("")
		self["artist"] = Label ("")
		self["album"] = Label ("")
		self["year"] = Label ("")
		self["genre"] = Label ("")
		self["nextsong"] = Label (_("Next Song"))
		self["nextsongtitle"] = Label ("")
		self["coverArt"] = YampCoverArtPixmap()
		self["screenSaver"] = YampScreenSaver()
		self["screenSaver"].hide()
		self["screenSaverBackground"] = Pixmap()
		self["screenSaverBackground"].hide()
		self["line1"] = Label (".")
		self["line2"] = Label (".")
		self["repeat"] = MultiPixmap()
		self["shuffle"] = MultiPixmap()
		self["saver"] = MultiPixmap()
		self["tv"] = MultiPixmap()
		self["radio"] = MultiPixmap()
		self["video"] = MultiPixmap()
		self["key_red"] = Label ("")
		self["key_green"] = Label ("")
		self["key_yellow"] = Label ("")
		self["key_blue"] = Label ("")

		self["fanartdownload"] = Label ("")
		self["musicbrainzlogo"] = Boolean(False)
		self["fanartlogo"] = Boolean(False)
		self["downloadBackground"] = Boolean(False)
		self["songTextsBackground"] = Boolean(False)

		self["bitrate"] = Label ("")
		self["noVideoLabelHide"] = Label ("")

		self["lyricsLine"] = Label ("")
		self["lyricsLineBackground"] = Boolean(False)
		self["lyricsLineBig"] = Label ("")
		self["lyricsLineBackgroundBig"] = Boolean(False)

		self["videoPreviewDummy"] = Pixmap()
		self["videoPreviewDummy"].hide()
		
# Timers
		self.isDreamOS = os.path.exists("/var/lib/dpkg/status")
		self.screenSaverTimer = eTimer()
		if self.isDreamOS: self.screenSaverTimer_conn=self.screenSaverTimer.timeout.connect(self.screenSaverStartTimeout)
		else:	self.screenSaverTimer.callback.append(self.screenSaverStartTimeout)
		self.nextSlideTimer = eTimer()
		if self.isDreamOS: self.nextSlideTimer_conn=self.nextSlideTimer.timeout.connect(self.nextSlide)
		else:	self.nextSlideTimer.callback.append(self.nextSlide)
		self.screenSaverManTimer = eTimer()
		if self.isDreamOS: self.screenSaverManTimer_conn=self.screenSaverManTimer.timeout.connect(self.screenSaverMaxManOff)
		else: self.screenSaverManTimer.callback.append(self.screenSaverMaxManOff)
		self.eofbugTimer = eTimer()
		if self.isDreamOS: self.eofbugTimer_conn=self.eofbugTimer.timeout.connect(self.checkEOF)
		else:	self.eofbugTimer.callback.append(self.checkEOF)
		self.startupTimer = eTimer()
		if self.isDreamOS: self.startupTimer_conn=self.startupTimer.timeout.connect(self.startupChecks)
		else:	self.startupTimer.callback.append(self.startupChecks)
		self.fanartDlTimer = eTimer()
		if self.isDreamOS: self.fanartDlTimer_conn=self.fanartDlTimer.timeout.connect(self.fanartDlOk)
		else:	self.fanartDlTimer.callback.append(self.fanartDlOk)
		self.fanartDisplayTimer = eTimer()
		if self.isDreamOS: self.fanartDisplayTimer_conn=self.fanartDisplayTimer.timeout.connect(self.fanartDisplayStop)
		else:	self.fanartDisplayTimer.callback.append(self.fanartDisplayStop)
		self.karaokeTimer = eTimer()
		if self.isDreamOS: self.karaokeTimer_conn=self.karaokeTimer.timeout.connect(self.checkKaraoke)
		else:	self.karaokeTimer.callback.append(self.checkKaraoke)
		self.karaokeTime = 250	
		self.karaokeMaxtimeTimer = eTimer()
		if self.isDreamOS: self.karaokeMaxtimeTimer_conn=self.karaokeMaxtimeTimer.timeout.connect(self.karaokeMaxtimeTimeout)
		else:	self.karaokeMaxtimeTimer.callback.append(self.karaokeMaxtimeTimeout)
#		self.karaokeMaxTime = 10000	
		self.checkVideoTimer = eTimer()
		if self.isDreamOS: self.checkVideoTimer_conn=self.checkVideoTimer.timeout.connect(self.checkVideoTimerTimeout)
		else:	self.checkVideoTimer.callback.append(self.checkVideoTimerTimeout)
		
		self.logpath = config.plugins.yampmusicplayer.debugPath.value
		self.timestamp = datetime_datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
		config.plugins.yampmusicplayer.debugFile.value = self.logpath + 'yamp_debug_' + self.timestamp  + '.log'			
		self.memlog = config.plugins.yampmusicplayer.yampDebugMemory.value
		self.memlogfile = self.logpath + 'yamp_mem_' + self.timestamp +'.log'

		
		
		# Action maps		
###		LOG('YampScreen: init: yampRemote: %s' %(config.plugins.yampmusicplayer.yampRemote.value), 'spe')
		if config.plugins.yampmusicplayer.yampRemote.value == 'default':
			self["Actions"] = HelpableActionMap(self,"YampActions",
			{
				"ok": (self.ok, _("Open dir, append to playlist | Play title, toggle screensaver")),
				"play": (self.play, _("Insert into playlist and play")),
				"pause": (self.pause, _("Pause/resume title")),
				"stop": (self.stopEntry, _("Stop title")),
				"exit": (self.exit, _("Exit screensaver, subscreens or YAMP")),
				"menu": (self.showMenu, _("Show context menu")),
				"prevTitle": (self.previousEntry, _("Play previous title")),
				"nextTitle": (self.nextEntry, _("Play next title")),
				"prevBouquet": (self.switchToPlayListOrShuffle, _("Switch to playlist | Toggle shuffle")),
				"nextBouquet" : (self.switchToFileListOrRepeat, _("Toggle repeat | Switch to filelist/database")),
				"right" : (self.pagedown, _("Next page, Screensaver: next slide")),
				"left" : (self.pageup, _("Previous page, Screensaver: previous slide")),
				"info" : (self.showLyrics, _("Show lyrics")),
				"toggleSaver": (self.pausePlaySlideShow, _("Pause/resume screensaver slideshow")),
				"altMoveTop": (self.skipToListbegin, _("Move to start of list")),
				"altMoveEnd": (self.skipToListend, _("Move to end of list")),
				"Audio" : (self.toggleSsBg, _("Toggle Screensaver Background")),
				"Pvr" : (self.pvrActions, _("Activate/deactivate screensaver slideshow")),
#				"Pvr" : (self.showhideScreenSaverMan, _("Activate/deactivate screensaver slideshow")),
				"Text" : (self.keyTextActions, _("Show/hide Lyrics Line ")),
				}, -2)
		else:
			self["Actions"] = HelpableActionMap(self,"YampActions",
			{
				"ok": (self.ok, _("Open dir, append to playlist | Play title, toggle screensaver")),
				"play": (self.play, _("Insert into playlist and play")),
				"pause": (self.pause, _("Pause/resume title")),
				"stop": (self.stopEntry, _("Stop title")),
				"exit": (self.exit, _("Exit screensaver, subscreens or YAMP")),
				"menu": (self.showMenu, _("Show context menu")),				
				"prevTitle": (self.previousEntry, _("Play previous title")),
				"nextTitle": (self.nextEntry, _("Play next title")),
				"prevBouquet": (self.pagedownspecial, _("Next page")),
				"nextBouquet" : (self.pageupspecial, _("Previous page")),
				"right" : (self.switchToPlayList, _("Switch to playlist, Screensaver: next slide")),
				"left" : (self.switchToFileList, _("Switch to filelist/database, Screensaver: previous slide")),
				"info" : (self.showLyrics, _("Show lyrics")),
				"toggleSaver": (self.pausePlaySlideShow, _("Pause/resume screensaver slideshow")),
				"altMoveTop": (self.skipToListbegin, _("Move to start of list")),
				"altMoveEnd": (self.skipToListend, _("Move to end of list")),
				"Tv": (self.toggleRepeat, _("Toggle repeat")),
				"Radio" : (self.toggleShuffle, _("Toggle shuffle")),
				"Audio" : (self.toggleSsBg, _("Toggle Screensaver Background")),
				"Pvr" : (self.pvrActions, _("Activate/deactivate screensaver slideshow")),
#				"Pvr" : (self.showhideScreenSaverMan, _("Activate/deactivate screensaver slideshow")),
				"Text" : (self.keyTextActions, _("Show/hide Lyrics Line ")),
				"TextLong" : (self.keyTextLongActions, _("Show/hide Lyrics Line ")),
			}, -2)

		self["OtherActions"] = ActionMap(["YampOtherActions"],
		{
			"up": self.moveup,
			"down": self.movedown,
			"moveTop": self.skipToListbegin,
			"moveEnd": self.skipToListend,
			"red": self.redActions,
			"redLong": self.redLongActions,
			"green": self.greenActions,
			"greenLong": self.greenLongActions,
			"yellow": self.yellowActions,
			"yellowLong": self.yellowLongActions,
			"blue": self.blueActions,
			"blueLong": self.blueLongActions,
		}, -2)
		
		self["HelpActions"] = ActionMap(["YampHelpActions"],
		{
			"help": self.showHelp,
		}, -2)
		
		InfoBarSeek.__init__(self, actionmap = "YampSeekActions")

		self.manKeyScreensaver = config.plugins.yampmusicplayer.manKeyScreensaver.value
		
		if config.plugins.yampmusicplayer.startWithDatabase.value:
			self.currList = "dblist"
		else:
			self.currList = "filelist"
		self.currPath = ""
		self.currFilePath = startingDir
		self.extPlaylistName = "" # no external playlist loaded
		self.repeat = config.plugins.yampmusicplayer.repeatPlaylistAtEnd.value
		self.ssBackground = config.plugins.yampmusicplayer.screenSaverBg.value
		self.ssBgHelpText = "."
		self.greenLongActive=False
		self.redLongActive=False
		self.yellowLongActive=False
		self.blueLongActive=False
		self.textLongActive=False
		self.dbStack = []  # for navigation in the database list
		self.dbTitleList = []
		self.dbArtistList = []
		self.dbAlbumList = []
		self.databaseChanged=True
		self.currTitle = ""
		self.currArtist = ""
		self.currAlbum = ""
		self.currLength = ""
		self.currTracknr = -1
		self.currGenre = ""
		self.currYear = ""
		self.currBitRate = ""
		self.oldTitle = ""
		self.oldArtist = ""
		self.oldAlbum = ""
		self.nextsongtitle = ""
		self.nextSongDisplay = ""
		self.oldIsVideo = False
		self.blankVideoInserted = -1
		self.slidePath = ""
		self.screenSaverActive = False
		self.slideShowPaused = False
		self.playerState = STATE_NONE
		self.currentIsVideo = False
		self.configScreenActive = False
		self.lyricsScreenActive = False
		self.dbScreenActive = False
		self.virtKeyboardActive = False
		self.pixNumCover = COVERS_NO

		self.requestNum = 0	
		self.requestNumSav1 = 0	
		self.requestNumSav2 = 0	
		self.coverIndex = 0
		self.coverChangedLyrics = False	#for Lyrics
		self.coverChangedGoogleLyrics = False	#for Lyrics
		
		self.artistIdFanart = 0		#for fanart download
		self.artistFanartDl = ''		#for fanart download
		self.artistFanartDlFinished = False
		self.fanartDlTime = FANARTDLTIME
		self.fanartNumberDl = 0			#total number of available new pictures
		self.fanartSuccessDl = 0		#total number of succesfully downloadedpictures

		self.infoBarNaReplace = 'n/a'
		
		self.fanarttvAppApikey='560a6463c90eaa2c586b6abe1c936826'
		self.fanarttvPersonalApikey=''
#		self.fanarttvPersonalApikey='f7521d44cb60298496da5341fa629653'
	
		self.videoStartMode = config.plugins.yampmusicplayer.videoAutoFull.value
		self.lyricsPlayOffsetTime90 = config.plugins.yampmusicplayer.lyricsPlayOffsetTime.value * 90
	
		self.lastPlaylistSearch = ''
		self.lastTextSearch = ''
		
		self.oldSkin = ''
#		self.oldLCD = ''
		self.oldLcdOff = 0
		self.oldLcdRolling = 0

		self.artistBgPicsList = []
		self.newArtistBg = False
		
		self.onLayoutFinish.append(self.layoutFinished)
		self.onClose.append(self.cleanup)

		self.oldLyricsText = ''				#last found Lyrics text
		self.displayLyricsText = ''		#lyrics text in display
		self.oldTiStampMsec90 = -1
		self.lyricsLineShow = False
		self.lyricsLineShowBig = False
		self["lyricsLine"].hide()
		self["lyricsLineBig"].hide()

		self.videoScreenActive = False

		self.videoTitleScreenWaitForClosed = False
		self.videoLyricsScreenWaitForClosed = False
#		self.videoTitleLyricsScreenWaitForClosed = False
		self.videoTitleScreenClose = False
		self.videoLyricsScreenClose = False
#		self.videoTitleLyricsScreenClose = False
		self.videoTitleScreenOpen = False
		self.videoLyricsScreenOpen = False
#		self.videoTitleLyricsScreenOpen = False
		self.menuRequest = False
		
		self.__event_tracker = ServiceEventTracker(screen=self, eventmap=
			{
				iPlayableService.evUpdatedInfo: self.__evUpdatedInfo,
				iPlayableService.evUser+10: self.__evAudioDecodeError,
				iPlayableService.evUser+11: self.__evVideoDecodeError,
				iPlayableService.evUser+12: self.__evPluginError,
			})
			
		try:
			import sys
			yampDirAbs = os.path.abspath(yampDir)
#			LOG('YampScreen: init: sys.path before append: %s' %(sys.path), 'err')
		except:
			LOG('YampScreen: init: sys.path before append: EXCEPT', 'err')
		try:
			if not yampDirAbs in sys.path:
				sys.path.append(yampDirAbs)
#				LOG('YampScreen: init: pathappend: appended: %s' %(yampDirAbs), 'err')
#			else:	
#				LOG('YampScreen: init: pathappend: path already exists: %s' %(yampDirAbs), 'err')
		except:
			LOG('YampScreen: init: pathappend: EXCEPT', 'err')
		try:
			pass
#			LOG('YampScreen: init: sys.path after append : %s' %(sys.path), 'err')
		except:
			LOG('YampScreen: init: sys.path after append : EXCEPT', 'err')
			

	def lockShow(self):
		pass

	def unlockShow(self):
		pass
		
			
#
# Start and end methods, handle some events from e2
#
	def layoutFinished(self):
		self.currDblist = DblistStackEntry()
		self.buildDbMenuList(MENULIST, menutitle=_("M u s i c  D a t a b a s e"))
		self.setLeftContent()
		self.setRepeatButton()
		self.setShuffleButton()
		self.setHelpTexts()
		self.setColorButtons()
		self.loadLastPlaylist()
		self.initScreenSaver()
		self.setSaverButton()
		self["noVideoLabelHide"].show()
		try:
			self.createLcdCoverImage(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, 'no_coverArt.png'))
		except: LOG('YampScreen: layoutFinished: createLcdCoverImage: EXCEPT', 'err')

		#Start playlist immediate
		try:
			if config.plugins.yampmusicplayer.startImmediate.value and len(self.playlist) > 0:
				self.changeEntry(self.playlist.getSelectionIndex())
###				LOG('YampScreen: layoutFinished: call startScreenSaverTimer', 'spe')
				self.startScreenSaverTimer()
		except:
			LOG('YampScreen: layoutFinished: EXCEPT', 'err') 
		#Screensaver in Background
		self.setScreensaverBg()
		self.startupTimer.start(1000, True)
		self.checkVideoTimer.start(200, False)
		
		try:
			self.infoBarNaReplace = INFOBARTEXTLIST[int(config.plugins.yampmusicplayer.displayUnknown.value)]
###			LOG('YampScreen: layoutFinished: infoBarNaReplace: %s' %(self.infoBarNaReplace), 'spe')
		except:
			LOG('YampScreen: layoutFinished: infoBarNaReplace: EXCEPT' ,'err')
			
		self.coverTestMode = config.plugins.yampmusicplayer.yampCoverLyricsTestMode.value
		self.searchFanart = config.plugins.yampmusicplayer.searchFanart.value
		LOG('YampScreen: layoutFinished: before read file: fanarttvPersonalApikey: %s' %(self.fanarttvPersonalApikey), 'all')
		LOG('YampScreen: layoutFinished: before read file: config fanarttvPersonal: %s' %(config.plugins.yampmusicplayer.fanarttvPersonal.value), 'all')
		if os.path.isfile(FANARTTVAPPAPIFILE):
			f = open(FANARTTVAPPAPIFILE,"r")
			keyname = f.read().decode('utf-8-sig').strip()
			if len(keyname) > 0: self.fanarttvAppApikey = keyname
			f.close()
			LOG('YampScreen: layoutFinished: read from file: fanarttvAppApikey: %s' %(self.fanarttvAppApikey), 'all')
		if os.path.isfile(FANARTTVPERSAPIFILE):
			f = open(FANARTTVPERSAPIFILE,"r")
			keyname = f.read().decode('utf-8-sig').strip()
			LOG('YampScreen: layoutFinished: read personal API from file: len: %d keyname: *%s*' %(len(keyname),keyname), 'all')
			if len(keyname) > 30: 
				self.fanarttvPersonalApikey = keyname
				config.plugins.yampmusicplayer.fanarttvPersonal.value = self.fanarttvPersonalApikey
				LOG('YampScreen: layoutFinished: personal api key written to config from file: %s' %(self.fanarttvPersonalApikey), 'all')
			f.close()
		self.fanarttvPersonalApikey = config.plugins.yampmusicplayer.fanarttvPersonal.value
		if self.fanarttvPersonalApikey.strip().startswith('1234567890abcdef'): self.fanarttvPersonalApikey = ''
		if 'abcdef7890' in self.fanarttvPersonalApikey.strip(): self.fanarttvPersonalApikey = ''
		LOG('YampScreen: layoutFinished: after read file: fanarttvPersonalApikey: %s' %(self.fanarttvPersonalApikey), 'all')
		LOG('YampScreen: layoutFinished: after read file: config fanarttvPersonal: %s' %(config.plugins.yampmusicplayer.fanarttvPersonal.value), 'all')

		try:
			self.filelistint.selectionEnabled(1)
		except:
			LOG('YampScreen: layoutFinished: len(filelistint): EXCEPT', 'err')
	
#		self.fanarttvAppApikey='560a6463c90eaa2c586b6abe1c936826'
#		self.fanarttvPersonalApikey='f7521d44cb60298496da5341fa629653'
		
		#Screensaver show clock
#		try:
#			if config.plugins.yampmusicplayer.showClockSS.value:
#				self["screenSaver"].setZPosition(5)
#			else:
#				self["screenSaver"].setZPosition(9)
#			LOG('YampScreen: layoutFinished: setZPosition : ok', 'err') 
#		except:
#			LOG('YampScreen: layoutFinished: setZPosition : EXCEPT', 'err') 

	def checkVideoTimerTimeout(self):
#		LOG('YampScreen: checkVideoTimerTimeout: Start; videoScreenActive: %d  videoTitleScreenClose: %d' %(self.videoScreenActive, self.videoTitleScreenClose), 'err')
#		if not self.videoScreenActive: return
#		LOG('\nYampScreen: checkVideoTimerTimeout: Start: videoTitleScreenClose: %d videoLyricsScreenClose: %d' %(self.videoTitleScreenClose, self.videoLyricsScreenClose), 'err')
#		LOG('YampScreen: checkVideoTimerTimeout: Start: videoTitleScreenWaitForClosed: %d videoLyricsScreenWaitForClosed: %d ' %(self.videoTitleScreenWaitForClosed, self.videoLyricsScreenWaitForClosed), 'err')
#		LOG('YampScreen: checkVideoTimerTimeout: videoTitleScreenOpen: %d videoLyricsScreenOpen: %d' %(self.videoTitleScreenOpen, self.videoLyricsScreenOpen), 'err')
		try:
			if self.videoTitleScreenClose:
				self.videoTitleScreenWaitForClosed = True
				self.videoTitleScreenClose = False
				self.session.close(YampVideoTitleV32)
#				LOG('YampScreen: checkVideoTimerTimeout: CloseVideoTitle: OK', 'err')
		except:
			LOG('YampScreen: checkVideoTimerTimeout: CloseVideoTitle: EXCEPT', 'err')
		try:
			if self.videoLyricsScreenClose:
				self.videoLyricsScreenWaitForClosed = True
				self.videoLyricsScreenClose = False
				self.session.close(YampVideoLyricsV32)
#				LOG('YampScreen: checkVideoTimerTimeout: CloseVideoLyrics: OK', 'err')
		except:
			LOG('YampScreen: checkVideoTimerTimeout: CloseVideoLyrics: EXCEPT', 'err')

		if self.videoTitleScreenWaitForClosed or self.videoLyricsScreenWaitForClosed: return
#		LOG('YampScreen: checkVideoTimerTimeout: waitforclosed = 0', 'err')
		try:
			if self.videoTitleScreenOpen:
				self.videoTitleScreenOpen = False
				self.session.open(YampVideoTitleV32, self)
#				LOG('YampScreen: checkVideoTimerTimeout: OpenVideoTitle: OK', 'err')
		except:
			LOG('YampScreen: checkVideoTimerTimeout: OpenVideoTitle: EXCEPT', 'err')
		try:
			if self.videoLyricsScreenOpen:
				self.videoLyricsScreenOpen = False
				self.session.open(YampVideoLyricsV32, self)
#				LOG('YampScreen: checkVideoTimerTimeout: OpenVideoLyrics: OK', 'err')
		except:
			LOG('YampScreen: checkVideoTimerTimeout: OpenVideoLyrics: EXCEPT', 'err')

	def setScreensaverBg(self):
		try:
#			if config.plugins.yampmusicplayer.screenSaverBg.value:
			if self.ssBackground:
				self["screenSaverBackground"].setZPosition(4)
				self["screenSaver"].setZPosition(5)
			else:
				self["screenSaverBackground"].setZPosition(8)
				self["screenSaver"].setZPosition(9)
			try: 
				if self.ssBgHelpText == ".": self.ssBgHelpText = ".."
				else: self.ssBgHelpText = "."
				self["line1"].instance.setText(self.ssBgHelpText)
				self["line2"].instance.setText(self.ssBgHelpText)
				self["screenSaver"].refreshPic()
			except: LOG('YampScreen: setScreensaverBg: setxxx: EXCEPT', 'err') 
		except:
			LOG('YampScreen: setScreensaverBg: setZPosition : EXCEPT', 'err') 

			
	def memoryLog(self):
		MEMCONFFILE = 'yamp_memlog.cfg'
		try:
#			LOG('YampScreen: memoryLog: Start', 'err')
			meminfo = open("/proc/meminfo","r")
			memlines = meminfo.read().split('\n')
			meminfo.close()
#			LOG('YampScreen: memoryLog: memlines: %s'  %(memlines), 'err')
			
			configfile = None
			conffilepath = self.logpath + MEMCONFFILE
#			LOG('YampScreen: memoryLog: conffilepath: %s'  %(conffilepath), 'err')
			if not os.access(conffilepath, os.R_OK):
				LOG('YampScreen: memoryLog: config file %s does not exist, debugging all meminfo, creating config file' %(conffilepath), 'err')
				configlist = []
				for x in range(len(memlines)-1):
					configlist.append(str(x))
				try:
					configfile = open(conffilepath, "w")
					configfile.write(','.join(configlist))
					configfile.close()
				except:
					LOG('YampScreen: memoryLog: could not create config file: %s' %(conffilepath), 'all')
			else:
				configfile = open(conffilepath, "r")
				configlist = configfile.read().split(',')
				configfile.close()
#			LOG('YampScreen: memoryLog: configlist: %s' %(configlist), 'err')

			logfile = open(self.memlogfile ,'a')
			logfile.write(datetime_datetime.now().strftime("%Y.%m.%d-%H:%M:%S.%f") + '\n')
			for line in configlist:
#				LOG('YampScreen: memoryLog: configlist line: %s int(line): %d memlines[int(line)]: %s' %(line,int(line),memlines[int(line)]), 'err')
				logfile.write(line + ':\t' + memlines[int(line)] +'\n')
			logfile.write('\n')
			logfile.close()	
			del logfile,meminfo,memlines,configfile,configlist,line
		except:
			LOG('YampScreen: memoryLog: EXCEPT', 'err')
	
	def createSummary(self):
		confLcd = config.plugins.yampmusicplayer.yampLCD.value
		if confLcd == 'off': return
		elif confLcd == 'rolling':
			return YampLCDRunningScreenV31
		else:
			return YampLCDScreenV31
		
	def exit(self):
#		LOG('YampScreen: exit() Start: newExitKey %d' %(config.plugins.yampmusicplayer.newExitKey.value), 'err')
#		LOG('YampScreen: exit() Start: screenSaverActive %d isVideoFullScreen: %d' %(self.screenSaverActive, self.isVideoFullScreen), 'err')
		self.exitYamp = True
		if self.screenSaverActive and not self.isVideoFullScreen:
			self.resetScreenSaverTimer()
			return
		if config.plugins.yampmusicplayer.newExitKey.value: 
			#new exit key: no immediate exit of yamp in dblist / filelist
			if self.currList == 'dblist': 
				if self.dblist.mode > MENULIST:
					self.exitYamp = False
			if self.currList == 'filelist':
#				LOG('\nYampScreeen: Exit: filelist: getSelection()[1] is dir: %s ' %(self.filelist.getSelection()[1]), 'spe')
#				try:
#					LOG('YampScreeen: Exit: filelist: getSelectionIndex(): %d ' %(self.filelist.getSelectionIndex()), 'spe')
#				except:		
#					LOG('\nYampScreeen: Exit: filelist: getSelectionIndex(): EXCEPT', 'spe')
				if self.filelist.getSelection()[1]:		#is directory
					try:
						strConfigPath = config.plugins.yampmusicplayer.filebrowserPath.value
						strPath = self.filelist.getSelection()[0]	
						if strPath == None: strPath = '' 
#						LOG('YampScreeen: Exit: filelist: strConfigPath: %s' %(strConfigPath), 'spe')
#						LOG('YampScreeen: Exit: filelist: strPath: %s' %(strPath), 'spe')

						#Spezialfall: auf "übergeordnetes Verzeichnis"
#						LOG('YampScreeen: Exit: filelist: special: getSelectionIndex(): %d ' %(self.filelist.getSelectionIndex()), 'spe')
#						LOG('YampScreeen: Exit: filelist: special: strPath: %s strConfigPath: %s' %(strPath,strConfigPath), 'spe')
						if self.filelist.getSelectionIndex() == 0 and strPath == strConfigPath:
#							LOG('YampScreeen: Exit: filelist: special', 'spe')
							self.exitYamp = False
						else:
							Pos = strPath.rfind('/',0,len(strPath)-1)
							strPath = strPath[:Pos+1]
#							LOG('YampScreeen: Exit: filelist: strPath2: %s' %(strPath), 'spe')
#							LOG('YampScreeen: Exit: filelist: len(strPath): %d len(strConfigPath): %d ' %(len(strPath),len(strConfigPath)), 'spe')
							if len(strPath) > len(strConfigPath):
								self.exitYamp = False
					except:
						LOG('YampScreen: exit(): len(strPath): EXCEPT', 'err')
				else:
					self.exitYamp = False
				
			if not self.exitYamp:
				try: self.skipToListbegin()
				except:	LOG('YampScreen: exit(): skipToListbegin: EXCEPT ', 'err')
				try: self.ok()
				except:	LOG('YampScreen: exit(): ok: EXCEPT ', 'err')
				self.setLeftContentTitle()	
				return;

		self.hideScreenSaver()
		if config.plugins.yampmusicplayer.yampConfirmExit.value:
			self.session.openWithCallback(self.exitConfirmed, MessageBox, _("Do you really want to exit?"))
		else:
			self.exitConfirmed(True)
				
				
	def exitConfirmed(self,answer):
		if not answer:		#no exit
#			if self.playerState < STATE_STOP and self.manKeyScreensaver and (
#						self.currList == "playlist" or config.plugins.yampmusicplayer.screenSaverLeft.value):
			self.resetScreenSaverTimer()

		else:	
			self.YampParserE2pls.clear()
			if config.plugins.yampmusicplayer.savePlaylistOnExit.value:
				for x in self.playlist.shadowList:
					self.YampParserE2pls.addService(ServiceReference(x[0]))
			self.YampParserE2pls.save(resolveFilename(SCOPE_CONFIG, "yampsl.e2pls"))
			self.YampParserE2pls.clear()
			if config.plugins.yampmusicplayer.savePlaylistOnExit.value:
				config.plugins.yampmusicplayer.lastPlaylistIndex.value = self.playlist.getCurrentIndex()
				config.plugins.yampmusicplayer.lastPlaylistIndex.save()
				for x in self.playlist.list:
					self.YampParserE2pls.addService(ServiceReference(x[0]))
			self.YampParserE2pls.save(resolveFilename(SCOPE_CONFIG, "yamp.e2pls"))

			if config.plugins.yampmusicplayer.saveLastFilebrowserPath.value:
				config.plugins.yampmusicplayer.filebrowserPath.value = self["filelist"].getCurrentDirectory()
				config.plugins.yampmusicplayer.filebrowserPath.save()

			config.plugins.yampmusicplayer.shuffle.value = self.playlist.isShuffeled	
			config.plugins.yampmusicplayer.shuffle.save()		
			config.plugins.yampmusicplayer.repeatPlaylistAtEnd.value = self.repeat
			config.plugins.yampmusicplayer.repeatPlaylistAtEnd.save()
			if self.leftContent == "dblist":
				config.plugins.yampmusicplayer.startWithDatabase.value = True
			else:
				config.plugins.yampmusicplayer.startWithDatabase.value = False
			config.plugins.yampmusicplayer.startWithDatabase.save()
			if len(self["screenSaver"]) > 0:
				config.plugins.yampmusicplayer.lastSlide.value = self["screenSaver"].getCurrentSlide()
			else:
				config.plugins.yampmusicplayer.lastSlide.value = ""
			config.plugins.yampmusicplayer.lastSlide.value = ""  #!!!! Test
			config.plugins.yampmusicplayer.lastSlide.save()
			self.currentIsVideo = False
			self.setVideoPreviewOnOff()
			configfile.save()

			del self["coverArt"].picload
			del self["screenSaver"].picload
			self.close(self.session, self.oldService)

	def cleanup(self):
		del self.screenSaverTimer
		del self.nextSlideTimer
		del self.eofbugTimer
		del self.screenSaverManTimer
		del self.startupTimer
		del self.fanartDlTimer
		del self.fanartDisplayTimer
		del self.karaokeTimer
		del self.karaokeMaxtimeTimer
		del self.checkVideoTimer
		for file in os.listdir('/tmp/'):
			if file.startswith('.coverart') or file == '.cover' or file == '.id3coverart':
				os.remove(os.path.join('/tmp/',file))
		#self.session.nav.playService(self.oldService)

	def makeNextSongDisplay(self):
#		LOG('\nYampScreen: makeNextSongDisplay: Start', 'err')
		selection = int(config.plugins.yampmusicplayer.displayNext.value)	
#		LOG('YampScreen: makeNextSongDisplay: selection: %d' %(selection),  'err')
#		LOG('YampScreen: makeNextSongDisplay: nextsongtitle: %s  album: %s  artist: %s' %(self.nextsongtitle, self.nextsongalbum,self.nextsongartist), 'err')
		if selection == 0: self.nextSongDisplay = self.nextsongtitle + "  -  " + self.nextsongartist + "  -  " + self.nextsongalbum
		elif selection == 1: self.nextSongDisplay = self.nextsongtitle + "  -  " + self.nextsongalbum + "  -  " + self.nextsongartist
		elif selection == 2: self.nextSongDisplay = self.nextsongartist + "  -  " + self.nextsongtitle + "  -  " + self.nextsongalbum
		elif selection == 3: self.nextSongDisplay = self.nextsongartist + "  -  " + self.nextsongalbum + "  -  " + self.nextsongtitle
		elif selection == 4: self.nextSongDisplay = self.nextsongtitle + "  -  " + self.nextsongartist
		elif selection == 5: self.nextSongDisplay = self.nextsongtitle + "  -  " + self.nextsongalbum
		elif selection == 6: self.nextSongDisplay = self.nextsongartist + "  -  " + self.nextsongtitle
		elif selection == 7: self.nextSongDisplay = self.nextsongartist + "  -  " + self.nextsongalbum
		elif selection == 8: self.nextSongDisplay = self.nextsongtitle
		
	def updateNextSong(self):
		if self.memlog: self.memoryLog()
		try:	
#			LOG('\nYampScreen: updateNextSong: getCurrentIndex(): %d  len(playlist): %d' %(self.playlist.getCurrentIndex(), len(self.playlist)), 'err')
			if len(self.playlist) == 0:
				self.nextsongtitle=self.nextsongalbum=self.nextsongartist=''
			elif self.playlist.getCurrentIndex() < len(self.playlist) - 1:
				pathname = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()+1].getPath()
				self.nextsongtitle, self.nextsongalbum, self.nextsonggenre, self.nextsongartist, self.nextsongdate, self.nextsonglength, self.nextsongtracknr, strBitrate = readID3Infos(pathname)
			elif self.repeat:
				pathname = self.playlist.getServiceRefList()[0].getPath()
				self.nextsongtitle, self.nextsongalbum, self.nextsonggenre, self.nextsongartist, self.nextsongdate, self.nextsonglength, self.nextsongtracknr, strBitrate = readID3Infos(pathname)
			else:
				self.nextsongtitle=self.nextsongalbum=self.nextsongartist=''
#			LOG('YampScreen: updateNextSong: nextsongtitle: %s  album: %s  artist: %s' %(self.nextsongtitle, self.nextsongalbum,self.nextsongartist), 'err')
		except:
			LOG('YampScreen: updateNextSong: nextsongtitle: EXCEPT', 'err')
		try:
			self.makeNextSongDisplay()
			self.updateSingleMusicInformation("nextsongtitle", self.nextSongDisplay)
#			self.updateSingleMusicInformation("nextsongtitle", self.nextsongtitle + "  -  " + self.nextsongartist + "  -  " + self.nextsongalbum)
		except:
			LOG('YampScreen: updateNextSong: EXCEPT', 'err')
			
	def __evUpdatedInfo(self):
#		LOG('\nYampScreen: __evUpdatedInfo: Start' , 'err')
		actPath = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()
#		LOG('YampScreen: __evUpdatedInfo: actPath: %s  currPath: %s' %(actPath, self.currPath), 'err')
		if self.currPath != actPath:
			# for 'next song'
#			LOG('YampScreen: __evUpdatedInfo: call updateNextSong()', 'err')
			self.updateNextSong()
			self.fanartDisplayStop()
			
			# for 'actual title'
			# __evUpdatedInfo may be called more than once per title, therefore:
			self.currPath = actPath
			#	Reading Info-Strings with getInfoString does not work with "Umlauts", so use mutagen methods
			pathname = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()
#			LOG('YampScreen: __evUpdatedInfo: actual title pathname: %s' %(pathname), 'err')
			title, album, genre, artist, date, length, tracknr, strBitrate = readID3Infos(pathname)
#			LOG('YampScreen: __evUpdatedInfo: actual title title: %s artist: %s album %s' %(title,artist, album), 'err')
			try:
				self.currentIsVideo = self.checkIsVideo(pathname)
###				LOG('YampScreen: __evUpdatedInfo: currentIsVideo: %d' %(self.currentIsVideo), 'spe')
			except:
				LOG('YampScreen: __evUpdatedInfo: currentIsVideo: EXCEPT', 'err')
#			self.currentIsVideo = self.checkIsVideo(currRef.getPath())
			self.updateMusicInformation(artist, title, album, date, genre, length, tracknr, strBitrate)
			#Karaoke
			try:
				self.buildNewLyrics()
			except:
				LOG('YampScreen: __evUpdatedInfo new Song: buildNewLyrics: EXCEPT', 'err')

	def buildNewLyrics(self):
#		LOG('\nYampScreen: buildNewLyrics: Start' , 'err')
		fileFound = False
		try:
			if self.karaokeTimer.isActive(): self.karaokeTimer.stop()
			self["lyricsLine"].setText('')
			self["lyricsLineBig"].setText('')
			self.oldLyricsText = ''
			songFilename = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()
			if config.plugins.yampmusicplayer.useSingleLyricsPath.value:
				self.lyricsFileName = os.path.join(config.plugins.yampmusicplayer.lyricsPath.value, os.path.basename(os.path.splitext(songFilename)[0] + ".txt"))
				self.lyricsFileNameLrc = os.path.join(config.plugins.yampmusicplayer.lyricsPath.value, os.path.basename(os.path.splitext(songFilename)[0] + ".lrc"))
			else:
				self.lyricsFileName = os.path.splitext(songFilename)[0] + ".txt"
				self.lyricsFileNameLrc = os.path.splitext(songFilename)[0] + ".lrc"
			LOG('YampScreen: buildNewLyrics: songFilename: %s lyricsFileName: %s' %(songFilename,self.lyricsFileName), 'all')
			LOG('YampScreen: buildNewLyrics: songFilename: %s lyricsFileName: %s' %(songFilename,self.lyricsFileNameLrc), 'all')
#			LOG('YampScreen: buildNewLyrics: songFilename: %s lyricsFileName: %s' %(songFilename,self.lyricsFileName), 'err')
#			LOG('YampScreen: buildNewLyrics: songFilename: %s lyricsFileName: %s' %(songFilename,self.lyricsFileNameLrc), 'err')
		except:
			LOG('YampScreen: buildNewLyrics: Start: EXCEPT', 'err')

		try:
			file = open(self.lyricsFileName, "r")
			self.lyrics = file.read()
			file.close()
#			LOG('YampScreen: buildNewLyrics: : lyrics: %s' %(self.lyrics), 'err')
			fileFound = True
		except:
			pass
		try:
			file = open(self.lyricsFileNameLrc, "r")
			self.lyrics = file.read()
			file.close()
#			LOG('YampScreen: buildNewLyrics: : lyricsFile: %s' %(self.lyrics), 'err')
			fileFound = True
		except:
			pass
		if fileFound == False:	
			self.oldLyricsText = self.displayLyricsText = _('no lyrics file existing')
			self["lyricsLine"].setText(self.oldLyricsText)
			self["lyricsLineBig"].setText(self.oldLyricsText)
#			if (self.screenSaverActive or self.isVideoFullScreen):
			if self.screenSaverActive:
				self.startKaraokeMaxTimer()
				if self.lyricsLineShow:
					self["lyricsLine"].show()
					self["lyricsLineBackground"].setBoolean(True)
				if self.lyricsLineShowBig:
					self["lyricsLineBig"].show()
					self["lyricsLineBackgroundBig"].setBoolean(True)
			return 

		try:
			self.tiStamp, self.tiStampMsec90, self.txtLines, self.minTiStamp = self.textToList(self.lyrics)
#			LOG('YampScreen: buildNewLyrics: : minTiStamp: %d' %(self.minTiStamp), 'err')
#			LOG('YampScreen: buildNewLyrics: : tiStamp: %s' %(self.tiStamp), 'err')
#			LOG('YampScreen: buildNewLyrics: : tiStampMsec90: %s' %(self.tiStampMsec90), 'err')
#			LOG('YampScreen: buildNewLyrics: : txtLines: %s' %(self.txtLines), 'err')
#			LOG('YampScreen: buildNewLyrics: len: %d' %(len(self.tiStampMsec90)-1), 'err')
		except: 
			LOG('YampScreen: buildNewLyrics: textToList: EXCEPT', 'err')
		try:
			if max(self.tiStampMsec90) == 0: 
#				LOG('YampScreen: buildNewLyrics: : tiStampMsec90: %s' %(self.tiStampMsec90), 'err')
				self.oldLyricsText = self.displayLyricsText = _('no time data in lyrics file')
				self["lyricsLine"].setText(self.oldLyricsText)
				self["lyricsLineBig"].setText(self.oldLyricsText)
				if self.screenSaverActive:
					self.startKaraokeMaxTimer()
				return
		except: 
			LOG('YampScreen: buildNewLyrics:  max(self.tiStampMsec90): EXCEPT', 'err')
		try:
#			if (self.screenSaverActive or self.isVideoFullScreen):
			if self.screenSaverActive:
				if self.lyricsLineShow:
					self["lyricsLine"].show()
					self["lyricsLineBackground"].setBoolean(True)
				if self.lyricsLineShowBig:
					self["lyricsLineBig"].show()
					self["lyricsLineBackgroundBig"].setBoolean(True)
			self.karaokeTimer.start(self.karaokeTime)
		except: 
			LOG('YampScreen: buildNewLyrics: startKaraokeTimer: EXCEPT', 'err')

			
	def checkKaraoke(self):
		if not self.videoScreenActive and self.menuRequest:
			self.menuRequest = False
			self.showMenu()

		try:
			length, pos = self.getSeekData()
#			LOG('YampScreen: checkKaraoke: length: %d songpos: %d' %(length,pos), 'err')
#			LOG('YampScreen: checkKaraoke: length: %d ' %(length), 'err')
#			LOG('YampScreen: checkKaraoke: songpos: %d' %(pos), 'err')
		except:
			LOG('YampScreen: checkKaraoke: songPos: EXCEPT', 'err')
		try:
			#play position < first text
			if pos < self.minTiStamp + self.lyricsPlayOffsetTime90:
				newText = self.oldLyricsText = self.displayLyricsText = ''
				self["lyricsLine"].setText(newText)
				self["lyricsLineBig"].setText(newText)
				return
		except:
			LOG('YampScreen: checkKaraoke: pos < self.minTiStamp: EXCEPT', 'err')
		try:
#			LOG('YampScreen: checkKaraoke: len: %d' %(len(self.tiStampMsec90)-1), 'err')
			for index in range(len(self.tiStampMsec90)-1, -1, -1):
#				LOG('YampLyricsScreen: checkKaraoke: index: %d songpos: %d tistamp: %d' %(index,songpos,self.tiStampMsec90[index]), 'err')
				if pos >= self.tiStampMsec90[index] + self.lyricsPlayOffsetTime90 and self.tiStampMsec90[index] != 0:
#					LOG('YampScreen: checkKaraoke: indexfound: idx: %d  stamp: %d songPos: %d' %(index,self.tiStampMsec90[index], self.songPos), 'err') ###Test
					newText = self.txtLines[index]
					if self.tiStampMsec90[index] != self.oldTiStampMsec90:
						self.oldTiStampMsec90 = self.tiStampMsec90[index]
						self.startKaraokeMaxTimer()
#						if self.karaokeMaxtimeTimer.isActive(): self.karaokeMaxtimeTimer.stop()
#						self.karaokeMaxtimeTimer.start(config.plugins.yampmusicplayer.karaokeMaxTime.value,True)
					if newText != self.oldLyricsText:
#						LOG('YampScreen: checkKaraoke: text: %s' %(newText), 'err')
						self.oldLyricsText = self.displayLyricsText = newText
						self["lyricsLine"].setText(newText)
						self["lyricsLineBig"].setText(newText)
					break	
					LOG('YampScreen: checkKaraoke: text: %s' %(self.txtLines[index]), 'err')
#			self.karaokeMaxtimeTimeout()
			if not self.karaokeMaxtimeTimer.isActive() and config.plugins.yampmusicplayer.karaokeMaxTime.value > 0:			
				self.oldLyricsText = ''
				self.displayLyricsText = ''
				self["lyricsLine"].setText('')
				self["lyricsLineBig"].setText('')
#				pass
#				LOG('YampScreen: checkKaraoke: found' , 'err')
		except:
			LOG('YampScreen: checkKaraoke: index: EXCEPT', 'err')

	def karaokeMaxtimeTimeout(self):
#		LOG('YampScreen: karaokeMaxtimeTimeout: Start', 'err')
		if not self.karaokeTimer.isActive() and config.plugins.yampmusicplayer.karaokeMaxTime.value > 0:
#			LOG('YampScreen: karaokeMaxtimeTimeout: execute', 'err')
#			self.oldLyricsText = ''
			self.displayLyricsText = ''
			self["lyricsLine"].setText('')
			self["lyricsLineBig"].setText('')

			
	def startKaraokeMaxTimer(self):
		maxTime = config.plugins.yampmusicplayer.karaokeMaxTime.value * 1000 
		if self.karaokeMaxtimeTimer.isActive(): self.karaokeMaxtimeTimer.stop()
		if maxTime > 0: 
#			LOG('YampScreen: startKaraokeMaxTimer: execute: maxTime: %d' %(maxTime), 'err')
			self.karaokeMaxtimeTimer.start(maxTime,True)
			
	def textToList(self, text):
		import re
#		LOG('YampLyricsScreen: textToList Start: text: %s' %(text), 'err') 
		textLines=[]
		tStamp=[]
		tStampmsecs90=[]
		tStampMin = -1
		import string, re
		try:
			textLines=string.split(text,'\n')
		except:
			LOG('YampLyricsScreen: textToList: text.split: EXCEPT', 'err') 
		try:
			for line in textLines:
#				LOG('YampLyricsScreen: textToList: line: %s' %(line), 'err') 
				tx=''
				timeStamp = re.findall(r'\[[0-9]+:[0-9]+.?[0-9]*\]',line)
#				LOG('YampLyricsScreen: textToList: timeStamp: %s' %(timeStamp), 'err') 
				if len(timeStamp): tx=timeStamp[len(timeStamp)-1]
				else: tx='[00:00.00]'
				if len(tx) == 7 and tx[6] == ']': #assuming milliseconds missing: [xx:yy]
					tx = tx[:6] + '.00]'
				tStamp.append(tx)
#				LOG('YampLyricsScreen: textToList: tx: %s' %(tx), 'err') 
				minutes = int(tx[1:3])
				seconds = float(tx[4:9]) 
				posMsec90 = int( ( (minutes * 60.0 + seconds) * 1000.0 ) * 90.0 ) 
				tStampmsecs90.append(posMsec90)
				if posMsec90 > 0:
					if posMsec90 < tStampMin or tStampMin == -1:
						tStampMin = posMsec90
#				LOG('YampLyricsScreen: textToList: minutes: %d seconds: %f  posMsecs90: %d' %(minutes,seconds,posMsec90), 'err') 
				
#				LOG('YampLyricsScreen: textToList: tStamp: %s' %(tStamp), 'err') 
#				LOG('YampLyricsScreen: textToList: timeStamp: %s  line: %s' %(tStamp[line], line), 'err') 
#				LOG('YampLyricsScreen: textToList: line: %s  type: %s' %(line, type(line)), 'err') 
#				textLines[line] = re.sub(r'\[[0-9]+:[0-9]+.?[0-9]*\]','',line)	#remove leading timecode [00:00.00]
		except:
			LOG('YampLyricsScreen: textToList: timeStamp: EXCEPT', 'err') 
#		try:
#			textLines = re.sub(r'\[[0-9]+:[0-9]+.?[0-9]*\]','',textLines)	#remove leading timecode [00:00.00]
#		except:
#			LOG('YampLyricsScreen: textToList: re.sub: EXCEPT', 'err') 
		try:
			textLines = re.sub(r'\[[0-9]+:[0-9]+.?[0-9]*\]','',text)	#remove leading timecode [00:00.00]
			textLines=string.split(textLines,'\n')
		except:
			LOG('YampLyricsScreen: textToList: re.sub: EXCEPT', 'err') 

		return tStamp, tStampmsecs90, textLines, tStampMin
		
			
	def __evAudioDecodeError(self):
		currPlay = self.session.nav.getCurrentService()
		sTagAudioCodec = currPlay.info().getInfoString(iServiceInformation.sTagAudioCodec)
		LOG("[YampScreeen: __evAudioDecodeError] audio-codec %s can't be decoded by hardware" %(sTagAudioCodec), 'err')
#		print "[YAMP __evAudioDecodeError] audio-codec %s can't be decoded by hardware" % sTagAudioCodec
		self.session.open(MessageBox, _("This Box can't decode %s streams!") % sTagAudioCodec, type = MessageBox.TYPE_INFO,timeout = 20 )

	def __evVideoDecodeError(self):
		currPlay = self.session.nav.getCurrentService()
		sTagVideoCodec = currPlay.info().getInfoString(iServiceInformation.sTagVideoCodec)
		LOG("[YampScreeen: __evVideoDecodeError] video-codec %s can't be decoded by hardware" % (sTagVideoCodec), 'err')
#		print "[YAMP __evVideoDecodeError] video-codec %s can't be decoded by hardware" % (sTagVideoCodec)
		self.session.open(MessageBox, _("This Box can't decode %s streams!") % sTagVideoCodec, type = MessageBox.TYPE_INFO,timeout = 20 )

	def __evPluginError(self):
		currPlay = self.session.nav.getCurrentService()
		message = currPlay.info().getInfoString(iServiceInformation.sUser+12)
		LOG( "[YampScreeen: __evPluginError] %s" %(message), 'err')
#		print "[YAMP __evPluginError]" , message
		self.session.open(MessageBox, message, type = MessageBox.TYPE_INFO,timeout = 20 )

#
# YAMP Main Screen actions
# There are 2 groups of actions, one is dependent of the active list (filelist, dblist or playlist), the other not
#
# List dependent actions:
#
	def ok(self):
###		LOG('\nYampScreeen: ok: start: screenSaverActive: %d  currList: %s' %(self.screenSaverActive, self.currList), 'spe' )
		previewVideoManual = self.currentIsVideo and self.videoStartMode == "manual"
###		LOG('YampScreeen: ok: : isVideoFullScreen: %d  ManOff: %d  IsVideo: %d' %(self.isVideoFullScreen, self.screenSaverManOff, self.currentIsVideo), 'spe' )

#		try:
#			self.close(YampVideoTitleV32)
#		except:
#			LOG('YampScreeen: ok: : close YampVideoTitleV32: EXCEPT', 'err' )
		
		#screensaver or Fullscreen Video: go back
		if self.screenSaverActive or self.isVideoFullScreen:
			if self.screenSaverActive:
				self.hideScreenSaver()
				if self.playerState < STATE_STOP and self.manKeyScreensaver: 
					self.resetScreenSaverTimer()
			if self.isVideoFullScreen:
				self.showYampScreenHideVideo()
			return
		if self.currList == "filelist":
			if self.filelist.canDescent():
				self.currFilePath = self.filelist.getSelection()[0]
				self.setLeftContentTitle()	
				self.filelist.descent()
				self.updateLCDInfo()
			else:
				self.addFile()
				self.playlist.moveToIndex(len(self.playlist)-1)
			self.movedown()
		elif self.currList == "dblist":
			self.dblistActions()
			self.updateLCDInfo()
			self.setLeftContentTitle()
			self.setColorButtons()
		elif self.currList == "playlist":
			alreadyPlaying = (self.playlist.getCurrentIndex() == self.playlist.getSelectionIndex()) and (self.playerState == STATE_PLAY)
			if not alreadyPlaying: 
###				LOG('YampScreen: ok: call changentry', 'spe')
				self.changeEntry(self.playlist.getSelectionIndex())

			previewVideoManual = self.currentIsVideo and self.videoStartMode == "manual"
			previewVideoImmediate = self.currentIsVideo and self.videoStartMode == "immediate"
			if self.playlist.getCurrentIndex() == self.playlist.getSelectionIndex():
###				LOG('YampScreen: ok: current == selection', 'spe')
				if self.screenSaverActive:
					pass
				else:
###					LOG('YampScreen: ok: not active', 'spe')
					if self.playerState == STATE_PLAY and self.session.nav.getCurrentlyPlayingServiceReference() is not None:
###						LOG('YampScreen: ok: STATE_PLAY  alreadyPlaying: %d' %(alreadyPlaying), 'spe')
						if not previewVideoManual:
###							LOG('YampScreen: ok: not previewVideoManual', 'spe')
							if alreadyPlaying or previewVideoImmediate: self.screenSaverStartTimeout()
							else: self.resetScreenSaverTimer()	
						else:
###							LOG('YampScreen: ok: VideoManual: isVideoFullScreen: %d SSactive: %d TimerActive: %d' %(self.isVideoFullScreen,self.screenSaverActive,self.screenSaverTimer.isActive()), 'spe')
							if alreadyPlaying:
###								LOG('YampScreen: ok: previewVideoManual: alreadyPlaying: hideYampScreenShowVideo', 'spe')
								self.hideYampScreenShowVideo()
							
			#prevent or switch to fullscreen video on start mode manual
###				LOG('YampScreen: ok: VideoManual: isVideoFullScreen: %d SSactive: %d TimerActive: %d' %(self.isVideoFullScreen,self.screenSaverActive,self.screenSaverTimer.isActive()), 'spe')
###			LOG('YampScreen: ok: currentIsVideo: %d videoStartMode: %s' %(self.currentIsVideo,self.videoStartMode), 'spe')
###			if previewVideoManual:
###				LOG('YampScreen: ok: VideoManual: isVideoFullScreen: %d SSactive: %d TimerActive: %d' %(self.isVideoFullScreen,self.screenSaverActive,self.screenSaverTimer.isActive()), 'spe')
#				self.screenSaverStartTimeout()
###				self.screenSaverTimer.stop()
###				if self.isVideoFullScreen: 
###					LOG('YampScreen: ok: previewVideoManual: self.showYampScreenHideVideo', 'spe')
###					self.showYampScreenHideVideo()
###				else:	
###					LOG('YampScreen: ok: previewVideoManual: self.hideYampScreenShowVideo', 'spe')
###					self.hideYampScreenShowVideo()

###			if not previewVideoManual: 
###				LOG('YampScreen: ok: call startScreenSaverTimer', 'spe')
###				self.startScreenSaverTimer()
			
	def play(self):
#		LOG('play Start', 'err')
#		try: LOG('YampScreen: play: Start: screenSaverActive: %d currentIsVideo: %d seekstate: %s' %(self.screenSaverActive, self.currentIsVideo, self.seekstate[3]), 'err')
#		except: LOG('YampScreen: play: Start: EXCEPT', 'err')
		if self.currList == "filelist":
#			if self.screenSaverActive or self.screenSaverTimer.isActive() or self.specialScreenActive(): #neuTest6
			if self.screenSaverActive or self.specialScreenActive():
				self.switchPlayPause()	
			else:											
				if self.filelist.canDescent():
					self.insertDir(self.filelist.getSelection()[0])
				else:
					self.insertFile()
###				LOG('YampScreen: play: call nextEntry 1', 'spe')
				self.nextEntry()
				self.movedown()
		elif self.currList == "dblist":
			try:
				navx = self.dblist.getSelection().nav
			except:
				navx = False
#			if not self.dblist.getSelection().nav:
#			if self.screenSaverActive or self.screenSaverTimer.isActive() or self.specialScreenActive(): #neuTest6
			if self.screenSaverActive or self.specialScreenActive(): #neuTest6
				self.switchPlayPause()		#neuTest6
			elif not navx:  #if not navx:   neuTest6
				mode = self.dblist.mode
				if mode >= TITLELIST:
					sref = ServiceReference(self.dblist.getSelection().ref)
					self.insertService(self.playlist.getCurrentIndex()+1, sref.ref, self.dblist.getSelection().title, self.dblist.getSelection().artist)
				elif mode in (PLAYLISTLIST, SEARCHPLAYLISTLIST):
					self.insertExtPlaylist(self.dblist.getSelection().filename)
				elif mode in (ARTISTLIST, SEARCHARTISTLIST):
					self.addCategory(query = self.dblist.getSelection().artistID, mode = "insert", pos = self.playlist.getCurrentIndex()+1)
				elif mode in (ALBUMLIST, ARTISTALBUMLIST, GENREALBUMLIST, SEARCHALBUMLIST):
					self.addCategory(query = self.dblist.getSelection().albumID, mode = "insert", pos = self.playlist.getCurrentIndex()+1)
				elif mode in (GENRELIST, SEARCHGENRELIST):
					self.addCategory(query = self.dblist.getSelection().genreID, mode = "insert", pos = self.playlist.getCurrentIndex()+1)
###				LOG('YampScreen: play: call nextEntry 2', 'spe')
				self.nextEntry()
				self.movedown()
		elif self.currList == "playlist":
			self.switchPlayPause()  #neuTest6
##			if config.plugins.yampmusicplayer.commonPlayPause.value:
##				if self.playerState == STATE_PLAY:
##					self.setSeekState(self.SEEK_STATE_PAUSE)
##					self.playerState = STATE_PAUSE
##				elif self.playerState == STATE_PAUSE:
##					self.setSeekState(self.SEEK_STATE_PLAY)
##					self.playerState = STATE_PLAY
##					self.startScreenSaverTimer()
##				elif self.playerState == STATE_STOP:
#					LOG('play:self.playerState == STATE_STOP', 'err')
##					self.changeEntry(self.playlist.getSelectionIndex())
##					self.startScreenSaverTimer()
##				elif self.playerState == STATE_NONE:
#					LOG('play:self.playerState == STATE_NONE', 'err')
##					self.changeEntry(self.playlist.getSelectionIndex())
##					self.startScreenSaverTimer()
##			else:
##				self.changeEntry(self.playlist.getSelectionIndex())
##				self.startScreenSaverTimer()

	def switchPlayPause(self): #neuTest6
###		LOG('YampScreen: switchPlayPause: call startSreenSaverTimer', 'spe')
		if config.plugins.yampmusicplayer.commonPlayPause.value:
			if self.playerState == STATE_PLAY:
				self.setSeekState(self.SEEK_STATE_PAUSE)
				self.playerState = STATE_PAUSE
			elif self.playerState == STATE_PAUSE:
				self.setSeekState(self.SEEK_STATE_PLAY)
				self.playerState = STATE_PLAY
				self.startScreenSaverTimer()
			elif self.playerState == STATE_STOP:
#				LOG('play:self.playerState == STATE_STOP', 'err')
				self.changeEntry(self.playlist.getSelectionIndex())
###				LOG('YampScreen: switchPlayPause: call changentry 1', 'spe')
				self.startScreenSaverTimer()
			elif self.playerState == STATE_NONE:
#				LOG('YampScreen: switchPlayPause: self.playerState == STATE_NONE', 'err')
###				LOG('YampScreen: switchPlayPause: call changentry 2', 'spe')
				self.changeEntry(self.playlist.getSelectionIndex())
				self.startScreenSaverTimer()
		else:
###			LOG('YampScreen: switchPlayPause: call changentry 3', 'spe')
			self.changeEntry(self.playlist.getSelectionIndex())
			self.startScreenSaverTimer()
				
	def pause(self):
#		LOG('YampScreen: Pause Start------------------', 'err')
#		try: LOG('YampScreen: pause: Start: screenSaverActive: %d currentIsVideo: %d seekstate: %s' %(self.screenSaverActive, self.currentIsVideo, self.seekstate[3]), 'err') #neuTest6
#		except: LOG('YampScreen: pause: Start: EXCEPT', 'err')
		self.pauseService()
		if self.seekstate == self.SEEK_STATE_PAUSE:
			self.hideScreenSaver()
			self.playerState = STATE_PAUSE
		else:
			if self.currList == "playlist":			
				self.showScreenSaver()
			self.playerState = STATE_PLAY
			
	def redActions(self):
#		LOG('YampScreen: redActions: Start', 'err')
		if self.redLongActive:		
			self.redLongActive = False
		else:	
			if not self.screenSaverActive and not self.isVideoFullScreen:
				if self.currList == "playlist":
					self.resetScreenSaverTimer()
					self.playingMoved = self.playlist.getSelectionIndex() == self.playlist.getCurrentIndex()
					self.playlist.moveEntryUp(self.playlist.getSelectionIndex(), config.plugins.yampmusicplayer.wrapMovePlaylist.value)
					self.playlist.updateList()
					self.updateNextSong()
				else:
					if self.currList == "filelist":
						self.currList = "dblist"
						if self.databaseChanged:
							self.updateDbMenuList()
							self.databaseChanged = False
					else:			#dblist
						self.currList = "filelist"
					self.setLeftContent()
					self.setColorButtons()
					self.updateLCDInfo()
		
	def greenActions(self):
#		LOG('YampScreen: greenActions Start', 'err')
		if self.greenLongActive:
			self.greenLongActive = False
		else:
			if not self.screenSaverActive and not self.isVideoFullScreen:
				if self.currList == "playlist":		#playlist
					self.resetScreenSaverTimer()
					self.playingMoved = self.playlist.getSelectionIndex() == self.playlist.getCurrentIndex()
					self.playlist.moveEntryDown(self.playlist.getSelectionIndex(), config.plugins.yampmusicplayer.wrapMovePlaylist.value)
					self.playlist.updateList()
					self.updateNextSong()
				elif self.currList == "filelist":
					if self.filelist.canDescent():
						self.addDir(self.filelist.getSelection()[0], recursive = False)
					else:
						self.addFile()
					self.playlist.moveToIndex(len(self.playlist)-1)
					self.movedown()
				else: 				#dblist
					try:
						navx = self.dblist.getSelection().nav
					except:
						navx = False
#					if not self.dblist.getSelection().nav:
					if not navx:
						mode = self.dblist.mode
						if mode >= TITLELIST:
							sref = ServiceReference(self.dblist.getSelection().ref)
							self.addService(sref.ref, self.dblist.getSelection().title, self.dblist.getSelection().artist)
							self.playlist.updateList()
							self.updateNextSong()
						elif mode in (PLAYLISTLIST, SEARCHPLAYLISTLIST):
							self.addExtPlaylist(self.dblist.getSelection().filename)
						elif mode in (ARTISTLIST, SEARCHARTISTLIST):
							self.addCategory(query = self.dblist.getSelection().artistID)
						elif mode in (ALBUMLIST, ARTISTALBUMLIST, GENREALBUMLIST, SEARCHALBUMLIST):
							self.addCategory(query = self.dblist.getSelection().albumID)
						elif mode in (GENRELIST, SEARCHGENRELIST):
							self.addCategory(query = self.dblist.getSelection().genreID)
						self.playlist.moveToIndex(len(self.playlist)-1)
						self.movedown()
	

	def redLongActions(self):
		LOG('YampScreen: redLongActions Start', 'all')
		self.redLongActive=True
		if self.screenSaverActive and not self.isVideoFullScreen:
			if self["screenSaver"].getLen() < 2:
				self.session.open(MessageBox, _("Slide is the last slide and can not be removed"), type = MessageBox.TYPE_INFO,timeout = 5 )
				return
			try:
				currSlide =self["screenSaver"].getCurrentSlide()
				LOG('YampScreen: redLongActions: getCurrentSlide: %s' %(currSlide), 'all')
			except:
				LOG('YampScreen: redLongActions: getCurrentSlide: EXCEPT ', 'err')
			try:
#				LOG('YampScreen: redLongActions: len(currSlide): %d' %(len(currSlide)), 'all')
#				LOG('YampScreen: redLongActions: blackpath: %s' %(yampDir + 'images/black.png'), 'all')
				if len(currSlide) > 0 and currSlide <> (yampDir + 'images/black.png'):
#					LOG('YampScreen: redLongActions: innerhalb if', 'all')
					if config.plugins.yampmusicplayer.screenSaverMode.value == "artwork":
						slidepath = config.plugins.yampmusicplayer.screenSaverArtworkPath.value
					else: slidepath = self.slidePath	
#					LOG('YampScreen: redLongActions: slidepath: %s' %(slidepath), 'all')
					message = _('this picture will be blacklisted to %sblacklist.txt\nand will not be shown any more!\n\ndo you really want to blacklist this picture?')	%(slidepath)
					try:
#						LOG('YampScreen: redLongActions: message: %s' %(message), 'all')
#						LOG('YampScreen: redLongActions: before messagebox', 'all')
						self.par1 = slidepath
						self.par2 = currSlide
						self.session.openWithCallback(self.blackListConfirmed, MessageBox, message)
#						LOG('YampScreen: redLongActions: after messagebox:', 'all')
					except:
						LOG('YampScreen: redLongActions: message: EXCEPT ', 'err')
			except:		
				LOG('YampScreen: redLongActions: getCurrentSlide: EXCEPT ', 'err')
				
	def blackListConfirmed(self,answer):
		try:
			LOG('YampScreen: blackListConfirmed: Start: slidepath: %s currslide: %s' %(self.par1,self.par2), 'all')
		except:	
			LOG('YampScreen: blackListConfirmed: Start: EXCEPT' , 'err')
		if not answer:				#no blacklist
#			LOG('YampScreen: blackListConfirmed: answer no', 'all')
			pass
		else:
#			LOG('YampScreen: blackListConfirmed: answer yes', 'all')
			try:
				file = open(self.par1 + 'blacklist.txt', "a")		#self.par1: slidepath
				file.write(self.par2 + '\n')										#self.par2: slide
				file.close
			except:
				self.session.open(MessageBox, _("blacklist file could not be saved"), type = MessageBox.TYPE_INFO,timeout = 5 )
			try:
				self["screenSaver"].removeSlide(self.par2)
#				self["screenSaver"].removeCurrentSlide()
				self.resetSlideTimer()
			except:
				LOG('YampScreen: blackListConfirmed: removeCurrentSlide: EXCEPT' , 'err')
		
	def greenLongActions(self):
		self.greenLongActive=True
		if not self.screenSaverActive and not self.isVideoFullScreen:
			if self.currList == "filelist" :
				if self.filelist.canDescent():
					self.addDir(self.filelist.getSelection()[0], recursive = True)
				else:
					self.addAllFiles()
				self.playlist.moveToIndex(len(self.playlist)-1)
			elif self.currList == "dblist":
				for x in self.dblist.getDatabaseList():
					if not x[0].nav and self.dblist.mode >= TITLELIST:
						sref = ServiceReference(x[0].ref)
						self.addService(sref.ref, x[0].title, x[0].artist)
				self.playlist.updateList()
				self.updateNextSong()
				self.playlist.moveToIndex(len(self.playlist)-1)

	def getSeekOwn(self):
		service = self.session.nav.getCurrentService()
		if service is None:
			return None

		seek = service.seek()

		if seek is None or not seek.isCurrentlySeekable():
			return None

		return seek
		
	def seekOwn(self, key):
		try:
			if key ==1 : secs = -15
			elif key ==3 : secs = 15
			elif key ==4 : secs = -60
			elif key ==6 : secs = 60
			elif key ==7 : secs = -300
			elif key ==9 : secs = 300
			elif key ==11 : 
				self.previousEntry()
				return
			elif key ==12 : 
				self.nextEntry()
				return
				
			pts = secs * 90000
#			LOG('YampScreen: seekOwn: secs: %d  pts: %d' %(secs,pts), 'err') 
			seekable = self.getSeek()
			if seekable is None:
				LOG('YampScreen: seekOwn: seekable NONE', 'err') 
				return
#			LOG('YampScreen: seekOwn: par1: %d  pts: %d' %(pts<0 and -1 or 1,abs(pts)), 'err') 

#			prevstate = self.seekstate
#			if self.seekstate == self.SEEK_STATE_EOF:
#				if prevstate == self.SEEK_STATE_PAUSE:
#					self.setSeekState(self.SEEK_STATE_PAUSE)
#				else:
#					self.setSeekState(self.SEEK_STATE_PLAY)

			seekable.seekRelative(pts<0 and -1 or 1, abs(pts))

#			service = self.session.nav.getCurrentService()
#			seek = service and service.seek()

#			seek.seekRelative(5 * 90000)
#			self.screen.seek.seekRelative(5 * 90000)
#			self.updatedSeekState()
		except: LOG('YampScreen: seekOwn: EXCEPT', 'err') 

		
	def sendRemoteKey(self, key):
		LOG('YampScreen: sendRemoteKey: %d' %(key), 'err') 
		self.seekOwn()
		return
		try:
			from enigma import eActionMap
			# Flags
			FLAG_MAKE = 0
			FLAG_BREAK = 1
			FLAG_REPEAT = 2
			FLAG_LONG = 3
			FLAG_ASCII = 4

			#Device Types
			TYPE_STANDARD = "dreambox remote control (native)"
			TYPE_ADVANCED = "dreambox advanced remote control (native)"
			TYPE_KEYBOARD = "dreambox ir keyboard"

			self.eam = eActionMap.getInstance()
		except: LOG('YampScreen: sendRemoteKey: import: EXCEPT', 'err') 
			
		try:
#				if config.misc.rcused.value == 0:
#					self.remotetype = self.TYPE_ADVANCED
#				else:
			remotetype = TYPE_STANDARD
#				remotetype = self.TYPE_STANDARD
#				key = 4
#				flag = self.FLAG_MAKE
			flag = FLAG_MAKE
		except: LOG('YampScreen: sendRemoteKey: flag: EXCEPT1', 'err') 
		try:
			#press the key with the desired flag
			LOG('YampScreen: sendRemoteKey: before pressed: %d' %(key), 'err') 
			self.eam.keyPressed(remotetype, key, flag)
				#Release the key
			LOG('YampScreen: sendRemoteKey: before break: %d' %(key), 'err') 
			self.eam.keyPressed(remotetype, key, FLAG_BREAK)
#				self.eam.keyPressed(remotetype, key, self.FLAG_BREAK)
			self.eam.keyPressed(remotetype, key, FLAG_BREAK)
			LOG('YampScreen: sendRemoteKey: after break: %d' %(key), 'err') 
		except: LOG('YampScreen: sendRemoteKey: keypressed: EXCEPT', 'err') 


	def yellowActions(self):
#		LOG('YampScreen: yellowActions: start: screenSaverActive: %d yellowLongActive: %d' %(self.screenSaverActive, self.yellowLongActive), 'err')
		if self.yellowLongActive:
			self.yellowLongActive = False
		else:
#			try: LOG('\nPlaylist: %s' %(self.playlist.list), 'err')	
#			except:	LOG('\nPlaylist: EXCEPT', 'err')	
#			try: LOG('Shadlist: %s' %(self.playlist.shadowList), 'err')	
#			except:	LOG('Shadlist: EXCEPT', 'err')	
#			return	
			if not self.screenSaverActive and not self.isVideoFullScreen:
				self.resetScreenSaverTimer()
				if self.currList == "filelist":
					if self.filelist.canDescent():
#						try: LOG('YampScreen: yellowActions: YampDatabaseScreen add: filename: %s' %(self.filelist.getFilename()), 'spe')
#						except: LOG('YampScreen: yellowActions: YampDatabaseScreen add: filename: EXCEPT', 'err')
						self.showDatabaseScreen()
					else:
						self.addFileToDb()
					self.movedown()
					self.databaseChanged=True
				elif self.currList == "dblist":
					numChoice = self.dblist.getSelectionIndex()
					choice = self.dblist.getSelection()
					if self.dblist.mode in (ARTISTLIST, SEARCHARTISTLIST):
						self.pushDblistStack (numChoice)
						self.buildDbMenuList (mode = ARTISTALBUMLIST, query = choice.artistID, menutitle = choice.artist)
					elif self.dblist.mode in (GENRELIST, SEARCHGENRELIST):
						self.pushDblistStack (numChoice)
						self.buildDbMenuList (mode = GENREALBUMLIST, query = choice.genreID, menutitle = choice.genre)
					elif self.dblist.mode in (TITLELIST, ARTISTTITLELIST, GENRETITLELIST, DATETITLELIST, SEARCHTITLELIST, SEARCHFILELIST, SEARCHARTISTTITLELIST):
						self.pushDblistStack (numChoice)
						self.buildDbMenuList (mode = ALBUMTITLELIST, query = choice.albumID, menutitle = choice.album)
					elif self.dblist.mode in (ALBUMTITLELIST, SEARCHALBUMTITLELIST):
						self.pushDblistStack (numChoice)
						self.buildDbMenuList (mode = ARTISTTITLELIST, query = choice.artistID, menutitle = choice.artist)
					self.setLeftContentTitle()
					self.setColorButtons()
				else:
					src = self["coverArt"].getFileName()
					if src != "":
						ext = os.path.splitext(os.path.basename(src))[1]
						dest = os.path.splitext(self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath())[0] + ext
						try:
							open(dest, 'wb').write(open(src, 'rb').read())
#							self.readWriteFileBinary(src, dest)
							self.session.open(MessageBox, _("Cover saved"), type = MessageBox.TYPE_INFO,timeout = 5 )
						except:
							self.session.open(MessageBox, _("Saving cover failed"), type = MessageBox.TYPE_INFO,timeout = 30 )

	def showDatabaseScreen(self, recursive=False):
		self.showYampScreenHideVideo()
		self.searchFanart = '0'
		self.hideScreenSaver()
		self.screenSaverManTimer.stop()
		self.dbScreenActive = True
#		LOG('YampScreen: showDbScreen:before open dbscreen', 'err')
		try: self.session.openWithCallback(self.showDbScreenCB, YampDatabaseScreenV32, "add", self.filelist.getFilename(),recursive = recursive)
		except: LOG('YampScreen: showDatabaseScreen: screenSaverManTimer: STOP', 'err') 
		
	def showDbScreenCB(self):
#		LOG('YampScreen: showDbScreenCB: self.searchFanart: %s' %(self.searchFanart), 'spe')
		self.searchFanart = config.plugins.yampmusicplayer.searchFanart.value
#		LOG('YampScreen: showDbScreenCB: self.searchFanart: %s' %(self.searchFanart), 'spe')
		self.startScreenSaverManTimer()
		self.dbScreenActive = False
		if not self.currentIsVideo or self.videoStartMode != "manual":
#			LOG('YampScreen: showDbScreenCB: resetScreenSaverTimer', 'spe') 
			self.resetScreenSaverTimer()
							
	def yellowLongActions(self):
#		LOG('YampScreen: yellowLongActions: start: screenSaverActive: %d isVideoFullScreen: %d' %(self.screenSaverActive, self.isVideoFullScreen), 'err')
		self.yellowLongActive=True
		if not self.screenSaverActive and not self.isVideoFullScreen:
			if self.currList == "filelist" :
				if self.filelist.canDescent():
#					try: LOG('YampScreen: yellowLongActions: YampDatabaseScreen add: filename: %s recursive: %d' %(self.filelist.getFilename(), recursive), 'spe2')
#					try: LOG('YampScreen: yellowLongActions: YampDatabaseScreen add: filename: %s recursive: %d' %(self.filelist.getFilename(), recursive), 'err')
#					except: LOG('YampScreen: yellowLongActions: YampDatabaseScreen LOG: EXCEPT', 'err')
					self.showDatabaseScreen(recursive = True)
				else:
					self.addFileToDb()
				self.movedown()
				self.databaseChanged=True
			elif self.currList == "dblist":
				if self.dblist.mode in (TITLELIST, ALBUMTITLELIST, GENRETITLELIST, SEARCHTITLELIST, SEARCHFILELIST, SEARCHALBUMTITLELIST):
					numChoice = self.dblist.getSelectionIndex()
					self.pushDblistStack (numChoice)
					choice = self.dblist.getSelection()
					self.buildDbMenuList (mode = ARTISTTITLELIST, query = choice.artistID, menutitle = choice.artist)
				self.setLeftContentTitle()
				self.setColorButtons()
			else:
				src = self["coverArt"].getFileName()
				if src != "":
					ext = os.path.splitext(os.path.basename(src))[1]
					dest = os.path.join(os.path.dirname(self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()),"albumcover") + ext
					try:
						open(dest, 'wb').write(open(src, 'rb').read())
#						self.readWriteFileBinary(src, dest)
						self.session.open(MessageBox, _("Albumcover saved"), type = MessageBox.TYPE_INFO,timeout = 5 )
					except:
						self.session.open(MessageBox, _("Saving albumcover failed"), type = MessageBox.TYPE_INFO,timeout = 30 )

#
# actions for switching between filelist/dblist and playlist and to toogle repeat and shuffle
#
	def switchToFileListOrRepeat(self):
#		LOG('YampScreen: switchToFileListOrRepeat: Start: screenSaverActive: %d playerState: %d' %(self.screenSaverActive, self.playerState), 'err')
		if self.screenSaverActive:
			self.hideScreenSaver()
		elif self.currList == "playlist":
#			if not config.plugins.yampmusicplayer.screenSaverLeft.value: 
#				LOG('YampScreen: switchToFileListOrRepeat: vor hideScreenSaver', 'err')
#				self.hideScreenSaver()
			if self.leftContent == "filelist":
				self.currList = "filelist"
				self.filelist.selectionEnabled(1)
			else:
				self.currList = "dblist"
				self.dblist.selectionEnabled(1)
			self.playlist.selectionEnabled(0)
			self.setColorButtons()
			self.updateLCDInfo()
#			if config.plugins.yampmusicplayer.screenSaverLeft.value: 
			self.resetScreenSaverTimer()
		else:
			self.toggleRepeat()
	
	#if not in standard V265 key mode		
	def switchToFileList(self):
#		LOG('YampScreen: switchToFileList: Start: screenSaverActive: %d playerState: %d' %(self.screenSaverActive, self.playerState), 'err')
		if self.screenSaverActive and self.playerState < STATE_STOP:
			self.prevSlide()
#			LOG('Yamp: switchToFileList: previousSlide:', 'all')
		elif self.currList == "playlist":
			if not config.plugins.yampmusicplayer.screenSaverLeft.value: 
#				LOG('YampScreen: switchToFileList: vor hideScreenSaver', 'err')
				self.hideScreenSaver()
			if self.leftContent == "filelist":
				self.currList = "filelist"
				self.filelist.selectionEnabled(1)
			else:
				self.currList = "dblist"
				self.dblist.selectionEnabled(1)
			self.playlist.selectionEnabled(0)
			self.setColorButtons()
			self.updateLCDInfo()
			self.resetScreenSaverTimer()
			
	def switchToPlayListOrShuffle(self):
#		LOG('switchToPlayListOrShuffle: Start', 'err')
###		LOG('YampScreen: switchToPlayListOrShuffle: call startScreenSaverTimer', 'spe')
		if self.screenSaverActive:
			self.hideScreenSaver()
		elif self.currList == "playlist":
			if not self.screenSaverActive:
				self.playlist.shuffleList()
				self.setShuffleButton()
		else:
			if len(self.playlist) > 0:
				self.currList = "playlist"
				self.filelist.selectionEnabled(0)
				self.dblist.selectionEnabled(0)
				self.playlist.selectionEnabled(1)
				self.setColorButtons()
				self.updateLCDInfo()
				self.startScreenSaverTimer()

	#if not in standard V265 key mode				
	def switchToPlayList(self):	
###		LOG('YampScreen: switchToPlayList: Start: screenSaverActive: %d playerState: %d' %(self.screenSaverActive, self.playerState), 'spe')
		if self.screenSaverActive:
			self.nextSlide()
#			LOG('Yamp: switchToPlayList: nextSlide:', 'all')
		elif self.currList != "playlist":
			if len(self.playlist) > 0:
				self.currList = "playlist"
				self.filelist.selectionEnabled(0)
				self.dblist.selectionEnabled(0)
				self.playlist.selectionEnabled(1)
				self.setColorButtons()
				self.updateLCDInfo()
###				LOG('YampScreen: switchToPlayList: call resetScreenSaverTimer', 'spe')
				self.resetScreenSaverTimer()
					
	def toggleSsBg(self):
#		LOG('YampScreen: toggleSsBg', 'err')
		if not self.isVideoFullScreen:
			if self["screenSaver"].refreshPic(test=True) == 0: return
			self.ssBackground = not self.ssBackground
			self.setScreensaverBg()

	def toggleRepeat(self):
		if not self.isVideoFullScreen:
			self.repeat = not self.repeat
			self.setRepeatButton()

	def setRepeatButton(self):
		if self.repeat:	self["repeat"].setPixmapNum(1)
		else:	self["repeat"].setPixmapNum(0)
		self.updateNextSong()

	def toggleShuffle(self):
#		LOG('YampScreen: toggleShuffle: Start', 'all')
		if not self.isVideoFullScreen:
			if self.currList == "playlist":
				if not self.screenSaverActive:
					self.playlist.shuffleList()
					self.setShuffleButton()
#					LOG('YampScreen: toggleShuffle: call updateNextSong()', 'err')
					self.updateNextSong()
			
	def setShuffleButton(self):
		if self.playlist.isShuffeled:	self["shuffle"].setPixmapNum(1)
		else:	self["shuffle"].setPixmapNum(0)
		
	def setSaverButton(self):
		if self.screenSaverTimer.isActive(): self["saver"].setPixmapNum(1)
		else:	self["saver"].setPixmapNum(0)

	def setHelpTexts(self):
		if config.plugins.yampmusicplayer.displayButtonHelp.value: 
			self["tv"].setPixmapNum(1)
			self["radio"].setPixmapNum(1)
			self["video"].setPixmapNum(1)
		else:	
			self["tv"].setPixmapNum(0)
			self["radio"].setPixmapNum(0)
			self["video"].setPixmapNum(0)
	
	def setColorButtons(self):
		self["key_blue"].setText(_("Delete Playlist Entry\nClear Playlist"))
		if self.currList == "filelist":
			self["key_red"].setText(_("Database Mode"))
			self["key_green"].setText(_("Add to Playlist\nAdd all to Playlist"))
			self["key_yellow"].setText(_("Add to Database\nAdd all to Database"))
		elif self.currList == "dblist":
			mode = self.dblist.mode
			self["key_red"].setText(_("Filelist Mode"))
			self["key_green"].setText("")
			if mode > MENULIST:
				if mode >= TITLELIST:
					self["key_green"].setText(_("Add to Playlist\nAdd all to Playlist"))
				else:
					self["key_green"].setText(_("Add to Playlist"))
			self["key_yellow"].setText("")
			if mode in (TITLELIST, GENRETITLELIST, SEARCHTITLELIST, SEARCHFILELIST):
				self["key_yellow"].setText(_("Show Album\nShow Artist"))
			elif mode in (ARTISTLIST, GENRELIST, SEARCHARTISTLIST, SEARCHGENRELIST):
				self["key_yellow"].setText(_("Show Albums"))
			elif mode in (ARTISTTITLELIST, SEARCHARTISTTITLELIST):
				self["key_yellow"].setText(_("Show Album"))
			elif mode in (ALBUMTITLELIST, SEARCHALBUMTITLELIST):
				self["key_yellow"].setText(_("Show Artist"))
		else: # playlist
			self["key_red"].setText(_("Move Entry Up"))
			self["key_green"].setText(_("Move Entry Down"))
			self["key_yellow"].setText(_("Save Cover\nSave Album Cover"))
			
	def setLeftContent(self):
		if self.currList == "dblist":
			self.leftContent = "dblist"
			self["filelist"].hide()
			self["dblist"].show()
			self.dblist.selectionEnabled(1)
			self.filelist.selectionEnabled(0)
		else:
			self.leftContent = "filelist"
			self["dblist"].hide()
			self["filelist"].show()
			self.filelist.selectionEnabled(1)
			self.dblist.selectionEnabled(0)
		self.setLeftContentTitle()

	def setLeftContentTitle(self):
		if self.leftContent == "filelist":
			layoutFileList = config.plugins.yampmusicplayer.fileListTitleLayout.value
			if self.currFilePath == None: self.currFilePath = ''
			if layoutFileList == 'titleOnly':
				self["leftContentTitle"].setText(_("M u s i c  F i l e s"))
			elif layoutFileList == 'pathOnly':	
				self["leftContentTitle"].setText(self.currFilePath)
			else:
				self["leftContentTitle"].setText(_("Music Files") + '  ' + self.currFilePath)
		else:
			self["leftContentTitle"].setText(self.dblist.title)

#
# list independent actions
#
	def showHelp(self):
		self.hideScreenSaver()
		if self.isVideoFullScreen: self.showYampScreenHideVideo()
		HelpableScreen.showHelp(self)
#		LOG('YampScreen: showHelp: before call HelpableScreen.showHelp', 'err')
#		try:
#			pass
#			self.session.openWithCallback(self.helpFinished, HelpableScreen)
#		except:	
#			LOG('YampScreen: showHelp: openWithCallback: EXCEPT', 'err')
#		LOG('YampScreen: showHelp: after call HelpableScreen.showHelp', 'err')
#		self.startScreenSaverTimer()		#!!!

#	def helpFinished(self):
#		LOG('YampScreen: helpFinished', 'err')
		
	def nextEntry(self):
		LOG('\nYampScreen: nextEntry: Start: specialScreenActive(): %d' %(self.specialScreenActive()), 'all')
		#reset SS background setting
		if config.plugins.yampmusicplayer.resetScreenSaverBg.value:
			self.ssBackground = config.plugins.yampmusicplayer.screenSaverBg.value
			self.setScreensaverBg()
			LOG('YampScreen: nextEntry: reset Screensaver Background to initial setting: %d' %(self.ssBackground), 'all')

		actual = self.playlist.getCurrentIndex()
		next = actual + 1
		if next < len(self.playlist):													#normal next song
###			LOG('YampScreen: nextEntry: call changentry 1', 'spe')
			self.changeEntry(next)
		elif ( len(self.playlist) > 0 ) and self.repeat:			#last song and repeat: jump to first	
###			LOG('YampScreen: nextEntry: call changentry 2', 'spe')
			self.changeEntry(0)
		elif ( len(self.playlist) == actual + 1 ) and not self.repeat:	#last song, no repeat
			self.stopEntry()
		else:
			pass
#			self.hideScreenSaver()
#			LOG('YampScreen: nextEntry: hide Screensaver' , 'spe')
#		if self.currentIsVideo:
#			pass
#		else:	
#			if not self.screenSaverActive: self.resetScreenSaverTimer()
		
	def previousEntry(self):
		LOG('\nYampScreen: previousEntry: Start: specialScreenActive(): %d' %(self.specialScreenActive()), 'all')
		#reset SS background setting
		if config.plugins.yampmusicplayer.resetScreenSaverBg.value:
			self.ssBackground = config.plugins.yampmusicplayer.screenSaverBg.value
			self.setScreensaverBg()
			LOG('YampScreen: previousEntry: reset Screensaver Background to initial setting: %d' %(self.ssBackground), 'all')

		next = self.playlist.getCurrentIndex() - 1
		if next >= 0:
###			LOG('YampScreen: previousEntry: call changentry 1', 'spe')
			self.changeEntry(next)
		elif self.repeat:
###			LOG('YampScreen: previousEntry: call changentry 2', 'spe')
			self.changeEntry(len(self.playlist)-1)
		else:
			pass
#			self.resetScreenSaverTimer()  # show player for some seconds
#		if self.currentIsVideo: 
#			pass
#		else:	
#			if not self.screenSaverActive: self.resetScreenSaverTimer()

	def pagedown(self):
		if self.screenSaverActive:
			self.nextSlide()
			LOG('YampScreen: pagedown: nextSlide:', 'all')
		else:
			self[self.currList].pageDown()
			self.resetScreenSaverTimer()
			self.updateLCDInfo()

	#if not in standard V265 key mode
	def pagedownspecial(self):
#		if not self.screenSaverActive:
		if not self.screenSaverActive and not self.isVideoFullScreen:
			self[self.currList].pageDown()
			self.updateLCDInfo()
		if not self.currentIsVideo or self.videoStartMode != "manual":
			self.resetScreenSaverTimer()	
		self.showYampScreenHideVideo()
			
	def pageup(self):
		if self.screenSaverActive:
			self.prevSlide()
		else:
			self[self.currList].pageUp()
			self.resetScreenSaverTimer()
			self.updateLCDInfo()
		
	#if not in standard V265 key mode
	def pageupspecial(self):
#		if not self.screenSaverActive:
		if not self.screenSaverActive and not self.isVideoFullScreen:
			self[self.currList].pageUp()
			self.updateLCDInfo()
		if not self.currentIsVideo or self.videoStartMode != "manual":
			self.resetScreenSaverTimer()	
		self.showYampScreenHideVideo()

	def moveup(self):
#		if self.config.plugins.yampmusicplayer.screenSaverLeft.value: 
#			self.resetScreenSaverTimer()
#			if self.playerState < STATE_STOP and self.manKeyScreensaver and (
#						self.currList == "playlist" or config.plugins.yampmusicplayer.screenSaverLeft.value):
###		LOG('YampScreen: moveup: screenSaverActive: %d isVideoFullScreen: %d' %(self.screenSaverActive, self.isVideoFullScreen), 'spe')
		if not self.screenSaverActive and not self.isVideoFullScreen:
			self[self.currList].up()
			self.updateLCDInfo()
		if not self.currentIsVideo or self.videoStartMode != "manual":
###			LOG('YampScreen: moveup: resetScreenSaverTimer' , 'spe')
			self.resetScreenSaverTimer()	
		self.showYampScreenHideVideo()

	def movedown(self):
		if not self.screenSaverActive and not self.isVideoFullScreen:
			self[self.currList].down()
			self.updateLCDInfo()
		if not self.currentIsVideo or self.videoStartMode != "manual":
			self.resetScreenSaverTimer()	
		self.showYampScreenHideVideo()

	def skipToListbegin(self):
		if not self.isVideoFullScreen:
			if not self.screenSaverActive:
				self[self.currList].moveToIndex(0)
				self.updateLCDInfo()
			self.resetScreenSaverTimer()

	def skipToListend(self):
		if not self.isVideoFullScreen:
			if not self.screenSaverActive:
				self[self.currList].moveToIndex(len(self[self.currList]) - 1)
				self.updateLCDInfo()
			self.resetScreenSaverTimer()
		
	def blueActions(self):
		if self.blueLongActive:
			self.blueLongActive = False
		else:
			if not self.screenSaverActive and not self.isVideoFullScreen:
				self.deleteEntry()
			self.resetScreenSaverTimer()
			
	def blueLongActions(self):
		self.blueLongActive = True
		
		self.resetScreenSaverTimer()
		if not self.screenSaverActive and not self.isVideoFullScreen:
			self.hideScreenSaver()
			self.session.openWithCallback(self.clearPlaylistConfirmed, MessageBox, _("Do you really want to delete the current playlist?"), default = False, timeout = 10)
#			self.session.openWithCallback(self.clearPlaylistConfirmed, MessageBox, _("Do you really want to delete the current playlist?"), default = False, timeout = 10, title = _('Clear current playlist'))
#			self.clearPlaylist()

	def clearPlaylistConfirmed(self,answer):
#		LOG('clearPlaylistConfirmed: answer; %d' %(answer), 'err')
		self.resetScreenSaverTimer()
		if answer: 
			self.clearPlaylist()
		
#
# Add and Insert methods from FileList into PlayList
#
	def addFile(self):
		f = self.filelist.getFilename()
		if f.lower().endswith(".m3u") or f.lower().endswith(".pls") or f.lower().endswith(".e2pls"):
			self.addExtPlaylist(f)
		else:
			self.addService(self.filelist.getServiceRef())
			self.playlist.updateList()
#			LOG('YampScreen: addFile: call updateNextSong()', 'err')
			self.updateNextSong()
			
	def addAllFiles(self):
		for x in self.filelist.getFileList():
			if x[0][1] == False: # ordinary file
				f = x[0][0].getPath()
				if f.lower().endswith(".m3u") or f.lower().endswith(".pls") or f.lower().endswith(".e2pls"):
					pass # don't copy playlists
				else:
					self.addService(x[0][0])
		self.playlist.updateList()
		self.updateNextSong()

	def addDir(self, dir, recursive = True):
		filelist = YampFileList(dir, matchingPattern = "(?i)^.*\.(mp2|mp3|ogg|ts|wav|m3u|pls|e2pls|mpg|vob|avi|divx|m4v|mkv|mp4|m4a|dat|flac|mov|m2ts)", useServiceRef = True, showMountpoints = False, isTop = True)
		for x in filelist.getFileList():
			if x[0][1] == True: #isDir
				if recursive:
					if x[0][0] != dir:
						self.addDir(x[0][0])
			else:
				self.addService(x[0][0])
		self.playlist.updateList()
		self.updateNextSong()
	
	def addExtPlaylist(self, pl):
		ext = os.path.splitext(pl)[1]
		if self.playlistparsers.has_key(ext):
			playlistParser = self.playlistparsers[ext]()
			list = playlistParser.open(pl)
			if list:
				for x in list:
					self.playlist.addService(x.ref)  # No layout change for playlists
				self.playlist.updateList()
				if ext == ".e2pls":
					self.extPlaylistName = os.path.splitext(os.path.basename(pl))[0]
				
	def insertFile(self):
		f = self.filelist.getFilename()
		ext = os.path.splitext(f)[1]
		if ext in (".m3u", ".pls", ".e2pls"):
			self.insertExtPlaylist(f)
		else:
			self.insertService(self.playlist.getCurrentIndex()+1, self.filelist.getServiceRef())
			self.playlist.updateList()
			self.updateNextSong()

	def insertDir(self, dir):
		filelist = YampFileList(dir, matchingPattern = "(?i)^.*\.(mp2|mp3|ogg|ts|wav|m3u|pls|e2pls|mpg|vob|avi|divx|m4v|mkv|mp4|m4a|dat|flac|mov|m2ts)", useServiceRef = True, showMountpoints = False, isTop = True)
		for x in reversed(filelist.getFileList()):
			if x[0][1] == True: #isDir
				pass # not recursive
			else:
				self.insertService(self.playlist.getCurrentIndex()+1, x[0][0])
		self.playlist.updateList()
		self.updateNextSong()
	
	def insertExtPlaylist(self, pl):
		ext = os.path.splitext(pl)[1]
		if self.playlistparsers.has_key(ext):
			playlistParser = self.playlistparsers[ext]()
			list = playlistParser.open(pl)
			if list:
				for x in list:
					self.insertService(self.playlist.getCurrentIndex()+1, x.ref)
				self.playlist.updateList()
				self.updateNextSong()
				if ext == ".e2pls":
					self.extPlaylistName = os.path.splitext(os.path.basename(pl))[0]

#
# Add and Insert methods from DatabaseList into PlayList
#
	def addCategory(self, query, mode = "append", pos = 0):
		con, result = ConnectDatabase()
		mypos = 0
		addMode = mode
		if addMode == "insert":
			mypos = pos - 1
		if con is not None:
			con.text_factory = str
			cursor = con.cursor()
			mode = self.dblist.mode

			if mode in (ARTISTLIST, SEARCHARTISTLIST):
				if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
					cursor.execute("SELECT min(title_id), sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id WHERE Titles.artist_id = %d GROUP BY upper(title);" % (query))
				else:
					cursor.execute("SELECT title_id, sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.artist_id = %d ORDER BY upper(title), upper(Albums.album);" % (query))

			elif mode in (ALBUMLIST, ARTISTALBUMLIST, GENREALBUMLIST, SEARCHALBUMLIST, DATEALBUMLIST):
				cursor.execute("SELECT title_id, sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.album_id = %d ORDER BY tracknr, filename;" % (query))
				
			elif mode in (GENRELIST, SEARCHGENRELIST):
				if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
					cursor.execute("SELECT min(title_id), sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id WHERE Titles.genre_id = %d GROUP BY upper(title);" % (query))
				else:
					cursor.execute("SELECT title_id, sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.genre_id = %d ORDER BY upper(title), upper(Albums.album);" % (query))

			elif mode in (DATELIST, SEARCHDATELIST):
				if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
					cursor.execute("SELECT min(title_id), sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id WHERE Titles.date_id = %d GROUP BY upper(title);" % (query))
				else:
					cursor.execute("SELECT title_id, sref, title, Artists.artist FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.date_id = %d ORDER BY upper(title), upper(Albums.album);" % (query))

			for row in cursor:
				sref = ServiceReference(row[1])
				if addMode == "append":
					self.addService(sref.ref, row[2], row[3])
				else:
					mypos += 1
					self.insertService(mypos, sref.ref, row[2], row[3])
			self.playlist.updateList()
			self.updateNextSong()
		cursor.close()  
		con.close()

#
# Add and Insert helper methods
#
	def addService(self, ref, title="", artist=""):
		self.applyLayout(ref, title, artist)
		self.playlist.addService(ref)
		
	def insertService(self, index, ref, title="", artist=""):
		self.applyLayout(ref, title, artist)
		self.playlist.insertService(index, ref)
		
	def applyLayout(self, sref, title, artist):
		path = sref.getPath()
		if config.plugins.yampmusicplayer.playlistLayout.value.startswith("fn"):
			path = os.path.basename(path)
			if config.plugins.yampmusicplayer.playlistLayout.value == "fn":		
				path = os.path.splitext(path)[0]
			sref.setName(path)
		elif config.plugins.yampmusicplayer.playlistLayout.value.startswith("tit"):
			if title == "" or artist == "":
				title, album, genre, artist, date, length, tracknr, strBitrate = readID3Infos(path)
			if config.plugins.yampmusicplayer.playlistLayout.value == "tit":
				name = title
			elif config.plugins.yampmusicplayer.playlistLayout.value == "titart":
				name = title + " - " + artist
			else:
				name = artist + " - " + title
			if name == "":
				name = os.path.splitext(os.path.basename(path))[0]
			sref.setName(name)
		else:  # for now no other layout options
			pass

#
# dblist actions
# For navigation, the dblist maintains its own stack
#
	def dblistActions(self):
		try:
			navx = self.dblist.getSelection().nav
		except:
			navx = False
		if navx:
#		if self.dblist.getSelection().nav:
			entry = self.popDblistStack()
			self.buildDbMenuList(mode = entry.mode, query = entry.query, menutitle = entry.menutitle, queryString = entry.queryString)
			self.dblist.moveToIndex(entry.selIndex)
		else:
			numChoice = self.dblist.getSelectionIndex()
			mode = self.dblist.mode
			if mode == MENULIST:
				self.pushDblistStack (numChoice)
				if numChoice == 0:
					self.buildDbMenuList(mode = PLAYLISTLIST, menutitle = _("E x t .  P l a y l i s t s"))
				elif numChoice == 1:
					self.buildDbMenuList(mode = ARTISTLIST, menutitle = _("A r t i s t s"))
				elif numChoice == 2:
					self.buildDbMenuList(mode = ALBUMLIST, menutitle = _("A l b u m s"))
				elif numChoice == 3:
					self.buildDbMenuList(mode = TITLELIST, menutitle = _("T i t l e s"))
				elif numChoice == 4:
					self.buildDbMenuList(mode = GENRELIST, menutitle = _("G e n r e s"))
				elif numChoice == 5:
					self.buildDbMenuList(mode = DATELIST, menutitle = _("D a t e s"))
			elif mode in (ARTISTLIST, SEARCHARTISTLIST):
				self.pushDblistStack (numChoice)
				choice = self.dblist.getSelection()
				self.buildDbMenuList (mode = ARTISTTITLELIST, query = choice.artistID, menutitle = choice.artist)
			elif mode in (ALBUMLIST, ARTISTALBUMLIST, GENREALBUMLIST, SEARCHALBUMLIST):
				self.pushDblistStack (numChoice)
				choice = self.dblist.getSelection()
				self.buildDbMenuList (mode = ALBUMTITLELIST, query = choice.albumID, menutitle = choice.album)
			elif mode in (GENRELIST, SEARCHGENRELIST):
				self.pushDblistStack (numChoice)
				choice = self.dblist.getSelection()
				self.buildDbMenuList (mode = GENRETITLELIST, query = choice.genreID, menutitle = choice.genre)
			elif mode in (DATELIST, SEARCHDATELIST):
				self.pushDblistStack (numChoice)
				choice = self.dblist.getSelection()
				self.buildDbMenuList (mode = DATETITLELIST, query = choice.dateID, menutitle = choice.date)
			elif mode in (PLAYLISTLIST, SEARCHPLAYLISTLIST):
				self.addExtPlaylist(self.dblist.getSelection().filename)
			elif mode >= TITLELIST:
				sref = ServiceReference(self.dblist.getSelection().ref)
				self.addService(sref.ref, self.dblist.getSelection().title, self.dblist.getSelection().artist)
				self.playlist.updateList()
#				LOG('YampScreen: dbListAction: call updateNextSong()', 'err')
				self.updateNextSong()
				self.movedown()
				self.playlist.moveToIndex(len(self.playlist)-1)
	
	def startupChecks(self):
		self.checkDatabaseConnection()
		dbUpgradeV31()
		self.updateLCDInfo()
	
	def checkDatabaseConnection(self):
		dbPath = os.path.join(config.plugins.yampmusicplayer.databasePath.value ,"yamp.db")
		con, result = ConnectDatabase()
		if result == 1: self.session.open(MessageBox, _('Cannot open database file %s.') %(dbPath), type = MessageBox.TYPE_ERROR, timeout = 15 )
		elif result == 2: self.session.open(MessageBox, _('Cannot write to database %s. Maybe Read-Only?') %(dbPath), type = MessageBox.TYPE_ERROR, timeout = 15 )
#		if result == 1: self.session.open(MessageBox, _('Cannot open database file %s.') %(dbPath), type = MessageBox.TYPE_ERROR, title = _('Yamp connection to database'), timeout = 15 )
#		elif result == 2: self.session.open(MessageBox, _('Cannot write to database %s. Maybe Read-Only?') %(dbPath), type = MessageBox.TYPE_ERROR, title = _('Yamp connection to database'), timeout = 15 )
	
	def buildDbMenuList(self, mode=None, query= -1, menutitle="", queryString=""):
		con, result = ConnectDatabase()
#		try: LOG('YampScreen: buildDbMenuList: ConnectDatabase: con: %s ' %(con), 'spe')
#		except:	LOG('YampScreen: buildDbMenuList: ConnectDatabase : EXCEPT' , 'err')
		if con is not None:
#			LOG('YampScreen: buildDbMenuList Start: mode: %d query: %d  menutitle: %s' %(mode, query, menutitle), 'err')
			con.text_factory = str
			cursor = con.cursor()
			if mode is not None:
				self.dblist.setMode(mode)
				if mode >= TITLELIST:
					self.dblist.setLines(2)
				else:
					self.dblist.setLines(1)
			self.dblist.setTitle(menutitle)
			self.dblist.list = []
			if self.dblist.mode == MENULIST:  # Main Menu
				# Playlists
				cursor.execute("SELECT COUNT (*) FROM Playlists;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Playlists (%d)") % row[0]))
				# Artists
				cursor.execute("SELECT COUNT (*) FROM Artists;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Artists (%d)") % row[0]))
				# Albums
				cursor.execute("SELECT COUNT (*) FROM Albums;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Albums (%d)") % row[0]))
				# Titles
				cursor.execute("SELECT COUNT (*) FROM Titles;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Titles (%d)") % row[0]))
				# Genres
				cursor.execute("SELECT COUNT (*) FROM Genres;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Genres (%d)") % row[0]))
				# Dates
				cursor.execute("SELECT COUNT (*) FROM Dates;")
				row = cursor.fetchone()
				self.dblist.append(DblistEntryComponent(text = _("Dates (%d)") % row[0]))
					
				self.dblist.updateList()

			else:	# Lists
				self.dblist.append(DblistEntryComponent(text = _("[back]"), nav=True))
				if self.dblist.mode == PLAYLISTLIST:
					cursor.execute("SELECT playlist_id, playlist_title, playlist_filename FROM Playlists ORDER BY upper(playlist_title);")
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], text = row[1], title = row[1], filename = row[2]))
				elif self.dblist.mode == SEARCHPLAYLISTLIST:
					cursor.execute("SELECT playlist_id, playlist_title, playlist_filename FROM Playlists WHERE playlist_filename LIKE '%s' ORDER BY upper(playlist_title);" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], text = row[1], title = row[1], filename = row[2]))
				elif self.dblist.mode == ARTISTLIST:
					if len(self.dbArtistList) == 0:
						cursor.execute("SELECT Artists.artist_id, Artists.artist, COUNT (*) FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id GROUP BY Titles.artist_id ORDER BY upper(Artists.artist);")
						for row in cursor:
							self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), artistID = row[0], artist = row[1]))
						self.dbArtistList = self.dblist.list[:]	# cache artist list
					else:
						self.dblist.list = self.dbArtistList[:]
				elif self.dblist.mode == SEARCHARTISTLIST:
					cursor.execute("SELECT Artists.artist_id, Artists.artist, COUNT (*) FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id WHERE Artists.artist LIKE '%s' GROUP BY Titles.artist_id ORDER BY upper(Artists.artist);" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), artistID = row[0], artist = row[1]))
				elif self.dblist.mode == ALBUMLIST:
					if len(self.dbAlbumList) == 0:
						cursor.execute("SELECT Albums.album_id, Albums.album, COUNT (*) FROM Titles INNER JOIN Albums ON Titles.album_id = Albums.album_id GROUP BY Titles.album_id ORDER BY Albums.album;")
						for row in cursor:
							self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), albumID = row[0], album = row[1]))
						self.dbAlbumList = self.dblist.list[:]	# cache album list
					else:
						self.dblist.list = self.dbAlbumList[:]
				elif self.dblist.mode == SEARCHALBUMLIST:
					cursor.execute("SELECT Albums.album_id, Albums.album, COUNT (*) FROM Titles INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Albums.album LIKE '%s' GROUP BY Titles.album_id ORDER BY Albums.album;" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), albumID = row[0], album = row[1]))
				elif self.dblist.mode == ARTISTALBUMLIST:
					cursor.execute("SELECT Albums.album_id, Albums.album FROM Titles INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.artist_id = %d GROUP BY Titles.album_id ORDER BY upper(Albums.album);" % (query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s" % (row[1]), albumID = row[0], album = row[1]))
				elif self.dblist.mode == GENREALBUMLIST:
					cursor.execute("SELECT Albums.album_id, Albums.album FROM Titles INNER JOIN Albums ON Titles.album_id = Albums.album_id WHERE Titles.genre_id = %d GROUP BY Titles.album_id ORDER BY upper(Albums.album);" % (query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s" % (row[1]), albumID = row[0], album = row[1]))
				elif self.dblist.mode == GENRELIST:
					cursor.execute("SELECT Genres.genre_id,Genres.genre, COUNT (*) FROM Titles INNER JOIN Genres ON Titles.genre_id = Genres.genre_id GROUP BY Titles.genre_id ORDER BY Genres.genre;")
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), genreID = row[0], genre = row[1]))
				elif self.dblist.mode == SEARCHGENRELIST:
					cursor.execute("SELECT Genres.genre_id,Genres.genre, COUNT (*) FROM Titles INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Genres.genre LIKE '%s' GROUP BY Titles.genre_id ORDER BY Genres.genre;" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), genreID = row[0], genre = row[1]))
				elif self.dblist.mode == DATELIST:
					try:
						cursor.execute("SELECT Dates.date_id,Dates.date, COUNT (*) FROM Titles INNER JOIN Dates ON Titles.date_id = Dates.date_id GROUP BY Titles.date_id ORDER BY Dates.date;")
						for row in cursor:
							self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), dateID = row[0], date = row[1]))
					except:
						LOG('YampScreen: buildDbMenuList: EXCEPT NO DATE IN DATABASE', 'err')
						self.session.open(MessageBox, _("No Year in Database, see Documentation"), type = MessageBox.TYPE_INFO,timeout = 20 )
				elif self.dblist.mode == SEARCHDATELIST:
					try:
						cursor.execute("SELECT Dates.date_id,Dates.date, COUNT (*) FROM Titles INNER JOIN Dates ON Titles.date_id = Dates.date_id WHERE Dates.date LIKE '%s' GROUP BY Titles.date_id ORDER BY Dates.date;" % (queryString))
						for row in cursor:
							self.dblist.append(DblistEntryComponent(text = "%s (%d)" % (row[1], row[2]), genreID = row[0], genre = row[1]))
					except:
						pass
						
				elif self.dblist.mode == TITLELIST:
					if len(self.dbTitleList) == 0:
						cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id ORDER BY upper(title), upper(Albums.album);")
						for row in cursor:
							self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
						self.dbTitleList = self.dblist.list[:]	# cache title list
					else:
						self.dblist.list = self.dbTitleList[:]
				elif self.dblist.mode == ARTISTTITLELIST:
					if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
						cursor.execute("SELECT min(title_id), title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.artist_id = %d GROUP BY upper(title);" % (query))
					else:
						cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.artist_id = %d ORDER BY upper(title), upper(Albums.album);" % (query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == ALBUMTITLELIST:
					cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.album_id = %d ORDER BY tracknr, filename;" % (query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == GENRETITLELIST:
					if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
						cursor.execute("SELECT min(title_id), title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.genre_id = %d GROUP BY upper(title);" % (query))
					else:
						cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.genre_id = %d ORDER BY upper(title), upper(Albums.album);" % (query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == DATETITLELIST:
					if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
						cursor.execute("SELECT min(title_id), title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Titles.date_id, Genres.genre, Dates.date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id INNER JOIN Dates ON Titles.date_id = Dates.date_id WHERE Titles.date_id = %d GROUP BY upper(title);" %(query))
					else:
						cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Titles.date_id, Genres.genre, Dates.date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id INNER JOIN Dates ON Titles.date_id = Dates.date_id WHERE Titles.date_id = %d GROUP BY upper(title);" %(query))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[9], date = row[10], length = row[11], tracknr = row[12], ref = row[13]))
				elif self.dblist.mode == SEARCHTITLELIST:
					cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.title LIKE '%s' ORDER BY upper(title), upper(Albums.album);" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == SEARCHFILELIST:
					cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Titles.filename LIKE '%s' ORDER BY upper(title), upper(Albums.album);" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == SEARCHARTISTTITLELIST:
					if config.plugins.yampmusicplayer.titleOnlyOnceInSelection.value:
						cursor.execute("SELECT min(title_id), title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Artists.artist LIKE '%s' GROUP BY upper(title);" % (queryString))
					else:
						cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Artists.artist LIKE '%s' ORDER BY upper(title), upper(Albums.album);" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				elif self.dblist.mode == SEARCHALBUMTITLELIST:
					cursor.execute("SELECT title_id, title, filename, Titles.artist_id, Artists.artist, Titles.album_id, Albums.album, Titles.genre_id, Genres.genre, date, length, tracknr, sref FROM Titles INNER JOIN Artists ON Titles.artist_id = Artists.artist_id INNER JOIN Albums ON Titles.album_id = Albums.album_id INNER JOIN Genres ON Titles.genre_id = Genres.genre_id WHERE Albums.album LIKE '%s' ORDER BY tracknr, filename;" % (queryString))
					for row in cursor:
						self.dblist.append(DblistEntryComponent(titleID = row[0], title = row[1], filename = row[2], artistID = row[3], artist = row[4], albumID = row[5], album = row[6], genreID = row[7], genre = row[8], date = row[9], length = row[10], tracknr = row[11], ref = row[12]))
				self.dblist.updateList()
				if len(self.dblist) > 1:
					self.dblist.moveToIndex(1)
			cursor.close()  
			con.close()
			self.currDblist = DblistStackEntry(self.dblist.mode, query, self.dblist.getSelectionIndex(), menutitle, queryString) # for refresh after DB changes

	def updateDbMenuList(self):
		self.dbTitleList = []
		self.dbArtistList = []
		self.dbAlbumList = []
		self.buildDbMenuList (mode = self.currDblist.mode, query = self.currDblist.query, menutitle = self.currDblist.menutitle, queryString = self.currDblist.queryString)

	def pushDblistStack(self, sel):
		self.dbStack.append(DblistStackEntry(mode = self.currDblist.mode, query = self.currDblist.query, selIndex = sel, menutitle = self.currDblist.menutitle, queryString = self.currDblist.queryString))

	def popDblistStack(self):
		entry = self.dbStack[-1]
		del self.dbStack[-1]
		return entry
			
#
# Methods for reading external playlists (.m3u, .pls, .e2pls) and writing e2pls playlists
#
	def addPlaylistParser(self, parser, extension):
		self.playlistparsers[extension] = parser

	def loadLastPlaylist(self):
		self.YampParserE2pls = YampParserE2pls()
		list = self.YampParserE2pls.open(resolveFilename(SCOPE_CONFIG, "yamp.e2pls"))
		if list:
			for x in list:
				self.playlist.addService(x.ref)  # No layout change for playlist
			self.playlist.setCurrentPlaying(config.plugins.yampmusicplayer.lastPlaylistIndex.value)
			self.playlist.stopService() # show stop icon
			self.playerState = STATE_STOP # inhibit screen saver
			list = self.YampParserE2pls.open(resolveFilename(SCOPE_CONFIG, "yampsl.e2pls"))
			if list:
				del self.playlist.shadowList[:]
				for x in list:
					self.playlist.addShadowService(x.ref)
			self.playlist.updateList()
#			LOG('YampScreen: loadLastPlaylist: call updateNextSong()', 'err')
#			self.updateNextSong()
			self.playlist.isShuffeled = config.plugins.yampmusicplayer.shuffle.value
			self.setShuffleButton()
			if config.plugins.yampmusicplayer.yampRemote.value == 'default': self.switchToPlayListOrShuffle()
			else: self.switchToPlayList()
			
	def sortPlaylist(self, mode):
		self.playlist.sortList(mode)
		self.setShuffleButton()

	def searchPlaylist(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchPlaylistCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text=self.lastPlaylistSearch)
		
	def searchPlaylistCallback(self, choice):
		if choice is not None:
#			LOG('YampScreen: searchPlaylistCallback: %s' %(choice), 'spe2')
			self.lastPlaylistSearch = choice.lower()
			self.playlist.searchList(self.lastPlaylistSearch)
		self.virtKeyboardActive = False		
	
	def savePlaylist(self, mode):
		self.par1 = mode
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.savePlaylistCallback, VirtualKeyBoard, title=_("YAMP - Please enter filename:"), text=self.extPlaylistName)

	def savePlaylistCallback(self, name):
#		LOG('YampScreen: savePlaylistCallback: Start: Mode: %d' %(self.par1), 'err')
		if name is not None:
			name = name.strip()
			if name == "":
				name = strftime("%y%m%d-%H%M%S")
			self.extPlaylistName = name
#			LOG('YampScreen: savePlaylistCallback: name: %s' %(name), 'err')
			
			# Save e2pls
			if self.par1 == 1 or self.par1 == 3:
				LOG('YampScreen: savePlaylistCallback: e2pls', 'err')
				fname = name + ".e2pls"
#				LOG('YampScreen: savePlaylistCallback: fname: %s' %(fname), 'err')
				self.YampParserE2pls = YampParserE2pls()
				self.YampParserE2pls.clear()
				for x in self.playlist.list:
					self.YampParserE2pls.addService(ServiceReference(x[0]))
				try:	
					nok=self.YampParserE2pls.save(os.path.join(config.plugins.yampmusicplayer.playlistPath.value, fname))
				except:
					nok=5
#				LOG('YampScreen: savePlaylistCallback: nok: %d' %(nok), 'err')
				if nok: self.session.open(MessageBox, _('Playlist saving as %s failed') %(fname), type = MessageBox.TYPE_INFO,timeout = 7 )
				else: self.session.open(MessageBox, _('Playlist saved as %s') %(fname), type = MessageBox.TYPE_INFO,timeout = 7 )
				
			# Save m3u
			if self.par1 == 2 or self.par1 == 3:
#				LOG('YampScreen: savePlaylistCallback: m3u', 'err')
				fname = name + ".m3u"
				try:
					self.YampParserM3u = YampParserM3u()
#					LOG('YampScreen: savePlaylistCallback: fname: %s' %(fname), 'err')
				except:
					LOG('YampScreen: savePlaylistCallback: self.YampParserM3u: EXCEPT', 'err')
				try:
					self.YampParserM3u.clear()
				except:
					LOG('YampScreen: savePlaylistCallback: clear: EXCEPT', 'err')
				try:
					for x in self.playlist.list:
						self.YampParserM3u.addService(ServiceReference(x[0]))
				except:
					LOG('YampScreen: savePlaylistCallback: for x : EXCEPT', 'err')
				try:	
					nok=self.YampParserM3u.save(os.path.join(config.plugins.yampmusicplayer.playlistPath.value, fname))
				except:
					nok=5
				if nok: self.session.open(MessageBox, _('Playlist saving as %s failed') %(fname), type = MessageBox.TYPE_INFO,timeout = 7 )
				else: self.session.open(MessageBox, _('Playlist saved as %s') %(fname), type = MessageBox.TYPE_INFO,timeout = 7 )
			self.filelist.refresh()
		self.virtKeyboardActive	= False			
		
	def deletePlaylist(self):
		playlist = self.filelist.getFilename()
		if os.path.exists(playlist):
			os.remove(playlist)
			self.filelist.refresh()

#
# More methods for playlists and playlist entries	
#
	def checkSkipShowHideLock(self):  # method override from InfoBarGenerics
		if self.seekstate == self.SEEK_STATE_PAUSE:
			self.playlist.pauseService()
			self.playerState = STATE_PAUSE
		elif self.seekstate == self.SEEK_STATE_PLAY:
			self.playlist.playService()
			self.playerState = STATE_PLAY

	def doEofInternal(self, playing):
#		LOG('YampScreen: doEofInternal: Start', 'spe')
		if playing:
#			LOG('YampScreen: doEofInternal: nextEntry() !!!', 'spe')
###			LOG('YampScreen: doEofInternal: call nextEntry 1', 'spe')
			self.nextEntry()
		else:
###			LOG('YampScreen: doEofInternal: hide Screensaver' , 'spe')
			self.hideScreenSaver()

	def deleteEntry(self):
		if len(self.playlist) > 0:
			deletedCurrentEntry = False
			if self.currList == "playlist":
				index = self.playlist.getSelectionIndex()
			else:
				index = self.playlist.getCurrentIndex()
			if index == self.playlist.getCurrentIndex():
				self.stopEntry()
				deletedCurrentEntry = True
			self.playlist.deleteService(index)
			self.playlist.updateList()
			self.updateNextSong()
			if len(self.playlist) == 0:
				self.switchToFileListOrRepeat()
			elif deletedCurrentEntry:
###				LOG('YampScreen: deleteEntry: call nextEntry 1', 'spe')
				self.nextEntry()
			self.updateLCDInfo()
			self.updateNextSong()

	def checkIsVideo(self, filePathName):
		try:
#			LOG('YampScreen: checkIsVideo: Start: filename: %s'  %(filePathName), 'spe2') 
			ext = os.path.splitext(filePathName)[1].lower()
#			LOG('YampScreen: checkIsVideo: ext: %s'  %(ext), 'spe2') #neuTest6
			if ext in audioExtensions: return False
			else:
				if  ext == '.mp4':
					pass  #neuTest6
#					if len(mutagen.File(filePathName,easy=True)) > 0 : return False  #neuTest6
#					else: return True #neuTest6
				return True		#neuTest6
		except: LOG('YampScreen: checkIsVideo: EXCEPT', 'err') 
		return False

	def insertBlankVideo(self, index):		
###		LOG('YampScreen: insertBlankVideo: Start: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
		try:
			self.blankVideoInserted = self.playlist.getCurrentIndex()+1
#			LOG('YampScreen: insertBlankVideo: %d' %(self.blankVideoInserted), 'spe2') 
###			LOG('YampScreen: insertBlankVideo: %d' %(self.blankVideoInserted), 'spe') 
			self.insertService(self.blankVideoInserted, self.filelistint.getServiceRef())
#			self.playlist.updateList()
		except:
			LOG('YampScreen: insertblankvideo: EXCEPT1', 'err')

		try:
#			self.showYampScreenHideVideo()
			currRef = self.playlist.getServiceRefList()[self.blankVideoInserted]
			self.playlist.setCurrentPlaying(self.blankVideoInserted)
			self.session.nav.playService(currRef)
		except:
			LOG('YampScreen: insertblankvideo: EXCEPT5', 'err')
###		LOG('YampScreen: insertBlankVideo: X2: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 

		try:
			self.oldIsVideo = False
			import time
			sleeptime = config.plugins.yampmusicplayer.blankVideoDelay.value / 1000.0
###			LOG('YampScreen: insertBlankVideo: sleeptime: %f' %(sleeptime), 'spe') 
			time.sleep(sleeptime)
###			LOG('YampScreen: insertBlankVideo: sleeptime: %f' %(sleeptime), 'spe') 
#			time.sleep(config.plugins.yampmusicplayer.blankVideoDelay.value / 1000.0)
###			LOG('YampScreen: insertBlankVideo: X33: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 

			config.plugins.yampmusicplayer.displayVideoPreview.value = False
			config.plugins.yampmusicplayer.displayVideoPreview.save()
			self["noVideoLabelHide"].show()
			
			self.playlist.stopService()
#			self.playerState = STATE_STOP
			self.session.nav.stopService()

#			self.stopEntry()
			self.playlist.deleteService(self.blankVideoInserted)
###			LOG('YampScreen: insertBlankVideo: End: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
###			LOG('YampScreen: insertBlankVideo: call changentry 1', 'spe')
			self.changeEntry(index)
		except:
			LOG('YampScreen: insertblankvideo: EXCEPT8', 'err')
	
	def changeEntry(self, index): 
#		LOG('\nYampScreen: changeEntry: Start: index: %d  len: %d' %(index, len(self.playlist.getServiceRefList())), 'all') 
###		LOG('\nYampScreen: changeEntry: Start: index: %d  blankinserted: %d currentIsVideo: %d  oldIsVideo: %d' %(index, self.blankVideoInserted, self.currentIsVideo, self.oldIsVideo), 'spe') 
#		LOG('\nYampScreen: changeEntry: Start: index: %d screenSaverActive: %d screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
		try:
			newRef = self.playlist.getServiceRefList()[index]
			newIsVideo = self.checkIsVideo(newRef.getPath())
###			LOG('YampScreen: changeEntry: newIsVideo: %d  newPath: %s' %(newIsVideo, newRef.getPath()), 'spe')
		except:
			LOG('YampScreen: changeEntry: newIsVideo: EXCEPT', 'err') 
			
		if config.plugins.yampmusicplayer.insertBlankVideo.value and not newIsVideo and self.oldIsVideo and (self.blankVideoInserted == -1):
			self.insertBlankVideo(index)
		else:
			if newIsVideo and not self.specialScreenActive(): 
				#Video Start Mode full or preview
###				LOG('YampScreen: changeEntry: newIsVideo: videoStartMode: %s' %(self.videoStartMode), 'spe')
				self.currentIsVideo = True
				if self.videoStartMode == "manual": 
###					LOG('YampScreen: changeEntry: VideoStartMode manual1: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
					self.resetScreenSaverTimer()	#hide SS and Start SS Timer
					self.screenSaverTimer.stop()
					self.setSaverButton()
###					LOG('YampScreen: changeEntry: VideoStartMode manual2: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
					if self.isVideoFullScreen: self.hideYampScreenShowVideo() # display Title/Lyrics-Screen
				elif self.videoStartMode == "immediate": 
###					LOG('YampScreen: changeEntry: VideoStartMode immediate: hideYampScreenShowVideo: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
					self.hideYampScreenShowVideo()
					if not self.screenSaverActive: self.startScreenSaverTimer()  #self.resetScreenSaverTimer()
				else: #screensaver
#					LOG('YampScreen: changeEntry: VideoStartMode screensaver: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
					if self.screenSaverActive: 
###						LOG('YampScreen: changeEntry: VideoStartMode else: hideYampScreenShowVideo: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
						self.hideYampScreenShowVideo()
					else:	self.resetScreenSaverTimer() 
###					LOG('YampScreen: changeEntry: VideoStartMode screensaver: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
			else: 
				#Music, no Video
				if not self.specialScreenActive():
					self.showYampScreenHideVideo()
###					LOG('\nYampScreen: changeEntry: newIsNOTVideo: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
					if not self.screenSaverActive: self.resetScreenSaverTimer()
###					LOG('YampScreen: changeEntry: newIsNOTVideo2: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
###			LOG('YampScreen: changeEntry: End: screenSaverActive: %d screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe')
			self.changeEntryNormal(index)

	def changeEntryNormal(self, index):
###		LOG('YampScreen: changeEntryNormal: Start: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
		samePlaying = False
		try:
			self.blankVideoInserted = -1
			currentPlaying = self.session.nav.getCurrentlyPlayingServiceReference()
			nextPlaying = self.playlist.getServiceRefList()[index]
#			LOG('YampScreen: changeEntryNormal: currentPlaying: %s nextPlaying: %s' %(currentPlaying,nextPlaying), 'all') 
		except:
			pass
#			LOG('YampScreen: changeEntryNormal: currentPlaying: EXCEPT', 'all')
		try:
			samePlaying = currentPlaying == nextPlaying
#			LOG('YampScreen: changeEntryNormal: samePlaying: %d' %(samePlaying), 'all') 
		except:
			pass
#			LOG('YampScreen: changeEntryNormal: samePlaying: %d : EXCEPT' %(samePlaying), 'all')
		self.playlist.setCurrentPlaying(index)
		if len(self.playlist.getServiceRefList()) == 1 or samePlaying:
			self.playlist.stopService()
			self.playerState = STATE_STOP			#check!!!
			self.session.nav.stopService()
			LOG('YampScreen: changeEntryNormal: Len = 1 or samePlaying: Stop', 'all') 
			if not self.repeat and self.session.nav.getCurrentlyPlayingServiceReference() is not None: 
				LOG('YampScreen: changeEntryNormal: Len = 1: Stop complete', 'all')
				self.stopEntry()
				self.currentIsVideo = False
				self.setVideoPreviewOnOff()
				return
		
		if len(self.playlist.getServiceRefList()):
###			LOG('YampScreen: changeEntryNormal: X1: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
			currRef = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()]
			self.session.nav.playService(currRef)
			self.setSeekState(self.SEEK_STATE_PLAY)
			self.playerState = STATE_PLAY
			self.currentIsVideo = self.checkIsVideo(currRef.getPath())
			self.setVideoPreviewOnOff()
#			LOG('YampScreen: changeEntryNormal: currentIsVideo: %d file: %s ' %(self.currentIsVideo,currRef.getPath()), 'spe2') 

			if self.eofbugTimer.isActive():
				self.eofbugTimer.stop()
			self.playPosition = -1
			self.playLength = -1
			self.eofbugTimer.start(1000)
###			LOG('YampScreen: changeEntryNormal: End: index: %d self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(index, self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 

	def showYampScreenHideVideo(self):		
		self.show()
		self.isVideoFullScreen = False

	def hideYampScreenShowVideo(self):		#show Video		
		self.hide()
		self.isVideoFullScreen = True
		autoStartTime =config.plugins.yampmusicplayer.showTimeVideoTitle.value
#			LOG('YampScreen: hideYampScreenShowVideo: start Videotitle with autostart: time: %d coverArtFile: *%s*' %(autoStartTime,self["coverArt"].getFileName()), 'spe') 
		if self.lyricsLineShow or self.lyricsLineShowBig:
			if autoStartTime > 0:
				self.session.open(YampVideoTitleV32, self, autoStartTime=autoStartTime)
###				LOG('YampScreen: hideYampScreenShowVideo: openVideoTitle', 'spe') 
			else:	
				self.session.open(YampVideoLyricsV32, self)
###				LOG('YampScreen: hideYampScreenShowVideo: openVideoLyrics', 'spe') 
		else:
			if autoStartTime > 0:
###				LOG('YampScreen: hideYampScreenShowVideo: openVideoTitle', 'spe') 
				self.session.open(YampVideoTitleV32, self, autoStartTime=autoStartTime)
			
#		else:
#			LOG('YampScreen: hideYampScreenShowVideo: NO start Videotitle with autostart: time: %d coverArtFile: *%s*' %(autoStartTime,self["coverArt"].getFileName()), 'spe') 
			
	def setVideoPreviewOnOff(self):
		if self.currentIsVideo:
			config.plugins.yampmusicplayer.displayVideoPreview.value = True
#			LOG('\nYampScreen: setVideoPreviewOnOff: True: IsVideo: %d VideoPreview: %d' %(self.currentIsVideo,config.plugins.yampmusicplayer.displayVideoPreview.value), 'err')
			self["noVideoLabelHide"].hide()
		else:	
			config.plugins.yampmusicplayer.displayVideoPreview.value = False
#			LOG('\nYampScreen: setVideoPreviewOnOff: False: IsVideo: %d VideoPreview: %d' %(self.currentIsVideo,config.plugins.yampmusicplayer.displayVideoPreview.value), 'err')
			self["noVideoLabelHide"].show()
		config.plugins.yampmusicplayer.displayVideoPreview.save()

	def stopEntry(self):
###		LOG('YampScreen: stopEntry: Start: self.screenSaverActive: %d self.screenSaverTimer.isActive: %d' %(self.screenSaverActive, self.screenSaverTimer.isActive()), 'spe') 
		config.plugins.yampmusicplayer.displayVideoPreview.value = False
		config.plugins.yampmusicplayer.displayVideoPreview.save()
		self["noVideoLabelHide"].show()
		self.playlist.stopService()
		self.playerState = STATE_STOP
		self.session.nav.stopService()
		self.updateMusicInformation(clear = True)
		self.currPath = ""
###		LOG('YampScreen: stopEntry: hidescreensaver' , 'spe') 
		self.hideScreenSaver()
		self.showYampScreenHideVideo()
		
	def clearPlaylist(self):
		if len(self.playlist) > 0:
			self.stopEntry()
			self.playlist.clear()
			self.setShuffleButton()
			self.switchToFileListOrRepeat()
			self.updateLCDInfo()
			self.updateNextSong()

	def checkEOF(self):
#		LOG('YampScreen: checkEOF: Start : Player State: %d:' %(self.playerState), 'spe') 
		gap = config.plugins.yampmusicplayer.gapCorrection.value
		if self.playerState == STATE_PLAY:
			len, pos = self.getSeekData()
			self.playLength = len
#			LOG('YampScreen: checkEOF: playpos: %d  pos: %d  len: %d' %(self.playPosition, pos, len), 'spe')
			if self.playPosition == pos:
###				LOG('YampScreen: checkEOF: call nextEntry 1', 'spe')
				self.nextEntry()
#				if not self.currentIsVideo: self.nextEntry()
			else:
				self.playPosition = pos
#				LOG('YampScreen: checkEOF: pos: %d ' %(pos), 'spe')
				if len and pos and gap: # and not self.currentIsVideo:
					if (len - pos) < 100000: # 90000 = 1 second -> poll for the last 1.1 seconds
						while(len - pos) > gap * 900:
							len, pos = self.getSeekData()
###						LOG('YampScreen: checkEOF: call nextEntry 2', 'spe')
						self.nextEntry()
		if self.eofbugTimer.isActive(): self.eofbugTimer.stop()			
		self.eofbugTimer.start(1000)
			
	def getSeekData(self):
		service = self.session.nav.getCurrentService()
#		try:
#			LOG('YampScreen: getSeekData: type(service): %s' %(type(service)), 'spe')
#		except:
#			LOG('YampScreen: getSeekData: EXCEPT' , 'spe')
#		try:
#			self.updatedSeekState()
#		except:
#			LOG('YampScreen: getSeekData: EXCEPT' , 'spe')
		seek = service and service.seek()
		if seek is None:
			return (0, 0)
		len = seek.getLength()
		pos = seek.getPlayPosition()
#		LOG('YampScreen: getSeekData: len: %s pos: %s' %(len,pos), 'err')
#		LOG('YampScreen: getSeekData: len[0]: %d pos[0]: %d' %(len[0],pos[0]), 'err')
#		LOG('YampScreen: getSeekData: len[1]: %d pos[1]: %d' %(len[1],pos[1]), 'err')
		if len[0] or pos[0]:
			return (0, 0)
		return (len[1], pos[1])
		
#
# Yamp Screen Saver
#
	
#	def initScreenSaver(self, artist = '', newArtist = False, initBlank = False):		#!!!Video
	def initScreenSaver(self, artist = ''):
#		LOG('YampScreen: initScreenSaver: Start: artist: %s newArtist: %d initBlank: %d' %(artist,newArtist,initBlank), 'all')
		LOG('YampScreen: initScreenSaver: Start: artist: %s' %(artist), 'all')

		#Blank Screensaver
		if self.currentIsVideo: ###	if initBlank:
			slidePath = yampDir + "images/"
			
		#Standard Screensaver
		elif config.plugins.yampmusicplayer.screenSaverMode.value == "standard":
			slidePath = yampDir + "saver/"

		#Custom Screensaver
		elif config.plugins.yampmusicplayer.screenSaverMode.value == "custom":
			slidePath = config.plugins.yampmusicplayer.screenSaverSlidePath.value
	
		#Artwork Screensaver
		elif config.plugins.yampmusicplayer.screenSaverMode.value == "artwork":
			LOG('YampScreen: initScreenSaver: Artwork: artist %s' %(artist),'all')
			filelist = []
			artworkPath = config.plugins.yampmusicplayer.screenSaverArtworkPath.value
			if artist != "":
				LOG('YampScreen: initScreenSaver: Artwork: artworkPath %s' %(artworkPath),'all')
				slidePath = self.getArtistArtDir(artworkPath,artist)
				LOG('YampScreen: initScreenSaver: Artwork: slidePath %s' %(slidePath),'all')
				try:
					filelist = self.fileListByExt(slidePath, [".jpg", ".png", ".bmp", ".gif"], config.plugins.yampmusicplayer.screenSaverSubDirs.value, 500)
				except:
					LOG('YampScreen: initScreenSaver: filelistArtist: EXCEPT: No artist-art files found', 'all')
			
			#artist name = "" or no pictures found: try default path
			try:
				if len(filelist) == 0: slidePath = artworkPath + 'Default/'
			except:
				LOG('YampScreen: initScreenSaver: len(filelist): EXCEPT', 'err')
				slidePath = artworkPath + 'Default/'

		#Blank Screensaver
		else:	slidePath = yampDir + "images/"

		if not os.path.exists(slidePath):	slidePath = yampDir + "saver/"	# fallback if no images #hier noch evtl. leere liste checken

#		LOG('YampScreen: initScreenSaver: self.slidePath: %s slidePath: %s  newArtist: %d artist: %s' %(self.slidePath,slidePath,newArtist,artist),'all')
		LOG('YampScreen: initScreenSaver: self.slidePath: %s slidePath: %s artist: %s' %(self.slidePath,slidePath,artist),'all')
#		if (self.slidePath != slidePath) or newArtist:
		if (self.slidePath != slidePath):
			try:
#				LOG('YampScreen: initScreenSaver: New slidePath: %s newArtist: %d' %(slidePath, newArtist), 'all')
				LOG('YampScreen: initScreenSaver: New slidePath: %s' %(slidePath), 'all')
			except:
				pass
			self.slidePath = slidePath
			slideList = []
			slideIndex = 0
			
			#Build Filelists and Slidelists
			
			if slidePath == yampDir + "images/":		#blank
				slideList.append(os.path.join(self.slidePath,"black.png"))

			#custom or artistart screensaver 
			else:
				fileList = self.buildFileList()			#scan folders (including subfolders), create filelist
			
				for filename in fileList:
					try:
#						LOG('YampScreen: initScreenSaver: filename: %s' %(filename), 'err')	
						if os.path.splitext(filename)[1].lower() in (".jpg", ".png", ".bmp", ".gif"):
							fullname = os.path.join(self.slidePath,filename)
							if os.path.isfile(fullname) and os.stat(fullname).st_size > 500:
								slideList.append(fullname)
					except:
						LOG('YampScreen: initScreenSaver: filename: EXCEPT'	, 'err')	
				LOG('YampScreen: initScreenSaver: before blacklist: len(slideList): %d slideList: %s' %(len(slideList),slideList), 'all')	
				try:
					slideList = self.checkBlackList(slideList, slidePath)
					LOG('YampScreen: initScreenSaver: after blacklist: len(slideList): %d slideList: %s' %(len(slideList),slideList), 'all')	
				except:
					LOG('YampScreen: initScreenSaver: call blacklist: EXCEPT', 'err')	

			sorting = config.plugins.yampmusicplayer.screenSaverSort.value
			if sorting == 'sortA': slideList.sort()
			elif sorting == 'sortZ': slideList.sort(reverse = True)
			elif sorting == 'shuffle': random.shuffle(slideList)
			ind = 0
			for slide in slideList:
				if slide == config.plugins.yampmusicplayer.lastSlide.value:
					slideIndex = ind - 1
					break
				ind += 1
#				LOG('YampScreen: initScreenSaver: slideIndex: %d' %(slideIndex), 'all')
				
			self["screenSaver"].setList(slideList, slideIndex, showImmediate=True)
#		self["screenSaver"].setList(slideList, slideIndex, showImmediate=newArtist)
			
##		LOG('YampScreen: initScreenSaver: before newArtist: %d\n' %(newArtist), 'all')
##		if newArtist == False:
#			self.resetScreenSaverTimer()
##			self.resetSlideTimer()
##			LOG('YampScreen: initScreenSaver`: End: resetSlideTimer() no new artist\n', 'spe2')
##		else:
##			self.resetSlideTimer()
##			LOG('YampScreen: initScreen End: resetSlideTimer() new Artist\n', 'spe2\n')
####		if initBlank and self.currentIsVideo: self.showScreenSaver()		#immediate start if video direct start 
#checksofort

	def checkBlackList(self,slidelist,basepath):
		try:
#			LOG('YampScreen: checkBlackList: slidelist start: basepath: %s slidelist: %s' %(basepath,slidelist), 'err')	
#			LOG('YampScreen: checkBlackList: file: %s' %(os.path.join(basepath,'blacklist.txt')), 'err')	
			blackFilePath = os.path.join(basepath,'blacklist.txt')
			LOG('YampScreen: checkBlackList: blackFilePath: %s ' %(blackFilePath), 'all')	
			if not os.access(blackFilePath, os.R_OK): 
				LOG('YampScreen: checkBlackList: file does not exist: %s ' %(blackFilePath), 'all')	
				return slidelist
			blackFile = open(blackFilePath, "r")
			for line in blackFile:
				blacklist=line.strip()
				LOG('YampScreen: checkBlackList: blacklist line: %s ' %(blacklist), 'all')	
				if blacklist in slidelist:
					LOG('YampScreen: checkBlackList: blacklisted slide found: %s ' %(blacklist), 'all')
					slidelist.remove(blacklist)
			blackFile.close()
		except:	
			LOG('YampScreen: checkBlackList: EXCEPT', 'err')	
		return slidelist

	#custom or artistart screensaver: scan folders (including subfolders), create filelist
	def buildFileList(self):
		try:
#			LOG('YampScreen: buildFileList: screenSaverSubDirs: %d' %(config.plugins.yampmusicplayer.screenSaverSubDirs.value), 'err')	
			fileList = []
			if config.plugins.yampmusicplayer.screenSaverSubDirs.value:
				try:
					subFolders=[]
					for dirpath, dirnames, files in os.walk(self.slidePath):
						try:
#							LOG('YampScreen: buildFileList: fileList2: dirpath: %s dirnames: %s files: %s' %(dirpath, dirnames, files), 'err')	
							for name in files:
#								LOG('YampScreen: buildFileList: fileList2: dirpath: %s name %s' %(dirpath, name), 'err')	
								fileList.append(os.path.join(dirpath, name))
						except:
							LOG('YampScreen: buildFileList: fileList2: EXCEPT', 'err')	
				except:
					LOG('YampScreen: buildFileList: fileList1: EXCEPT', 'err')	
				LOG('YampScreen: buildFileList: fileList: %s' %(fileList), 'all')	
			else:
				for filename in os.listdir(self.slidePath):
					fileList.append(filename)
				LOG('YampScreen: buildFileList: fileList: %s' %(fileList), 'all')	
		except:
			LOG('YampScreen: buildFileList: fileList complete: EXCEPT', 'err')	
		return fileList
			
	def getArtistArtDir(self,artworkPath,artist):
		LOG('YampScreen: getArtistArtDir: Start: artworkPath: %s artist: %s' %(artworkPath,artist), 'all')
		artist = self.getArtistAlias(artworkPath,artist)
		LOG('YampScreen: getArtistArtDir: artist after alias: %s' %(artist), 'all')
		subDirArtist=""	
		try:
			import re
			subDirArtist = titlecase(re.sub(r'[/\\.]*','',artist).strip())		#delete / \. from artist for directory
		except:
			LOG('YampScreen: getArtistArtDir: subDirArtist: EXCEPT', 'err')
		LOG('YampScreen: getArtistArtDir: subDirArtist: %s' %(subDirArtist), 'all')
		fullname = artworkPath + subDirArtist
		try:
			for subDir in os.listdir(artworkPath):
#				LOG('YampScreen: getArtistArtDir: subDir: %s' %(subDir), 'all')
				if os.path.isdir(artworkPath + subDir):
#					LOG('YampScreen: getArtistArtDir: isdir: subDir: %s' %(subDir), 'err')
					if subDirArtist.lower() == subDir.lower():
						fullname = artworkPath + subDir
						break
				else:
					pass
#					LOG('YampScreen: getArtistArtDir: NODIR: subDir: %s' %(subDir), 'err')
		except:
			LOG('YampScreen: getArtistArtDir: for subDir: EXCEPT', 'err')
		LOG('YampScreen: getArtistArtDir: return dir: %s' %(fullname), 'all')
		return fullname	

	def getArtistAlias(self,artworkPath,artist):
		try:
			aliasfilename=os.path.join(artworkPath, 'YampArtistReplace.txt')
#			LOG('YampScreen: getArtistAlias: aliasfilename: %s' %(aliasfilename), 'err')
		except:
			LOG('YampScreen: getArtistAlias: aliasfilename: EXCEPT', 'err')
			return artist
		if not os.path.exists(aliasfilename) or not os.path.isfile(aliasfilename): return artist 
		LOG('YampScreen: getArtistAlias: aliasfilename: %s existing' %(aliasfilename), 'all')
		artistalias = artist
		try:
			file = open(aliasfilename, "r")
			for line in file:
#				artistreplace = line.strip().split(',').encode("utf-8", 'ignore')
				artistreplace = line.strip().split(',')
				try:
#					LOG('YampScreen: getArtistAlias: artistreplace: *%s*' %(artistreplace), 'err')
					if artistreplace[0].lower() == artist.lower():
#						LOG('YampScreen: getArtistAlias: found: artistreplace: *%s*  artist: *%s*' %(artistreplace[0], artist), 'err')
						if len(artistreplace[1]) > 0:
							artistalias = artistreplace[1]
						break
				except:
					LOG('YampScreen: getArtistAlias: artistreplace: EXCEPT' , 'err')
			file.close()
#			LOG('YampScreen: getArtistAlias: artist: %s  artistalias: *%s*' %(artist, artistalias), 'err')
		except:
			LOG('YampScreen: getArtistAlias: file: EXCEPT: no artistalias-file' , 'all')
		return artistalias

		
	def resetScreenSaverTimer(self):
###		LOG('YampScreen: resetScreenSaverTimer: Start: hideScreenSaver call Startscreensavertimer', 'spe')
		self.hideScreenSaver()
		self.startScreenSaverTimer()
		
	def startScreenSaverTimer(self):
#		LOG('YampScreen:  startScreenSaverTimer: Timer.isActive: %d  screenSaverActive: %d' %(self.screenSaverTimer.isActive(),self.screenSaverActive), 'err')
#		LOG('YampScreen: startScreenSaverTimer: self.currList == "playlist": %d  self.playerState < STATE_STOP: %d' %(self.currList == "playlist",self.playerState < STATE_STOP), 'err')
###		LOG('YampScreen: startScreenSaverTimer: Timer.isActive: %d  screenSaverActive: %d screenSaverManOff: %d' %(self.screenSaverTimer.isActive(),self.screenSaverActive,self.screenSaverManOff), 'spe')
		if self.screenSaverManOff: return
		msecs = config.plugins.yampmusicplayer.screenSaverWait.value * 1000
###		LOG('YampScreen: startScreenSaverTimer: playerstate: %d  currlist: %s	ssleft: %d msecs: %d'  %(self.playerState,self.currList,config.plugins.yampmusicplayer.screenSaverLeft.value,msecs), 'spe')
		if self.screenSaverTimer.isActive() or self.screenSaverActive:
###			LOG('YampScreen: startScreenSaverTimer: pass', 'spe')
			pass
		elif  self.playerState < STATE_STOP and ( self.currList == "playlist" or
						self.currList != "playlist" and config.plugins.yampmusicplayer.screenSaverLeft.value) \
						and msecs > 0:
###			LOG('YampScreen: startScreenSaverTimer: StartTimer: msecs: %d' %(msecs), 'spe')
			self.screenSaverTimer.start(msecs)
###		LOG('YampScreen: startScreenSaverTimer End: Timer.isActive: %d  screenSaverActive: %d screenSaverManOff: %d' %(self.screenSaverTimer.isActive(),self.screenSaverActive,self.screenSaverManOff), 'spe')
		self.setSaverButton()

		
	def hideScreenSaver(self):
###		LOG('YampScreen: hideScreenSaver: Start currentIsVideo: %d' %(self.currentIsVideo), 'spe')
		self.screenSaverActive = False
		self["lyricsLine"].hide()
		self["lyricsLineBackground"].setBoolean(False)
		self["lyricsLineBig"].hide()
		self["lyricsLineBackgroundBig"].setBoolean(False)
		self["screenSaverBackground"].hide()
		self["screenSaver"].hide()
		if self.screenSaverTimer.isActive(): self.screenSaverTimer.stop()
		if self.nextSlideTimer.isActive(): self.nextSlideTimer.stop()
		self.setSaverButton()
		try:
			self["songTextsBackground"].setBoolean(False)
			self["downloadBackground"].setBoolean(False)
		except: 
			LOG('YampScreen: hideScreenSaver: self[songTextsBackground].setBoolean(False): EXCEPT', 'err')
		self.updateLCDInfo()
#		if self.isVideoFullScreen:
#			LOG('YampScreen: hideScreenSaver: call showYampScreenHideVideo !!!', 'err')
#			self.showYampScreenHideVideo()
		
	def screenSaverMaxManOff(self):		#timer-callback
###		LOG('YampScreen: screenSaverMaxManOff Feedback: TimerIsActive: %d' %(self.screenSaverManTimer.isActive()), 'spe')
		self.screenSaverManOff = False
#		self.screenSaverManTimer.stop()	
#		LOG('YampScreen: screenSaverMaxManOff Nach Stop: TimerIsActive: %d' %(self.screenSaverManTimer.isActive()), 'err')
		self.startScreenSaverTimer()
		self.setSaverButton()
		
	def startScreenSaverManTimer(self):
		secs = config.plugins.yampmusicplayer.maxTimeSSoffMan.value - config.plugins.yampmusicplayer.screenSaverWait.value
		if secs < 0: secs = 0
		if secs > 0: 
			self.screenSaverManTimer.start(secs * 1000, True)
			LOG('YampScreen: startScreenSaverManTimer: Start: secs: %d' %(secs), 'all')
		else: 
			self.screenSaverManTimer.stop()	
			LOG('YampScreen: startScreenSaverManTimer: Stop', 'all')
#		try:
#			LOG('YampScreen: startScreenSaverManTimer: isActive: %d' %(self.screenSaverManTimer.isActive()), 'err')
#		except:
#			LOG('YampScreen: startScreenSaverManTimer: isActive: EXCEPT', 'err')
	
	def keyTextActions(self):
		if self.textLongActive:		
			self.textLongActive = False
		else:	
			self.lyricsLineShow = not self.lyricsLineShow
			self.lyricsLineShowBig = False
			self.displayHideLyricsLine()
			if self.isVideoFullScreen:
				try: 
					self.videoLyricsScreenOpen = True
				except: LOG('YampScreen: keyTextActions: openVideoLyricsScreen: EXCEPT', 'err') 
			
	def keyTextLongActions(self):
		self.textLongActive = True
		self.lyricsLineShowBig = not self.lyricsLineShowBig
		self.lyricsLineShow = False
		self.displayHideLyricsLine()
		if self.isVideoFullScreen:
			try: 
				self.videoLyricsScreenOpen = True
			except: LOG('YampScreen: keyTextLongActions: openVideoLyricsScreen: EXCEPT', 'err') 
			
	def displayHideLyricsLine(self):		
		try:
			if not self.lyricsLineShow:
				self["lyricsLine"].hide()
				self["lyricsLineBackground"].setBoolean(False)
			if not self.lyricsLineShowBig:
				self["lyricsLineBig"].hide()
				self["lyricsLineBackgroundBig"].setBoolean(False)
			if self.lyricsLineShow or self.lyricsLineShowBig:
				if (self.screenSaverActive or self.isVideoFullScreen) and self.playerState < STATE_STOP:
					if self.lyricsLineShow or self.isVideoFullScreen:
						self["lyricsLine"].setText(self.oldLyricsText)
						self["lyricsLine"].show()
						self["lyricsLineBackground"].setBoolean(True)
					else:	
						self["lyricsLineBig"].setText(self.oldLyricsText)
						self["lyricsLineBig"].show()
						self["lyricsLineBackgroundBig"].setBoolean(True)
					self.startKaraokeMaxTimer()
		except:
			LOG('YampScreen: displayHideLyricsLine: EXCEPT', 'err')
			
	def pvrActions(self):
		if self.isVideoFullScreen:
			try: 
				self.videoTitleScreenOpen = True
			except: LOG('YampScreen: pvrActions: openYampVideoTitleV32: EXCEPT', 'err') 
		elif not self.currentIsVideo or self.videoStartMode != "manual":
			self.showhideScreenSaverMan()

	def showhideScreenSaverMan(self):
###		LOG('YampScreen: showhideScreenSaverMan: TimerIsActive: %d' %(self.screenSaverManTimer.isActive()), 'spe')
		if self.manKeyScreensaver:
			if not (self.screenSaverTimer.isActive() or self.screenSaverActive) and self.playerState < STATE_STOP:
				self.screenSaverManOff = False
				self.startScreenSaverTimer()
#				LOG('YampScreen: showhideScreenSaverMan End1: self.screenSaverManOff: %d' %(self.screenSaverManOff), 'err')
			else:
#				LOG('showhideScreenSaverMan: hideScreenSaver', 'err')
				self.screenSaverManOff = True
				self.startScreenSaverManTimer()
				self.hideScreenSaver()
#				LOG('YampScreen: showhideScreenSaverMan End2: self.screenSaverManOff: %d' %(self.screenSaverManOff), 'err')
			self.setSaverButton()

	def screenSaverStartTimeout(self):
#		LOG('YampScreen: : screenSaverStartTimeout: FirstCall: %d' %(self.screenSaverFirstCall), 'all')
#		LOG('YampScreen: : screenSaverStartTimeout: FirstCall: %d' %(self.screenSaverFirstCall), 'spe2')
###		LOG('YampScreen: : screenSaverStartTimeout: currentIsVideo: %d  isVideoFullScreen: %d' %(self.currentIsVideo, self.isVideoFullScreen), 'spe')
		self.showScreenSaver()
		if self.lyricsLineShow or self.lyricsLineShowBig: 
			if not self.karaokeTimer.isActive(): self["lyricsLine"].setText(self.oldLyricsText)
			if self.lyricsLineShow:
				self["lyricsLine"].show()
				self["lyricsLineBackground"].setBoolean(True)
			else:	
				self["lyricsLineBig"].show()
				self["lyricsLineBackgroundBig"].setBoolean(True)
			self.startKaraokeMaxTimer()
		
		if self.currentIsVideo and not self.isVideoFullScreen: 
###			LOG('YampScreen: screenSaverStartTimeout: HideYampScreenShowVideo', 'spe')
			self.hideYampScreenShowVideo()
		if self.screenSaverFirstCall:
			self.screenSaverFirstCall = False
			self.resetSlideTimer()
			if config.plugins.yampmusicplayer.screenSaverMode.value == 'custom':
				self.nextSlide()		
		else:
			self.nextSlide()		

	def specialScreenActive(self):  #neuTest6
		return self.lyricsScreenActive or self.configScreenActive or self.dbScreenActive or self.virtKeyboardActive #neuTest6
			
	def showScreenSaver(self):
#		LOG('YampScreen: showScreenSaver: Start: screenSaverManOff: %d' %(self.screenSaverManOff), 'all')
#		LOG('YampScreen: showScreenSaver: Start: currentIsVideo: %d screenSaverManOff: %d' %(self.currentIsVideo,self.screenSaverManOff), 'all')
###		LOG('YampScreen: showScreenSaver: Start: currentIsVideo: %d screenSaverManOff: %d' %(self.currentIsVideo,self.screenSaverManOff), 'spe')
		if self.screenSaverManOff or self.specialScreenActive(): return

		self.screenSaverActive = True
		self.setSaverButton()

#		self.showYampScreenHideVideo()
		if self.screenSaverTimer.isActive(): self.screenSaverTimer.stop()
		self["screenSaverBackground"].show()
		self["screenSaver"].show() 

		if config.plugins.yampmusicplayer.showTextBg.value:
			self["songTextsBackground"].setBoolean(True)
			try:
				if len(self["fanartdownload"].getText()) > 0:
					self["downloadBackground"].setBoolean(True)
					self.showFanartNumberDl()
			except:
				LOG('YampScreen: showScreenSaver: self[songTextsBackground].setBoolean(True): EXCEPT', 'err')
		self.updateLCDInfo()
		LOG('YampScreen: : showScreenSaver: End', 'all')
#		if nextslide: self.nextSlide()
		
	def nextSlide(self):
		if not self.currentIsVideo:	self["screenSaver"].showNextPic()
		self.resetSlideTimer()
	
	def prevSlide(self):
		if not self.currentIsVideo:	self["screenSaver"].showPrevPic()
		self.resetSlideTimer()
			
	def resetSlideTimer(self):
#		LOG('YampScreen: resetSlideTimer', 'err')
		msecs = config.plugins.yampmusicplayer.screenSaverNextSlide.value * 1000
		if not self.slideShowPaused:
			self.nextSlideTimer.stop()
			self.nextSlideTimer.start(msecs)
#			LOG('YampScreen: resetSlideTimer: Stop and start', 'err')  ###Test
		
	def pausePlaySlideShow(self):
#		LOG('YampScreen: pausePlaySlideShow', 'err')
#		LOG('YampScreen: pausePlaySlideShow: screenSaverActive: %d' %(self.screenSaverActive), 'err')
		if self.screenSaverActive and not self.isVideoFullScreen:
			if self.slideShowPaused:
				self.slideShowPaused = False
				self.nextSlide()
				LOG('YampScreen: pausePlaySlideShow: nextSlide:', 'all')
			else:
				if self.nextSlideTimer.isActive():
					self.nextSlideTimer.stop()
					self.slideShowPaused = True
		self.setSaverButton()
	
#
# Menu key actions
#
# These actions depend on the active list and/or the current item in the list
# The configuration is always callable
#
	def showMenu(self):						#Build Menu Entries
#		self.showYampScreenHideVideo()
		self.hideScreenSaver()
#		try:
#			self.session.close(YampVideoTitleV32)
#		except:
#			LOG('YampScreen: showMenu: close VideoTitle: EXCEPT', 'err')
		menu = []
		if self.currList == "playlist":
			actSortMode = self.playlist.getSortMode()
#			LOG('\nYampScreen: showMenu: actSortMode: %d\r\n' %(actSortMode), 'spe2' )
			menu.append((_("Search in Playlist"), "searchplaylist"));
		if self.leftContent == "dblist" and self.currTitle != "":
			menu.append((_("Search current title"), "searchcurrentsong"));
		if self.leftContent == "dblist" and self.currArtist != "":
			menu.append((_("Search current artist"), "searchcurrentartist"));
		if self.leftContent == "dblist" and self.currAlbum != "":
			menu.append((_("Search current album"), "searchcurrentalbum"));
		if self.currList == "playlist":
			if actSortMode != 0: menu.append((_("Unsort Playlist"), "unsortplaylist"));
			if actSortMode != 1: menu.append((_("Sort Playlist A-Z"), "sortplaylistaz"));
			if actSortMode != 2: menu.append((_("Sort Playlist Z-A"), "sortplaylistza"));
			menu.append((_("Save Playlist e2pls"), "saveplayliste2pls"));
			menu.append((_("Save Playlist m3u"), "saveplaylistm3u"));
			menu.append((_("Save Playlist e2pls + m3u"), "saveplaylistboth"));
			menu.append((_("Search path/filename of current title"), "searchfilename"))
		elif self.currList == "filelist":
			menu.append((_("Search pathname"), "searchfilelist1"))		#pathnames
			menu.append((_("Search filename"), "searchfilelist2"))		#filenames
			if self.filelist.getFilename() is not None:
				if self.filelist.getFilename().lower().endswith(".e2pls") or self.filelist.getFilename().lower().endswith(".m3u"):
					menu.append((_("Delete playlist"), "deleteplaylist"))
		else:	# database list
			if self.dblist.mode == MENULIST:
				if self.dblist.getSelectionIndex() == 0: # Playlists
					menu.append((_("Search playlists"), "searchplaylists"))
				elif self.dblist.getSelectionIndex() == 1: # Artistlist
					menu.append((_("Search artists"), "searchartists"))
					menu.append((_("Delete ALL artist's musicbrainz-ID entries"), "deleteartistmbidallentries"))
				elif self.dblist.getSelectionIndex() == 2: # Albumlist
					menu.append((_("Search albums"), "searchalbums"))
				elif self.dblist.getSelectionIndex() == 3: # Titellist
					menu.append((_("Search titles"), "searchsongs"))
					menu.append((_("Search files"), "searchfiles"))
				elif self.dblist.getSelectionIndex() == 4: # Genrelist
					menu.append((_("Search genres"), "searchgenres"))
				elif self.dblist.getSelectionIndex() == 5: # Datelist
					menu.append((_("Search dates"), "searchdates"))
			elif self.dblist.mode == ARTISTLIST:
				menu.append((_("Search artists"), "searchartists"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete artist's title entries"), "deleteartistsongentries"))
				menu.append((_("Delete artist's musicbrainz-ID entries"), "deleteartistmbidentries"))
			elif self.dblist.mode in (ALBUMLIST, GENREALBUMLIST, ARTISTALBUMLIST, DATEALBUMLIST):
				menu.append((_("Search albums"), "searchalbums"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete album entries"), "deletealbumsongentries"))
			elif self.dblist.mode == GENRELIST:
				menu.append((_("Search genres"), "searchgenres"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete genre's title entries"), "deletegenresongentries"))
			elif self.dblist.mode == DATELIST:
				menu.append((_("Search dates"), "searchdates"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete date's title entries"), "deletedatesongentries"))
			elif self.dblist.mode == PLAYLISTLIST:
				menu.append((_("Search playlists"), "searchplaylists"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete ext. playlist entry"), "deleteplaylistentry"))
			elif self.dblist.mode >= TITLELIST:
				menu.append((_("Show similar titles"), "showsimilarsongs"))
				menu.append((_("Search titles"), "searchsongs"))
				menu.append((_("Search files"), "searchfiles"))
				menu.append((_("Jump to entry"), "jumptoentry"))
				menu.append((_("Delete title entry"), "deletesongentry"))
#			menu.append((_("Check music database"), "checkdatabase"))
			menu.append((_("Compact music database"), "compactdatabase"))
			menu.append((_("Clear music database"), "cleardatabase"))
					
		menu.append((_("Edit Configuration"), "config"))
		
		if len(menu) > 1:
			if self.currList == "playlist":
				self.session.openWithCallback(self.menuCallback, ChoiceBox, title=_("Playlist actions"), list=menu)
			elif self.currList == "filelist":
				self.session.openWithCallback(self.menuCallback, ChoiceBox, title=_("Filelist actions"), list=menu)
			else: # database list
				self.session.openWithCallback(self.menuCallback, ChoiceBox, title=_("Database actions"), list=menu)
		else:
			self.config()
		
	def menuCallback(self, choice):
###		LOG('YampScreen: menuCallback: call startScreenSaverTimer', 'spe')
		if choice is None:
			if not self.currentIsVideo or self.videoStartMode != "manual":
#				LOG('YampScreen: menuCallback1: resetScreenSaverTimer', 'all') 
				self.resetScreenSaverTimer()
#			self.startScreenSaverTimer()
			return
		if choice[1] == "searchfilelist1":
			self.searchFilelist1()
		elif choice[1] == "searchfilelist2":
			self.searchFilelist2()
		elif choice[1] == "unsortplaylist":
			self.sortPlaylist(0)
		elif choice[1] == "sortplaylistaz":
			self.sortPlaylist(1)
		elif choice[1] == "sortplaylistza":
			self.sortPlaylist(2)
		elif choice[1] == "searchplaylist":
			self.searchPlaylist()
		elif choice[1] == "saveplayliste2pls":
			self.savePlaylist(1)
		elif choice[1] == "saveplaylistm3u":
			self.savePlaylist(2)
		elif choice[1] == "saveplaylistboth":
			self.savePlaylist(3)
		elif choice[1] == "searchfilename":
			self.getPlaylistFile()
		elif choice[1] == "deleteplaylist":
			self.deletePlaylist()
		elif choice[1] == "jumptoentry":
			self.jumpToEntry()
		elif choice[1] == "deleteartistsongentries":
			self.deleteArtistSongEntries()
		elif choice[1] == "deleteartistmbidentries":
			self.deleteArtistMbid()
		elif choice[1] == "deleteartistmbidallentries":
			self.deleteArtistMbidAll()
		elif choice[1] == "deletealbumsongentries":
			self.deleteAlbumSongEntries()
		elif choice[1] == "deletegenresongentries":
			self.deleteGenreSongEntries()
		elif choice[1] == "deletedatesongentries":	#!!!!
			self.deleteDateSongEntries()
		elif choice[1] == "deleteplaylistentry":
			self.deletePlaylistEntry()
		elif choice[1] == "deletesongentry":
			self.deleteSongEntry()
		elif choice[1] == "showsimilarsongs":
			self.showSimilarSongs()
		elif choice[1] == "searchsongs":
			self.searchSongs()
		elif choice[1] == "searchfiles":
			self.searchFiles()
		elif choice[1] == "searchplaylists":
			self.searchPlaylists()
		elif choice[1] == "searchartists":
			self.searchArtists()
		elif choice[1] == "searchalbums":
			self.searchAlbums()
		elif choice[1] == "searchgenres":
			self.searchGenres()
		elif choice[1] == "searchcurrentsong":
			self.searchCurrentSong()
		elif choice[1] == "searchcurrentartist":
			self.searchCurrentArtist()
		elif choice[1] == "searchcurrentalbum":
			self.searchCurrentAlbum()
		elif choice[1] == "checkdatabase":
			pass	# to be implemented
		elif choice[1] == "compactdatabase":
			self.compactDatabase()
		elif choice[1] == "cleardatabase":
			self.clearDatabase()
		elif choice[1] == "config":
			self.config()
		if choice[1] != "config":			#no Screensaver start in config
			if not self.currentIsVideo or self.videoStartMode != "manual":
#				LOG('YampScreen: menuCallback2: resetScreenSaverTimer', 'all') 
				self.resetScreenSaverTimer()
#			self.startScreenSaverTimer()

	def getPlaylistFile(self):
		songpath = ''	
		try:
			sel = self.playlist.getSelection()
			if sel is not None:
				songpath = sel.getPath()
		except: LOG('YampScreen: getPlaylistFile: get songpath: EXCEPT', 'err')		
#		try: LOG('YampScreen: getPlaylistFile: songpath: %s' %(songpath), 'err')
#		except: LOG('YampScreen: getPlaylistFile: songpath: EXCEPT', 'err') 	
#		try: LOG('YampScreen: getPlaylistFile: dirname songpath: %s' %(os.path.dirname(songpath)), 'err')
#		except: LOG('YampScreen: getPlaylistFile: dirname songpath: EXCEPT', 'err') 	
		if songpath == '': return

		if not os.path.exists(songpath) or not os.path.isfile(songpath):
			msg = _("file %s not existing or not reachable") %(songpath)
			self.session.open(MessageBox, msg, type = MessageBox.TYPE_INFO )
#			self.session.open(MessageBox, msg, type = MessageBox.TYPE_INFO,timeout = 10 )
			return

		try:
			if not self.leftContent == "filelist":
				self.leftContent = "filelist"
				self.setLeftContent()
				self.setColorButtons()
				self.updateLCDInfo()

				
#				self.currList = "filelist"
#				self.filelist.selectionEnabled(1)
#			else:
#				self.currList = "dblist"
#				self.dblist.selectionEnabled(1)
#			if self.currList == "dblist":
#				self.currList = "filelist"
#				self.setLeftContent()
#				self.setColorButtons()
#				self.updateLCDInfo()
		
			self.currFilePath = os.path.dirname(songpath) 
			self.filelist.changeDir(self.currFilePath)
			self.filelist.searchPath(self.currFilePath)
#			self.filelist.refresh()
			self.setLeftContentTitle()

		except: LOG('YampScreen: getPlaylistFile: searchFile: EXCEPT', 'err') 	
	
#		try: LOG('YampScreen: getPlaylistFile: lenfilelist: %d' %(len(self.filelist)), 'err')
#		except: LOG('YampScreen: getPlaylistFile: lenfilelist: EXCEPT', 'err') 	
				
		try:
			for idx in range(len(self.filelist)):				
	#			LOG('YampScreen: getPlaylistFile: idx: %d' %(idx), 'err')
				self.filelist.moveToIndex(idx)
				actFileName = self.filelist.getFilename()
#				LOG('YampScreen: getPlaylistFile: getFilename: %s' %(actFileName), 'err')
				if actFileName == songpath: break
			self.switchToFileList()
#			self.filelist.refresh()
		except: LOG('YampScreen: getPlaylistFile: idx: EXCEPT', 'err') 	
#			self.filelist.selectionEnabled(1)

	def config(self):
###		LOG('YampScreen: Start Config: hidescreensaver', 'spe')
		self.hideScreenSaver()
		self.oldSkin = config.plugins.yampmusicplayer.yampSkin.value
#		self.oldLCD = config.plugins.yampmusicplayer.yampLCD.value
		self.oldLcdOff = config.plugins.yampmusicplayer.yampLCD.value == 'off'
		self.oldLcdRolling = config.plugins.yampmusicplayer.yampLCD.value == 'rolling'
		self.screenSaverManTimer.stop()
		self.configScreenActive = True
		self.session.openWithCallback(self.configFinished, YampConfigScreenV31)
		
	def configFinished(self, result):
#		LOG('YampScreen: ConfigFinished Start', 'err')
#		LOG('YampScreen: configFinished: call startSreenSaverTimer', 'spe')
		newLcdOff = config.plugins.yampmusicplayer.yampLCD.value == 'off'
		newLcdRolling = config.plugins.yampmusicplayer.yampLCD.value == 'rolling'
		self.repeat=config.plugins.yampmusicplayer.repeatPlaylistAtEnd.value
		self.searchFanart = config.plugins.yampmusicplayer.searchFanart.value
		if self.oldSkin != config.plugins.yampmusicplayer.yampSkin.value:
			self.session.open(MessageBox, _("Skin changed, Restart required to apply new Skin"), type = MessageBox.TYPE_INFO,timeout = 10 )
#		elif self.oldLCD != config.plugins.yampmusicplayer.yampLCD.value:
		elif self.oldLcdOff != newLcdOff or self.oldLcdRolling != newLcdRolling:
			self.session.open(MessageBox, _("Box-Display changed, Yamp restart required to apply new selection"), type = MessageBox.TYPE_INFO,timeout = 10 )
		self.setRepeatButton()
		self.configScreenActive = False
#		self.initScreenSaver(artist = self.currArtist, newArtist = True)
		self.initScreenSaver(artist = self.currArtist)
		self.startScreenSaverTimer()
		self.setHelpTexts()	
		self.infoBarNaReplace = INFOBARTEXTLIST[int(config.plugins.yampmusicplayer.displayUnknown.value)]
		self.lyricsPlayOffsetTime90 = config.plugins.yampmusicplayer.lyricsPlayOffsetTime.value * 90

		if self.logpath != config.plugins.yampmusicplayer.debugPath.value:
			self.logpath = config.plugins.yampmusicplayer.debugPath.value
			self.timestamp = datetime_datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
			config.plugins.yampmusicplayer.debugFile.value = self.logpath + 'yamp_debug_' + self.timestamp  + '.log'			
		self.memlog = config.plugins.yampmusicplayer.yampDebugMemory.value
		self.memlogfile = self.logpath + 'yamp_mem_' + self.timestamp +'.log'
		self.coverTestMode = config.plugins.yampmusicplayer.yampCoverLyricsTestMode.value
		self.videoStartMode = config.plugins.yampmusicplayer.videoAutoFull.value
		
		fanarttvpersonal = config.plugins.yampmusicplayer.fanarttvPersonal.value.strip()
#		LOG('YampScreen: configFinished: len: %d  fanarttvPersonalApikey: *%s*' %(len(fanarttvpersonal),fanarttvpersonal), 'err')
		if len(fanarttvpersonal) == 32 and not fanarttvpersonal.strip().startswith('1234567890abcdef') and not 'abcdef7890' in fanarttvpersonal.strip(): #neuTest6
			if self.fanarttvPersonalApikey != fanarttvpersonal:
				self.fanarttvPersonalApikey = fanarttvpersonal
				LOG('YampScreen: configFinished: len: %d  fanarttvPersonalApikey: %s' %(len(self.fanarttvPersonalApikey),self.fanarttvPersonalApikey), 'all')
				try:
					f = open(FANARTTVPERSAPIFILE,"w")
					f.write(fanarttvpersonal)
					LOG('YampScreen: configFinished: Personal api key written to file: *%s*' %(fanarttvpersonal), 'all')
				except:
					LOG('YampScreen: configFinished: EXCEPT: Personal api key could not be written to file', 'err')
				try: f.close()
				except:	pass

		self.startScreenSaverManTimer()
		if self.screenSaverActive:
			if config.plugins.yampmusicplayer.showTextBg.value:
				try: self["songTextsBackground"].setBoolean(True)
				except:	LOG('YampScreen: configFinished: self[songTextsBackground].setBoolean(True): EXCEPT', 'err')
			else:
				try: self["songTextsBackground"].setBoolean(False)
				except:	LOG('YampScreen: configFinished: self[songTextsBackground].setBoolean(False): EXCEPT', 'err')
		#Screensaver in Background
		self.ssBackground = config.plugins.yampmusicplayer.screenSaverBg.value
		self.setScreensaverBg()
				
#		
#
# Update methods for title info and cover
#
	def updateMusicInformation(self, artist = "", title = "", album = "", year = "", genre = "", length = "", tracknr = -1, strBitrate = "", clear = False):
		try: LOG('\nYampScreen: updateMusicInfo: title, artist, album: %s - %s - %s' %(title, artist, album), 'all')	
		except: LOG('\nYampScreen: updateMusicInfo: title, artist, album: EXCEPT' , 'err')	
#		try: LOG('\nYampScreen: updateMusicInfo: title, artist, album: %s - %s - %s currentIsVideo: %d' %(title, artist, album, self.currentIsVideo), 'spe2')	
#		except: LOG('\nYampScreen: updateMusicInfo: title, artist, album 2: EXCEPT' , 'err')	
		self.currTitle = title
		self.currArtist = artist
		self.currAlbum = album
		self.currLength = length
		self.currTracknr = tracknr
		self.currGenre = genre
		self.currYear = year
		self.currBitRate = strBitrate
		newCover = False
		newArtist = False
		if self.currTitle	!= self.oldTitle: 
			self.updateSingleMusicInformation("songtitle", title)
			self.updateSingleMusicInformation("year", year)
			self.updateSingleMusicInformation("genre", genre)
			self.updateSingleMusicInformation("bitrate", strBitrate)
			newCover = True
		if self.currArtist != self.oldArtist: 
			self.updateSingleMusicInformation("artist", artist)
			newArtist = True
		if self.currAlbum != self.oldAlbum: 
			self.updateSingleMusicInformation("album", album)
			newCover = True
		
		self.oldTitle = title
		self.oldArtist = artist
		self.oldAlbum = album
		if self.oldIsVideo == self.currentIsVideo: newArtist = True #neuTest6
		self.oldIsVideo = self.currentIsVideo
		
		if clear:	self["coverArt"].showDefaultCover()
		else:
			if newCover: self.updateCover(artist, album, title)

		if config.plugins.yampmusicplayer.yampLCD.value == 'rolling':
			self.updateLCDText(title + ' - ' + artist + ' - ' + album,1)
#			LOG('YampScreen: updateMusicInfo: title, artist, album: %s - %s - %s' %(title, artist, album), 'all')	
		else:
			self.updateLCDText(title,1)
			self.updateLCDText(artist,2)
			self.updateLCDText(album,3)
		LOG('YampScreen: UpdateMusicInfo: Artist: %s' %(artist), 'all')
		
#		if clear:
#			self["coverArt"].showDefaultCover()
#		else:
#			if newCover: self.updateCover(artist, album, title)
		
		if config.plugins.yampmusicplayer.screenSaverMode.value == "artwork":
			if clear:
###				LOG('YampScreen: updateMusicInfo: hidescreensaver' , 'spe') 
				self.hideScreenSaver()
			else:
				if newArtist: # and not self.currentIsVideo:		#!!!Video
#					self.initScreenSaver(artist, newArtist = True, initBlank = self.currentIsVideo)
					self.initScreenSaver(artist)
					if artist != 'n/a' and self.searchFanart != '0': 
						self.getArtistPics(artist)
			

	def updateSingleMusicInformation(self, name, info):
		try:
			info=info.replace('n/a', self.infoBarNaReplace)
		except:
			LOG('YampScreen: updateSingleMusicInformation: EXCEPT', 'err' )
		if self[name].getText() != info:
			self[name].setText(info)

	def coverChanged(self, filename):
		LOG('YampScreen: coverChanged: Start: filename: %s pixnum: %d' %(filename,self.pixNumCover), 'all')
#		LOG('YampScreen: coverChanged: Start: filename: %s pixnum: %d' %(filename,self.pixNumCover), 'err')
		num = self.pixNumCover % COVERS_NO

		if self.coverTestMode == 3:
			waitForLyricsCoverupdate = 0
			waitForLyricsCoverGoogleupdate = 0
			while (self.coverChangedLyrics):
				waitForLyricsCoverupdate += 1			
			while (self.coverChangedGoogleLyrics):
				waitForLyricsCoverGoogleupdate += 1			
			LOG('YampScreen: coverChanged: Testmode 3: wait: %d waitGoogle: %d' %(waitForLyricsCoverupdate,waitForLyricsCoverGoogleupdate), 'all')
			
		try:
			if num == 0:
				LOG('YampScreen: coverChanged: default cover', 'all')
				self["coverArt"].showDefaultCover()
				name, pix = self.getCoverArtFile()
				LOG('YampScreen: coverChanged: coverChangedLyrics = True (num=0): getCoverArtFile: %s' %(name), 'all')
				if self.lyricsScreenActive: self.coverChangedLyrics = True	#for Lyrics
				try: self.createLcdCoverImage(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, 'no_coverArt.png'))
				except:	LOG('YampScreen: coverChanged num=0: createLcdCoverImage: EXCEPT', 'err')
				return
			if num == COVERS_GOOGLE:
				LOG('YampScreen: coverChanged: coverChangedGoogle = True', 'all')
				self["coverArt"].showCoverArt(filename)
				name, pix = self.getCoverArtFile()
				LOG('YampScreen: coverChanged: coverChangedGoogleLyrics = True: getCoverArtFile: %s' %(name), 'all')
				if self.lyricsScreenActive: self.coverChangedGoogleLyrics = True	#for Lyrics
				self.saveCoverAuto(name)
			else:
				LOG('YampScreen: coverChanged: coverChanged else', 'all')
				self["coverArt"].showCoverArt(filename)
				name, pix = self.getCoverArtFile()
				LOG('YampScreen: coverChanged: coverChangedLyrics = True  getCoverArtFile: %s' %(name), 'all')
				if self.lyricsScreenActive: self.coverChangedLyrics = True	#for Lyrics
			try: 
#				LOG('YampScreen: coverChanged: call createLcdCoverImage:  name: %s' %(name), 'err')
				self.createLcdCoverImage(name)
			except:	LOG('YampScreen: coverChanged: createLcdCoverImage2: EXCEPT', 'err')
		except:
			LOG('YampScreen: coverChanged: EXCEPT', 'err')

	def createLcdCoverImage(self, picFile):
#		LOG('YampScreen: createLcdCoverImage: picFile: %s' %(picFile), 'err')
		lcdMode = config.plugins.yampmusicplayer.yampLCD.value
		if lcdMode == 'off' or lcdMode == 'rolling': return
		if not ('cover' in lcdMode):
			self.summaries.setCover() #delete old cover
			self.updateLCDInfo()
			return
		try:
			import Image
			LOG('YampScreen: createLcdCoverImage: picfile: %s' %(picFile), 'all')
		except: LOG('YampScreen: createLcdCoverImage: import Image: EXCEPT', 'err' )
		try: im = Image.open(picFile)
		except: LOG('YampScreen: createLcdCoverImage: image load: EXCEPT', 'err' )	
		try:
			color = config.plugins.yampmusicplayer.lcdCoverColor.value
			if color == 'bw': ic = im.convert("L")
			elif color == 'bwt': ic = im.convert("LA")	
			elif color == 'color': ic = im.convert("RGB")	
			else: ic = im.convert("RGBA")	
		except: LOG('YampScreen: createLcdCoverImage: image convert: EXCEPT', 'err' )	
		try:
			size = config.plugins.yampmusicplayer.lcdCoverSize.value
			ir = ic.resize((size,size))
		except: LOG('YampScreen: createLcdCoverImage: image resize: EXCEPT', 'err' )	
		try:
			ir.save('/tmp/coverlcd.png')
#			ir.save('/media/usb/Yamp/coverlcd.png')
		except: LOG('YampScreen: createLcdCoverImage: image save: EXCEPT', 'err' )	
		self.summaries.setCover()
		self.updateLCDInfo()

			
	def saveCoverAuto(self, srcPathName):
		configSave = config.plugins.yampmusicplayer.saveCover.value	
		try:
			LOG('YampScreen: saveCoverAuto: srcPathName: %s configSave: %s' %(srcPathName,configSave), 'spe')
			if configSave == '0': return
		except:
			LOG('YampScreen: saveCoverAuto: srcPathName, configSave: EXCEPT: return', 'err')
			return
		if srcPathName == '': return
		try:
			ext = os.path.splitext(os.path.basename(srcPathName))[1].strip()
		
			if configSave in ['titleNoPicture', 'titleNoTitle','titleAlways']: 
				destPathName = os.path.splitext(self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath())[0] + ext
			else:
				destPathName = os.path.join(os.path.dirname(self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()), 'albumcover' + ext)
			LOG('YampScreen: saveCoverAuto: destPathName: %s' %(destPathName), 'spe')

			#check for existing file
			if configSave == 'titleNoPicture':
				filelist = self.fileListByExt(os.path.dirname(destPathName),[".jpg", ".png", ".jpeg", ".gif"])
				LOG('YampScreen: saveCoverAuto: filelist: %s' %(filelist), 'spe')
				if len(filelist) > 0:
					LOG('YampScreen: saveCoverAuto: at least 1 pic existing in: %s, do not save cover' %(os.path.dirname(destPathName)), 'spe')
					return
			if configSave == 'titleNoTitle':
				filelist = self.fileListByExt(os.path.dirname(destPathName),[".jpg", ".png", ".jpeg", ".gif"])
				LOG('YampScreen: saveCoverAuto: filelist: %s' %(filelist), 'spe')
				for filename in filelist:
					if os.path.splitext(os.path.basename(filename))[0].lower().strip() == os.path.splitext(os.path.basename(destPathName))[0].lower().strip():
						LOG('YampScreen: saveCoverAuto: <title>.<ext> existing in: %s, do not save cover' %(os.path.dirname(destPathName)), 'spe')
						return
			if configSave == 'albumNoPicture':
				filelist = self.fileListByExt(os.path.dirname(destPathName),[".jpg", ".png", ".jpeg", ".gif"])
				for filename in filelist:
					if os.path.splitext(os.path.basename(filename))[0].lower().strip() == 'albumcover':
						LOG('YampScreen: saveCoverAuto: albumcover.<ext> existing in: %s, do not save cover' %(os.path.dirname(destPathName)), 'spe')
						return
			try:
				self.readWriteFileBinary(srcPathName, destPathName)
#				self.session.open(MessageBox, _("Cover saved"), type = MessageBox.TYPE_INFO,timeout = 5 )
			except:
				LOG('YampScreen: saveCoverAuto: writecover: EXCEPT', 'err')
#				self.session.open(MessageBox, _("Saving cover failed"), type = MessageBox.TYPE_INFO,timeout = 30 )
		except:
			LOG('YampScreen: saveCoverAuto: END: EXCEPT', 'err')
		
			
	#Look for cover in ID3 tag (mp3s/flac), in current directory, albumCover in current directory, Search cover with Google, if enabled
	def updateCover(self, artist, album, title, pathname=""):
		self.coverFoundPrio = COVERS_NO
		self.pixNumCover = COVERS_NO
		self.requestNum = (self.requestNum + 1) % 10
		
		LOG('YampScreen: UpdateCover Start: self.requestNum: %d pathname: %s' %(self.requestNum, pathname), 'spe')
		if pathname == '':
			pathname = self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath()
		LOG('YampScreen: UpdateCover Start: self.requestNum: %d pathname: %s' %(self.requestNum, pathname), 'spe')
		
		prioList = [config.plugins.yampmusicplayer.prioCover1.value, config.plugins.yampmusicplayer.prioCover2.value,
								config.plugins.yampmusicplayer.prioCover3.value, config.plugins.yampmusicplayer.prioCover4.value,
								config.plugins.yampmusicplayer.prioCover5.value]	
		LOG('YampScreen: UpdateCover prioList: %s' %(prioList), 'spe')

		filename = self.searchCoverID3(pathname, prioList)
		LOG('YampScreen: updateCover: before searchCoverDirectoryTitle: filename: %s' %(filename), 'spe')
		filename = self.searchCoverDirectoryTitle(pathname,filename,title,prioList)
		LOG('YampScreen: updateCover: before searchCoverAlbum: filename: %s' %(filename), 'spe')
		filename = self.searchCoverAlbum(pathname,filename, prioList)
		LOG('YampScreen: updateCover: before searchCoverDirectoryAny: filename: %s' %(filename), 'spe')
		filename = self.searchCoverDirectoryAny(pathname,filename,prioList)
		LOG('YampScreen: updateCover: call coverChanged: filename: %s' %(filename), 'spe')
		self.coverChanged(filename)
#		LOG('YampScreen: updateCover: before searchCoverGoogle', 'all')
		self.searchCoverGoogle(artist, album, title, prioList)


	#Search cover with Google, if enabled
	def searchCoverGoogle(self, artist, album, title, prioList):
		if 'coverGoogle' in prioList: prio = prioList.index('coverGoogle') + 1
		else: return

		LOG('YampScreen: searchCoverGoogle: currentIsVideo: %d  configSearchVideo: %d' %(self.currentIsVideo,config.plugins.yampmusicplayer.searchGoogleCoverVideo.value), 'spe') 
		if self.currentIsVideo and not config.plugins.yampmusicplayer.searchGoogleCoverVideo.value: return
		
		LOG('YampScreen: searchCoverGoogle: prio: %d  coverFoundPrio: %d' %(prio,self.coverFoundPrio), 'spe')
		if self.coverFoundPrio < prio: return				#already found with higher priority

		LOG('YampScreen: call getGoogleCover', 'spe')
		self.getGoogleCover(artist, album, title, prio)

	def getGoogleCover(self, artist,album, title, prio):
		LOG('YampScreen: getGoogleCover: Start: artist: %s  album: %s  title: %s' %(artist,album,title), 'spe')
		user_agent = 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3'
		headers = { 'User-Agent' : user_agent }

		if artist == 'n/a': artist = ''
		if album == 'n/a': album = ''
		if title == 'n/a': title = ''
		
		if artist != "" and album != "":
#			url = "http://images.google.de/images?q=%%22%s%%22+%%22%s%%22+OR+%%22%s%%22&btnG=Bilder-Suche" %(quote_plus(artist),quote_plus(title),quote_plus(album) )
			url = "http://images.google.de/images?q=%s+%s&btnG=Bilder-Suche" %(quote_plus(artist),quote_plus(album) )
##			LOG('YampScreen: getGoogleCover a, a: url: %s' %(url), 'spe')
###			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound,prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
			getPage(url, timeout=10,).addCallback(self.coverGoogleUrlFound,prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
#		if artist != "" and album != "" and title != "":
#			url = "http://images.google.de/images?q=%%22%s%%22+%%22%s%%22+OR+%%22%s%%22&btnG=Bilder-Suche" %(quote_plus(artist),quote_plus(title),quote_plus(album) )
#			url = "http://images.google.de/images?q=%s+%s+%s&btnG=Bilder-Suche" %(quote_plus(artist),quote_plus(album),quote_plus(title) )
#			LOG('YampScreen: getGoogleCover a, a, t: url: %s' %(url), 'all')
#			LOG('YampScreen: getGoogleCover a, a, t: url: %s' %(url), 'err')
#			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound,prio,self.requestNum).addErrback(self.coverGoogleUrlFailed,prio,self.requestNum)
		elif artist != "" and title != "":
			url = "http://images.google.de/images?q=%%22%s%%22+%%22%s%%22&btnG=Bilder-Suche" %(quote_plus(artist),quote_plus(title))
##			LOG('YampScreen: getGoogleCover a, t: url: %s' %(url), 'spe')
###			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound,prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound,prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
		elif title != "":
			url = "http://images.google.de/images?q=%%22%s%%22&btnG=Bilder-Suche" %(quote_plus(title))
##			LOG('YampScreen: getGoogleCover title: url: %s' %(url), 'spe')
###			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound, prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
			getPage(url, timeout=10, headers=headers).addCallback(self.coverGoogleUrlFound, prio,self.requestNum).addErrback(self.coverGoogleUrlFailed)
		else:
			LOG('YampScreen: getGoogleCover No artist, title, album', 'err')

	def coverGoogleUrlFound(self, result, prio, requestNum):
		self.requestNumSav1 = requestNum
		LOG('YampScreen: coverGoogleUrlFound self.requestNum: %d self.requestNumSav1 : %d' %(self.requestNum, self.requestNumSav1), 'spe')
		if self.requestNum != self.requestNumSav1: return

#	LOG('YampScreen: coverGoogleUrlFound: Start: result: %s' %(result), 'spe')
##		if len(result) > 30000: result = result[:30000]
####		LOG('YampScreen: coverGoogleUrlFound: Start: result: %s' %(result), 'spe')
##		try:	
##			soup = BeautifulSoup(''.join(result	))
#			LOG('YampScreen: coverGoogleUrlFound: soup: %s' %(soup), 'err')
##		except:
##			LOG('YampScreen: coverGoogleUrlFound: soup: EXCEPT', 'err')
#		try:	
#			x=soup.find('img')
#			LOG('YampScreen: coverGoogleUrlFound: soup: x: %s' %(x), 'err')
#		except:
#			LOG('YampScreen: coverGoogleUrlFound: soup: x: EXCEPT', 'err')
##		try:	
##			url=soup.find('img')['src']
#			LOG('YampScreen: coverGoogleUrlFound: soup: type(url): %s url: %s' %(type(url),url), 'err')
##		except:
##			LOG('YampScreen: coverGoogleUrlFound: soup: url: EXCEPT', 'err')
##		url=str(url)
##		if len(url) > 10:
#		LOG('YampScreen: coverGoogleUrlFound: Start: result: %s' %(result), 'all')
#		LOG('YampScreen: coverGoogleUrlFound: Start: result: %s' %(result), 'err')
		foundPos = result.find(' src="http://t')
#		foundPos2 = result.find('" width',foundPos)
		foundPos2 = result.find('&amp;s',foundPos)
####		LOG('YampScreen: coverGoogleUrlFound: foundPos: %d  foundPos2: %d' %(foundPos,foundPos2), 'spe')
		if foundPos != -1 and foundPos2 != -1:
			url = result[foundPos+6:foundPos2]
			LOG('YampScreen: coverGoogleUrlFound URL: %s ' %(url), 'spe')
#			filename = url.split("/")[-1]
#			ext = os.path.splitext(filename)[1]
			ext = '.jpg'
			filename = "/tmp/.coverartgoogle" + str(self.coverIndex) + ext
			self.coverIndex = (self.coverIndex + 1) % COVER_MAXINDEX
			LOG('YampScreen: coverGoogleUrlFound: filename: %s ' %(filename), 'spe')
#			downloadPage(url, filename).addCallback(boundFunction(self.coverDownloadFinished, filename, prio, requestNum)).addErrback(self.coverDownloadFailed, requestNum)
			downloadPage(url, filename).addCallback(boundFunction(self.coverDownloadFinished, filename, prio, requestNum)).addErrback(self.coverGoogleUrlFailed)
		else:
			LOG('YampScreen: coverGoogleUrlFound: No cover found', 'spe')

#	def coverGoogleUrlFailed(self,result,requestNum):
	def coverGoogleUrlFailed(self,result):
#		LOG('YampScreen: coverGoogleUrlFailed', 'all')
		LOG('YampScreen: coverGoogleUrlFailed: %s' % str(result.getErrorMessage()), 'err')

	def coverDownloadFinished(self, filename, prio, requestNum, result):
		self.requestNumSav2 = requestNum
		LOG('YampScreen: coverDownloadFinished self.requestNum: %d self.requestNumSav2 : %d' %(self.requestNum, self.requestNumSav2), 'spe')
		if self.requestNum != self.requestNumSav2: return
		LOG('YampScreen: coverDownloadFinished: filename: %s' %(filename), 'spe')
#	LOG('YampScreen: coverDownloadFinished', 'spe')
		self.pixNumCover = COVERS_GOOGLE
		self.coverFoundPrio = prio
		LOG('YampScreen: coverDownloadFinished: call coverChanged: filename: %s' %(filename), 'spe')
		self.coverChanged(filename)
		
			
	# Mp3 / Flac: ID3 Cover search
	def searchCoverID3(self,pathname, prioList):
		# mp3: ID3 Tag search
		filename = ''
		if 'coverID3' in prioList: prio = prioList.index('coverID3') + 1
		else: return

		LOG('YampScreen: searchCoverID3: prio: %d  coverFoundPrio: %d' %(prio,self.coverFoundPrio), 'spe')
		if self.coverFoundPrio < prio: return			#already found with higher priority
		
		mime = None
		isFlac = isMp4 = False

		# mp3, mp4
		if pathname.lower().endswith(".mp3") or pathname.lower().endswith(".mp4") or pathname.lower().endswith(".m4a"):
			LOG('YampScreen: searchCoverID3: Mp3/Mp4', 'spe')
			try: audio = ID3(pathname)
			except:	
				LOG('YampScreen: searchCoverID3 : audio = NONE', 'spe')
				audio = None
			try:
				title, album, genre, artist, date, length, tracknr, strBitrate = readID3Infos(pathname)
#				LOG('YampScreen: searchCoverID3 : date: %s length %s: tracknr: %d' %(date, length, tracknr), 'err')
				LOG('YampScreen: searchCoverID3 : date: %s length %s: tracknr: %d' %(date, length, tracknr), 'spe')
			except:
				LOG('YampScreen: searchCoverID3 : date, length, tracknr: EXCEPT', 'err')

		if pathname.lower().endswith(".mp3"):
			if audio:
				apicframes = audio.getall("APIC")
				LOG('YampScreen: searchCoverID3 : len apicframes: %d ' %(len(apicframes)), 'spe')
				if len(apicframes) > 0:
					mime = apicframes[0].mime
					data = apicframes[0].data 	# check for type=3 (FrontCover)?
					type = apicframes[0].type
		if pathname.lower().endswith(".mp4") or pathname.lower().endswith(".m4a"):
			isMp4 = True
			try: from mutagen.mp4 import MP4, MP4Cover
			except:	LOG('YampScreen: searchCoverID3 : import MP4: EXCEPT', 'err')
			try: video = MP4(pathname)
			except:	
				LOG('YampScreen: searchCoverID3: Mp4: video = NONE', 'spe')
				video = None
			try:
				if video:
					if len(video["covr"]):
						data = video["covr"][0]
						mime = 'jpeg'
						type = 0
			except:	pass
#			except:	LOG('YampScreen: searchCoverID3 : data EXCEPT', 'err')
		#flac				
		elif pathname.lower().endswith(".flac"):
			LOG('YampScreen: searchCoverID3: Flac', 'spe')
			isFlac = True
			try: flacInfo = FLAC(pathname)
			except:	flacInfo = None
			if flacInfo:
				try:
					LOG('YampScreen: searchCoverID3 : 	flacInfo: %s' %(flacInfo), 'spe')
				except:
					LOG('YampScreen: searchCoverID3 : 	flacInfo: EXCEPT', 'err')
				picture_list = flacInfo.pictures		
				if len(picture_list) > 0:
					mime = picture_list[0].mime
					data = picture_list[0].data 	# check for type=3 (FrontCover)?
					type = picture_list[0].type
		# mp3, mp4 and flac			
		if mime:			
			LOG('YampScreen: searchCoverID3 : 	mime: %s' %(mime), 'spe')
			LOG('YampScreen: searchCoverID3 : 	type: %d' %(type), 'spe')
			if mime.lower().endswith("png"): ext = ".png"
			elif mime.lower().endswith("jpg"): ext = ".jpg"
			elif mime.lower().endswith("jpeg"): ext = ".jpeg"
			else:
				try: ext = mime.lower().rsplit('/',1)[1] 
				except: ext = ""
				LOG('YampScreen: searchCoverID3: MP3/Flac: ignored cover file with extension %s' %(ext), 'spe')
				ext = "" 		# ignore all other files (bmp, ...)
			if ext <> "":
				filename = "/tmp/.coverartid3_" + str(self.coverIndex) + ext
				self.coverIndex = (self.coverIndex + 1) % COVER_MAXINDEX
				LOG('YampScreen: searchCoverID3: MP3/Flac: write coverart file: %s' %(filename), 'spe')
				coverArtFile = open(filename, 'wb')
				coverArtFile.write(data) 
				coverArtFile.close()
				if isMp4: self.pixNumCover = COVERS_MP4
				elif isFlac: self.pixNumCover = COVERS_FLAC
				else : self.pixNumCover = COVERS_MP3
				self.coverFoundPrio = prio
		LOG('YampScreen: searchCoverID3: end: return filename: %s' %(filename), 'spe')
		return filename


	#Look for <title>.<ext> in current directory (without albumcover.*) 			
	def searchCoverDirectoryTitle(self,pathname,fullname,title,prioList):	
		if 'coverDirTitle' in prioList: prio = prioList.index('coverDirTitle') + 1
		else: return fullname

		LOG('YampScreen: searchCoverDirectoryTitle: prio: %d  coverFoundPrio: %d' %(prio,self.coverFoundPrio), 'spe')
		if self.coverFoundPrio < prio: return fullname			#already found with higher priority

		dir = os.path.dirname(pathname)
		titleFileName = os.path.basename(os.path.splitext(self.playlist.getServiceRefList()[self.playlist.getCurrentIndex()].getPath())[0])
		LOG('YampScreen: searchCoverDirectoryTitle: dir: %s titleFileName: %s' %(dir,titleFileName), 'spe')
		filelist = self.fileListByExt(dir, COVER_EXTENSIONS, False, COVER_MINSIZE)
		LOG('YampScreen: searchCoverDirectoryTitle: len(filelist): %d' %(len(filelist)), 'spe')
		
		coverNameExt = ''
		if len(filelist) > 0:
			for singleFile in filelist:
				LOG('YampScreen: searchCoverDirectoryTitle: singleFile: %s titleFileName: %s' %(singleFile,titleFileName), 'spe')
			#	LOG('YampScreen: searchCoverDirectoryTitle: singleFile: %s titleFileName: %s' %(singleFile,titleFileName), 'all')
				if os.path.splitext(singleFile)[0].lower().strip() == titleFileName.lower().strip():
					coverNameExt  = singleFile
					self.pixNumCover = COVERS_DIR
					self.coverFoundPrio = prio
					LOG('YampScreen: searchCoverDirectoryTitle: cover found: %s' %(singleFile), 'spe')
					break
		if coverNameExt == '': return fullname

		coverExt = os.path.splitext(coverNameExt)[1]
		try:
			coverSourcePathName = os.path.join(dir, coverNameExt)
			fullname = "/tmp/.coverartdirtitle" + str(self.coverIndex) + coverExt
			self.coverIndex = (self.coverIndex + 1) % COVER_MAXINDEX
			self.readWriteFileBinary(coverSourcePathName, fullname)
		except:	
			LOG('YampScreen: searchCoverDirectoryTitle: write coverartdirtitle: EXCEPT', 'err')
		LOG('YampScreen: searchCoverDirectoryTitle: end: fullname %s' %(fullname), 'spe')
		return fullname

	#Look for any picture in current directory (without albumcover.*) 			
	def searchCoverDirectoryAny(self,pathname,fullname,prioList):	
		if 'coverDirAny' in prioList: prio = prioList.index('coverDirAny') + 1
		else: return fullname

		LOG('YampScreen: searchCoverDirectoryAny: prio: %d  coverFoundPrio: %d' %(prio,self.coverFoundPrio), 'spe')
		if self.coverFoundPrio < prio: 
			LOG('YampScreen: searchCoverDirectoryAny: already found: fullname %s' %(fullname), 'spe')
			return fullname			#already found with higher priority

		dir = os.path.dirname(pathname)
		LOG('YampScreen: searchCoverDirectoryAny: dir: %s' %(dir), 'spe')
		filelist = self.fileListByExt(dir, COVER_EXTENSIONS, False, COVER_MINSIZE)
		LOG('YampScreen: searchCoverDirectoryAny: len(filelist): %d' %(len(filelist)), 'spe')
		if len(filelist) > 0:
			for singleFile in filelist:
				LOG('YampScreen: searchCoverDirectoryAny: singlefile: %s' %(singleFile), 'spe')
				if singleFile.lower().startswith('albumcover'):
					LOG('YampScreen: searchCoverDirectoryAny: singleFile in Filelist startswith: %s' %(singleFile), 'spe')
					filelist.remove(singleFile)
		if len(filelist) > 0:
			self.pixNumCover = COVERS_DIR
			self.coverFoundPrio = prio

			coverNameExt = filelist[0]
			LOG('YampScreen: searchCoverDirectoryAny: coverNameExt: %s' %(coverNameExt), 'spe')
			coverExt = os.path.splitext(coverNameExt)[1]
			try:
				coverSourcePathName = os.path.join(dir, coverNameExt)
				LOG('YampScreen: searchCoverDirectoryAny: coverSourcePathName: %s' %(coverSourcePathName), 'spe')
				fullname = "/tmp/.coverartdirany" + str(self.coverIndex) + coverExt
				self.coverIndex = (self.coverIndex + 1) % COVER_MAXINDEX
				self.readWriteFileBinary(coverSourcePathName, fullname)
			except:	
				LOG('YampScreen: searchCoverDirectoryAny: write coverartdirany: EXCEPT', 'err')
			LOG('YampScreen: searchCoverDirectoryAny: end: coverArtfile: %s' %(fullname), 'spe')
		return fullname


	#	Look for albumCover in current directory
	def searchCoverAlbum(self,pathname,fullname, prioList):
		if 'coverAlbum' in prioList: prio = prioList.index('coverAlbum') + 1
		else: return fullname

		LOG('YampScreen: searchCoverAlbum: prio: %d  coverFoundPrio: %d' %(prio,self.coverFoundPrio), 'spe')
		if self.coverFoundPrio < prio: return	fullname		#already found with higher priority

#		fullname=""
		dir = os.path.dirname(pathname)
		LOG('YampScreen: searchCoverAlbum: dir: %s' %(dir), 'spe')
		filelist = list(self.fileListByExt(dir, COVER_EXTENSIONS, False, COVER_MINSIZE))
		LOG('YampScreen: searchCoverAlbum: len(filelist): %d' %(len(filelist)), 'spe')
		LOG('YampScreen: searchCoverAlbum: filelist): %s' %(filelist), 'spe')
		if len(filelist) > 0:
			for singleFile in filelist:
				LOG('YampScreen: searchCoverAlbum: singlefile: %s' %(singleFile), 'spe')
				if not singleFile.lower().startswith('albumcover'):
					LOG('YampScreen: searchCoverAlbum: singleFile in Filelist startswith: %s' %(singleFile), 'spe')
					try:
						LOG('YampScreen: searchCoverAlbum: remove singlefile: %s' %(singleFile), 'spe')
						filelist.remove(singleFile)
					except:
						LOG('YampScreen: searchCoverAlbum: remove singlefile: EXCEPT', 'err')
						
		LOG('YampScreen: searchCoverAlbum: len(filelist): %d' %(len(filelist)), 'spe')
		if len(filelist) > 0:
			LOG('YampScreen: searchCoverAlbum: filelist[0]: %s' %(filelist[0]), 'spe')
			self.pixNumCover = COVERS_ALB
			self.coverFoundPrio = prio
			coverNameExt = filelist[0]
			coverExt = os.path.splitext(coverNameExt)[1]
			LOG('YampScreen: searchCoverAlbum: coverNameExt: %s  coverExt: %s' %(coverNameExt,coverExt), 'spe')
			try:
				coverSourcePathName = os.path.join(dir, coverNameExt)
				fullname = "/tmp/.coverartalbum" + str(self.coverIndex) + coverExt
				self.coverIndex = (self.coverIndex + 1) % COVER_MAXINDEX
				LOG('YampScreen: searchCoverAlbum: coverSourcePathName: %s  fullname: %s' %(coverSourcePathName,fullname), 'spe')
				self.readWriteFileBinary(coverSourcePathName, fullname)
			except:	
				LOG('YampScreen: searchCoverAlbum: write coverartalbum: EXCEPT', 'err')
			
			LOG('YampScreen: searchCoverAlbum: end: coverArtfile: %s' %(fullname), 'spe')
		return fullname

	def fileListByExt(self, path, extensions, subDirs = False, minSize = 0):
		list = []
#		LOG('YampScreen: fileListByExt: path: %s ' %(path), 'all')	
		if subDirs:
			subFolders=[]
			for dirpath, dirnames, files in os.walk(path):
				try:
#					LOG('YampScreen: fileListByExt: subDirs: os.walk: dirpath: %s dirnames: %s files: %s' %(dirpath, dirnames, files), 'all')	
					for file in files:
#						LOG('YampScreen: fileListByExt: subDirs: os.walk: dirpath: %s file %s' %(dirpath, file), 'all')	
						ext = os.path.splitext(file)[1]
						try:
							size = os.stat(dirpath + '/' + file).st_size
#							LOG('YampScreen: fileListByExt: subDirs: os.walk: filesize %d' %(size), 'all')	
						except:
							size = 0
							LOG('YampScreen: fileListByExt: subDirs: os.walk: filesize EXCEPT', 'err')	
						if ext.lower() in extensions:
							if size >= minSize:		
								list.append(file)
				except:
					LOG('YampScreen: fileListByExt: subDirs: : EXCEPT', 'err')	
		else:
			for file in os.listdir(path):
				ext = os.path.splitext(file)[1]
				try:
					size = os.stat(path + '/' + file).st_size
#					LOG('YampScreen: fileListByExt: os.walk: path: %s filesize %d' %(path, size), 'all')	
				except:
					size = 0
					LOG('YampScreen: fileListByExt: os.walk: filesize EXCEPT', 'err')	
				if ext.lower() in extensions:
					if size >= minSize:	
						list.append(file)
		try:
			LOG('YampScreen: fileListByExt: list: %s' %(list), 'all')	
		except:
			LOG('YampScreen: fileListByExt: LogList: EXCEPT', 'err')	
		return list

	def readWriteFileBinary(self, filePathSrc, filePathDest):
		try:
			file = open(filePathSrc, 'rb')
			data = file.read()
			file.close()
			file = open(filePathDest, 'wb')
			file.write(data)
			file.close()
		except:
			LOG('YampScreen: readWriteFile: EXCEPT', 'err')

		
	def showLyrics(self):
		if not self.currentIsVideo or config.plugins.yampmusicplayer.showLyricsOnVideo.value: #neuTest6
			self.hideScreenSaver()
			self.screenSaverManTimer.stop()
			self.lyricsScreenActive = True
#			LOG('YampScreen: showLyrics: screenSaverManTimer: STOP', 'err') 
			self.session.openWithCallback(self.showLyricsCB, YampLyricsScreenV32, self)
		
	def showLyricsCB(self):
#		LOG('YampScreen: showLyricsCB: Start: lyricsScreenActive: %d' %(self.lyricsScreenActive), 'err')
		self.startScreenSaverManTimer()
		self.coverChangedLyrics = False
		self.coverChangedGoogleLyrics = False
		self.lyricsScreenActive = False
		if not self.currentIsVideo or self.videoStartMode != "manual":
#			LOG('YampScreen: showLyricsCB: resetScreenSaverTimer', 'all') 
			self.resetScreenSaverTimer()

	def	getCoverArtFile(self):	
		self.coverArtFile = self["coverArt"].getFileName()
#		LOG('YampScreen: getCoverArtFile: coverArtFile: %s' %(self.coverArtFile), 'err') 
		return self.coverArtFile, self.pixNumCover % COVERS_NO
		
		
#
# LCD display methods
#
	def updateLCDInfo(self):
		lcdMode = config.plugins.yampmusicplayer.yampLCD.value
#		LOG('\nYampScreen: updateLCDInfo: start: lcdMode: %s' %(lcdMode), 'err')
		if lcdMode == 'off': return
		textList = []
		text1 = ""
		text2 = ""
		text3 = ""

		if self.currList == "filelist":
			text1 = _("filelist")
			text2 = self.filelist.getName()
		elif self.currList == "dblist":
			text1 = _("database")
			sel = self.dblist.getSelection()
			if sel is not None:
				if sel.text.strip() <> '': text2 = sel.text.strip()
				else:
					text2 = sel.title.strip()
					text3 = sel.artist.strip()
		else:
			sel = self.playlist.getSelection()
#			try: LOG('YampScreen: updateLCDInfo: playlist getSelection: %s' %(sel), 'err')
#			except: LOG('YampScreen: updateLCDInfo: playlist getSelection: EXCEPT', 'err') 	
			if sel is not None:
				showActSong = (self.playlist.getCurrentIndex() == self.playlist.getSelectionIndex()) \
							and (self.playerState < STATE_STOP) or self.screenSaverActive or self.isVideoFullScreen
#				try: LOG('YampScreen: updateLCDInfo: playlist showActSong: %d' %(showActSong), 'err')
#				except: LOG('YampScreen: updateLCDInfo: playlist showActSong": EXCEPT', 'err') 	
				if showActSong:
					text1 = self.currTitle
					text2 = self.currArtist
					text3 = self.currAlbum
				else:	
					text1 = _("playlist")
					text2 = sel.getName()
#					try: LOG('YampScreen: updateLCDInfo: playlist text2: %s' %(text2), 'err')
#					except: LOG('YampScreen: updateLCDInfo: playlist text2": EXCEPT', 'err') 	
					if text2 == "":	text2 = os.path.splitext(os.path.basename(sel.getPath()))[0]

					textList = text2.rsplit('-',1)
					try: text2 = textList[0].strip()
					except: pass
					try: text3 = textList[1].strip()
					except: pass
#				try: LOG('YampScreen: updateLCDInfo: playlist text1: %s text2: %s text3: %s' %(text1,text2,text3), 'err')
#				except: LOG('YampScreen: updateLCDInfo: playlist text": EXCEPT', 'err') 	
	
		if lcdMode == 'rolling':
			self.updateLCDText(text1 + ' - ' + text2 + ' - ' + text3, 1)
		elif lcdMode == 'oneline' or lcdMode == 'cover1':	
			self.updateLCDText(text1 + ' ' + text2 + ' ' + text3,1)
#		elif lcdMode == 'threelines' or lcdMode == 'cover3':
		else:
			self.updateLCDText(text1,1)
			self.updateLCDText(text2,2)
			self.updateLCDText(text3,3)
		
	def updateLCDText(self, text, line):
		lcdMode = config.plugins.yampmusicplayer.yampLCD.value
		if lcdMode == 'off': return
		if lcdMode == 'rolling':
			try:
				self.LcdText = text
#				LOG('YampScreen: updateLCDText: self.LcdText: %s' %(self.LcdText), 'all')
			except:	pass
		else:	self.summaries.setText(text,line)

	def getLcdText(self):	#for LCD Running Text
#		LOG('YampScreen: getLcdText: LcdText: %s' %(self.LcdText), 'all')
		return(self.LcdText)

#
# Filelist actions
#
	def searchFilelist1(self): #Suche in Pfadname
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchFilelistCallback1, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text=self.lastTextSearch)

	def searchFilelist2(self): #Suche in Dateiname
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchFilelistCallback2, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text=self.lastTextSearch)

	def searchFilelistCallback1(self, choice):
		if choice is not None:
			self.lastTextSearch = choice;
			self.filelist.searchPath(choice)
			self.updateLCDInfo()
		self.virtKeyboardActive = False
			
	def searchFilelistCallback2(self, choice):
		if choice is not None:
			self.lastTextSearch = choice;
			self.filelist.searchFile(choice)
			self.updateLCDInfo()
		self.virtKeyboardActive = False

#
# Database actions
# Only quick actions are executed in foreground	
# Slow actions are executed in background and are interruptable, ideally without changes to the database
#
	def addFileToDb(self):
		con, result = ConnectDatabase()
		if con is not None:
			con.text_factory = str
			cursor = con.cursor()
			f = self.filelist.getFilename()
			if f.lower().endswith(".m3u") or f.lower().endswith(".pls") or f.lower().endswith(".e2pls"):
				dbInsert(con, cursor, self.filelist.getServiceRef(), table="playlists", update=True)
			else:
				dbInsert(con, cursor, self.filelist.getServiceRef(), table="titles", update=True)
			con.commit()
			cursor.close()  
			con.close()

	def deletePlaylistEntry(self):
		con, result = ConnectDatabase()
		if con is not None:
			try:
				actIndex = self.dblist.getSelectionIndex()
			except:
				LOG('YampScreen: deletePlaylistEntry: actIndex','err' )
			con.text_factory = str
			cursor = con.cursor()
			cursor.execute('DELETE FROM Playlists WHERE playlist_id = %d;' % self.dblist.getSelection().titleID)
			con.commit()
			cursor.close()  
			con.close()
			self.updateDbMenuList()
			try:
				self.dblist.moveToIndex(actIndex)	
			except:
				LOG('YampScreen: deletePlaylistEntry: moveToIndex','err' )
			
	def deleteSongEntry(self):
		con, result = ConnectDatabase()
		if con is not None:
			try:
				actIndex = self.dblist.getSelectionIndex()
			except:
				LOG('YampScreen: deleteSongEntry: actIndex','err' )
			con.text_factory = str
			cursor = con.cursor()
			cursor.execute('DELETE FROM Titles WHERE title_id = %d;' % self.dblist.getSelection().titleID)
			con.commit()
			cursor.close()  
			con.close()
			self.updateDbMenuList()
			try:
				self.dblist.moveToIndex(actIndex)	
			except:
				LOG('YampScreen: deleteSongEntry: moveToIndex','err' )
			
	def deleteArtistMbid(self):
		con, result = ConnectDatabase()
		LOG('YampScreen: deleteArtistMbid: result: %d con: %s' %(result, con), 'err')
		if con is not None:
#			con.text_factory = str
			cursor = con.cursor()
			try:
				artistId = self.dblist.getSelection().artistID
#				LOG('YampScreen: deleteArtistMbid: artistId: %d' %(artistId), 'err')
			except:
				LOG('YampScreen: deleteArtistMbid: artistId: EXCEPT', 'err')
			try:
				cursor.execute("UPDATE Artists SET mbid = NULL WHERE artist_id = %d" %(artistId))
			except:
				LOG('YampScreen: deleteArtistMbid: mbid: EXCEPT', 'err')
			try:
				cursor.execute("UPDATE Artists SET picsloaded = '0' WHERE artist_id = %d" %(artistId))
			except:
				LOG('YampScreen: deleteArtistMbid: picsloaded: EXCEPT', 'err')
			con.commit()
			cursor.close()  
			con.close()
#			self.updateDbMenuList()

	def deleteArtistMbidAll(self):
		self.session.openWithCallback(self.deleteArtistMbidAllConfirmed, MessageBox, _("Do you really want to delete the musicbrainz-IDs for all artists from database?"))

	def deleteArtistMbidAllConfirmed(self, confirmed):
		con, result = ConnectDatabase()
		if con is not None:
#			con.text_factory = str
			cursor = con.cursor()
			cursor.execute("UPDATE Artists SET mbid = NULL")
			cursor.execute("UPDATE Artists SET picsloaded = '0';")
			con.commit()
			cursor.close()  
			con.close()

	def deleteArtistSongEntries(self):
		self.session.openWithCallback(self.deleteArtistSongEntriesConfirmed, MessageBox, _("Do you really want to delete all titles of artist %s from database?") % self.dblist.getSelection().artist)

	def deleteArtistSongEntriesConfirmed(self, confirmed):
		if confirmed:
			con, result = ConnectDatabase()
			if con is not None:
				try:
#					nav = self.dblist.getSelection().nav
#					LOG('YampScreen: deleteArtistSongEntriesConfirmed: nav: %s' %(nav),'err' )
#					LOG('YampScreen: deleteArtistSongEntriesConfirmed: getselection: %s' %(self.dblist.getSelection()),'err' )
#					LOG('YampScreen: deleteArtistSongEntriesConfirmed: type getselection: %s' %(type(self.dblist.getSelection())),'err' )
#					LOG('YampScreen: deleteArtistSongEntriesConfirmed: getselection artist: %s' %(self.dblist.getSelection().artist),'err' )
#					LOG('YampScreen: deleteArtistSongEntriesConfirmed: getselection index: %d' %(self.dblist.getSelectionIndex()),'err' )
					actIndex = self.dblist.getSelectionIndex()
				except:
					LOG('YampScreen: deleteArtistSongEntriesConfirmed: actIndex','err' )
				con.text_factory = str
				cursor = con.cursor()
				cursor.execute('DELETE FROM Titles WHERE artist_id = %d;' % self.dblist.getSelection().artistID)
				cursor.execute('DELETE FROM Artists WHERE artist_id = %d;' % self.dblist.getSelection().artistID)
				con.commit()
				cursor.close()  
				con.close()
				self.updateDbMenuList()
				try:
					self.dblist.moveToIndex(actIndex)	
				except:
					LOG('YampScreen: deleteArtistSongEntriesConfirmed: moveToIndex','err' )

	def deleteAlbumSongEntries(self):
		self.session.openWithCallback(self.deleteAlbumSongEntriesConfirmed, MessageBox, _("Do you really want to delete all titles of album %s from database?") % self.dblist.getSelection().album)

	def deleteAlbumSongEntriesConfirmed(self, confirmed):
		if confirmed:
			con, result = ConnectDatabase()
			if con is not None:
				try:
					actIndex = self.dblist.getSelectionIndex()
				except:
					LOG('YampScreen: deleteAlbumSongEntriesConfirmed: actIndex','err' )
				con.text_factory = str
				cursor = con.cursor()
				cursor.execute('DELETE FROM Titles WHERE album_id = %d;' % self.dblist.getSelection().albumID)
				cursor.execute('DELETE FROM Albums WHERE album_id = %d;' % self.dblist.getSelection().albumID)
				con.commit()
				cursor.close()  
				con.close()
				self.updateDbMenuList()
				try:
					self.dblist.moveToIndex(actIndex)	
				except:
					LOG('YampScreen: deleteAlbumSongEntriesConfirmed: moveToIndex','err' )

	def deleteGenreSongEntries(self):
		self.session.openWithCallback(self.deleteGenreSongEntriesConfirmed, MessageBox, _("Do you really want to delete all titles of genre %s from database?") % self.dblist.getSelection().genre)

	def deleteGenreSongEntriesConfirmed(self, confirmed):
		if confirmed:
			con, result = ConnectDatabase()
			if con is not None:
				try:
					actIndex = self.dblist.getSelectionIndex()
				except:
					LOG('YampScreen: deleteGenreSongEntriesConfirmed: actIndex','err' )
				con.text_factory = str
				cursor = con.cursor()
				cursor.execute('DELETE FROM Titles WHERE genre_id = %d;' % self.dblist.getSelection().genreID)
				cursor.execute('DELETE FROM Genres WHERE genre = %d;' % self.dblist.getSelection().genreID)
				con.commit()
				cursor.close()  
				con.close()
				self.updateDbMenuList()
				try:
					self.dblist.moveToIndex(actIndex)	
				except:
					LOG('YampScreen: deleteGenreSongEntriesConfirmed: moveToIndex','err' )

	def deleteDateSongEntries(self):
		self.session.openWithCallback(self.deleteDateSongEntriesConfirmed, MessageBox, _("Do you really want to delete all titles of date %s from database?") % self.dblist.getSelection().date)

	def deleteDateSongEntriesConfirmed(self, confirmed):
		if confirmed:
			con, result = ConnectDatabase()
			if con is not None:
				try:
					actIndex = self.dblist.getSelectionIndex()
				except:
					LOG('YampScreen: deleteDateSongEntriesConfirmed: actIndex','err' )
				con.text_factory = str
				cursor = con.cursor()
				cursor.execute('DELETE FROM Titles WHERE date_id = %d;' % self.dblist.getSelection().dateID)
				cursor.execute('DELETE FROM Dates WHERE date = %d;' % self.dblist.getSelection().dateID)
				con.commit()
				cursor.close()  
				con.close()
				self.updateDbMenuList()
				try:
					self.dblist.moveToIndex(actIndex)	
				except:
					LOG('YampScreen: deleteDateSongEntriesConfirmed: moveToIndex','err' )

	def showSimilarSongs(self):
		numChoice = self.dblist.getSelectionIndex()
		self.pushDblistStack (numChoice)
		choice = self.dblist.getSelection()
		qs = "%" + choice.title.replace("'","''") + "%"
		self.buildDbMenuList (mode = SEARCHTITLELIST, queryString = qs, menutitle = choice.title)
		self.setLeftContentTitle()
		self.setColorButtons()

	def searchSongs(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchSongsCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchSongsCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHTITLELIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchFiles(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchFilesCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchFilesCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHFILELIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchPlaylists(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchPlaylistsCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchPlaylistsCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHPLAYLISTLIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchArtists(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchArtistsCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchArtistsCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHARTISTLIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchAlbums(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchAlbumsCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchAlbumsCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHALBUMLIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchGenres(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.searchGenresCallback, VirtualKeyBoard, title=_("YAMP - Please enter query:"), text="")

	def searchGenresCallback(self, choice):
		if choice is not None:
			numChoice = self.dblist.getSelectionIndex()
			self.pushDblistStack (numChoice)
			qs = "%" + choice.replace("'","''") + "%"
			self.buildDbMenuList (mode = SEARCHGENRELIST, queryString = qs, menutitle = _("Search: %s") % choice)
			self.setLeftContentTitle()
			self.setColorButtons()
		self.virtKeyboardActive = False

	def searchCurrentSong(self):
		numChoice = self.dblist.getSelectionIndex()
		self.pushDblistStack (numChoice)
		choice = self.currTitle
		qs = "%" + choice.replace("'","''") + "%"
		self.buildDbMenuList (mode = SEARCHTITLELIST, queryString = qs, menutitle = choice)
		self.setLeftContentTitle()
		self.setColorButtons()

	def searchCurrentArtist(self):
		numChoice = self.dblist.getSelectionIndex()
		self.pushDblistStack (numChoice)
		choice = self.currArtist
		qs = "%" + choice.replace("'","''") + "%"
		self.buildDbMenuList (mode = SEARCHARTISTTITLELIST, queryString = qs, menutitle = choice)
		self.setLeftContentTitle()
		self.setColorButtons()

	def searchCurrentAlbum(self):
		numChoice = self.dblist.getSelectionIndex()
		self.pushDblistStack (numChoice)
		choice = self.currAlbum
		qs = "%" + choice.replace("'","''") + "%"
		self.buildDbMenuList (mode = SEARCHALBUMTITLELIST, queryString = qs, menutitle = choice)
		self.setLeftContentTitle()
		self.setColorButtons()

	def jumpToEntry(self):
		self.virtKeyboardActive = True
		self.session.openWithCallback(self.jumpToEntryCallback, VirtualKeyBoard, title=_("YAMP - Jump to letter(s):"), text="")

	def jumpToEntryCallback(self, choice):
		if choice is not None:
			upperChoice = choice.upper()
			length = len(self.dblist)
			if length > 1:
				self.dblist.moveToIndex(1)
				if self.dblist.mode < TITLELIST:
					while self.dblist.getSelection().text.upper() < upperChoice and self.dblist.getSelectionIndex() < length - 1:
						self.dblist.down()
				else:
					while self.dblist.getSelection().title.upper() < upperChoice and self.dblist.getSelectionIndex() < length - 1:
						self.dblist.down()
				self.dblist.updateList()
		self.virtKeyboardActive = False

	def compactDatabase(self):
		msg = _("Do you really want to compact the database?\nThis may take a while...") 
		self.session.openWithCallback(self.compactDatabaseConfirmed, MessageBox, msg)

	def compactDatabaseConfirmed(self, confirmed):
		if confirmed:
			try:
				con, result = ConnectDatabase()
			except:
				LOG('YampScreen: compactDatabaseConfirmed: connect to Database: EXCEPT', 'err')
			if con is not None:	
				try:
					con.execute('VACUUM')
				except:
					LOG('YampScreen: compactDatabaseConfirmed: compact (VACUUM): EXCEPT', 'err')
				try:
					con.close()
					self.updateDbMenuList()
				except:
					LOG('YampScreen: compactDatabaseConfirmed: close Database: EXCEPT', 'err')

	def clearDatabase(self):
		self.session.openWithCallback(self.clearDatabaseConfirmed, MessageBox, _("Do you really want to clear the whole database?"))

	def clearDatabaseConfirmed(self, confirmed):
		if confirmed:
			try:
				os.remove(os.path.join(config.plugins.yampmusicplayer.databasePath.value ,"yamp.db"))
			except:
				LOG('YampScreen: clearDatabaseConfirmed: delete Database: EXCEPT', 'err')
			try:
				con, result = ConnectDatabase()
			except:
				LOG('YampScreen: clearDatabaseConfirmed: build new Database: EXCEPT', 'err')
			try:
				con.close()
				self.updateDbMenuList()
			except:
				LOG('YampScreen: clearDatabaseConfirmed: close Database: EXCEPT', 'err')
			return

			con, result = ConnectDatabase()
			if con is not None:
				con.text_factory = str
				cursor = con.cursor()
				cursor.execute('DELETE FROM Titles;')
				cursor.execute('DELETE FROM Artists;')
				cursor.execute('DELETE FROM Albums;')
				cursor.execute('DELETE FROM Genres;')
				cursor.execute('DELETE FROM Dates;')
				cursor.execute('DELETE FROM Playlists;')
				con.commit()
				cursor.close()  
				con.close()
				self.dbStack = []  # nothing more to navigate
				self.buildDbMenuList(MENULIST, menutitle=_("M u s i c  D a t a b a s e"))

#	def getCurrentSelection(self):   #!!! das ist neu
#		# Returns None or (serviceref, info, begin, len)
#		return self["list"].l.getCurrentSelection()
#		return "abcdefghijklmnopqrstuvwxyz  "
	
#------------------

	def getArtistPics(self, artist):
		if self.fanarttvPersonalApikey.strip() == '':	self.fanartDlTime = FANARTDLTIME
		else:	self.fanartDlTime = FANARTDLTIMEPERSONAL
		LOG('YampScreen: getArtistPics: Start: artist: %s artistFanartDl: %s fanartdltime: %d' %(artist,self.artistFanartDl,self.fanartDlTime), 'all')
#		LOG('YampScreen: getArtistPics: Start: artist: %s artistFanartDl: %s fanartdltime: %d' %(artist,self.artistFanartDl,self.fanartDlTime), 'spe2')

		if config.plugins.yampmusicplayer.capitalizeTitleAndArtist.value: artist = titlecase(artist)
		if self.artistFanartDl != '': return		# last download not yet finished
		if self.fanartDisplayTimer.isActive:
			self.fanartDisplayTimer.stop()
		self.artistFanartDl = artist
		self.getArtistMbid() 						#Muscibrainz Artist id from database or online

	def getArtistPicsCB(self, artistId, mbid, picsloaded = 0):
		self.showFanartNumberDl()
		if mbid is None: 
			mbid = ''
			self.artistFanartDl = ''
		LOG('YampScreen: getArtistPicsCB: artist: %s artistid: %d mbid: %s ' %(self.artistFanartDl,artistId,mbid), 'all')
#		LOG('YampScreen: getArtistPicsCB: artist: %s artistid: %d mbid: %s ' %(self.artistFanartDl,artistId,mbid), 'spe2')
#		if len(self.artistBgPicsList) > 0: return		# last download not yet finished
		self.newArtistBg = False
		try:
			if mbid != '':
				try:
					LOG('YampScreen: getArtistPicsCB: mbid Start: picsloaded: %d' %(picsloaded), 'all')
#					LOG('YampScreen: getArtistPicsCB: mbid Start: picsloaded: %d' %(picsloaded), 'spe2')
				except:
					LOG('YampScreen: getArtistPicsCB: mbid Start: picsloaded: EXCEPT', 'err')
				if picsloaded and self.searchFanart != 'always':
					LOG('YampScreen: getArtistPicsCB: artist: %s: pics already loaded, finish' %(self.artistFanartDl), 'all')
#					LOG('YampScreen: getArtistPicsCB: artist: %s: pics already loaded, finish' %(self.artistFanartDl), 'spe2')
					self.artistFanartDl = ''
					return
				artworkPath = config.plugins.yampmusicplayer.screenSaverArtworkPath.value
				self.pathArtistPic = self.getArtistArtDir(artworkPath,self.artistFanartDl)	
				LOG('YampScreen: getArtistPicsCB: pathArtistPic: %s' %(self.pathArtistPic), 'all')
				try:
					if not os.path.exists(self.pathArtistPic): os.makedirs(self.pathArtistPic)
					try:
						self.getFanarttv(mbid,artistId)
					except:
						LOG('YampScreen: getArtistPicsCB: getFanarttv: EXCEPT', 'err')
						self.artistFanartDl = ''
				except:
					LOG('YampScreen: getArtistPicsCB: EXCEPT: could not create Artistartdir %s' %(self.pathArtistPic) , 'err')
					self.artistFanartDl = ''
		except:
			LOG('YampScreen: getArtistPicsCB: End: EXCEPT' , 'err')
			self.artistFanartDl = ''

	def fanartDisplayStop(self):
		if self.fanartDisplayTimer.isActive(): self.fanartDisplayTimer.stop() 
		self.showFanartNumberDl()
		
	def showFanartNumberDl(self, text = '', failed = False):
		try:
			if failed: self.fanartDisplayTimer.start(8000, True)
			if text == '':
				lenList = len(self.artistBgPicsList)
				if lenList > 0:
					txt = _("Artist-Art-Pictures\ndownloading: ")
					txt = txt + str(self.fanartNumberDl - lenList + 1) + ' / ' + str(self.fanartNumberDl)
				else: txt = ''
			else: txt = text
			if txt != '':
				self["musicbrainzlogo"].setBoolean(True)
				self["fanartlogo"].setBoolean(True)
				if self.screenSaverActive and config.plugins.yampmusicplayer.showTextBg.value:
					self["downloadBackground"].setBoolean(True)
			else:
				self["musicbrainzlogo"].setBoolean(False)
				self["fanartlogo"].setBoolean(False)
				self["downloadBackground"].setBoolean(False)
			self["fanartdownload"].setText(txt)
		except:
			LOG('YampScreen: showFanartNumberDl: End: EXCEPT' , 'err')
			
		
	# get artist mbid form database or muicbrainz online	
	def getArtistMbid(self):
		con, result = ConnectDatabase()
		mbid = None
		if con is not None:
			LOG('YampScreen: getArtistMbid: search artist: %s' %(self.artistFanartDl), 'all')
#			LOG('YampScreen: getArtistMbid: search artist: %s' %(self.artistFanartDl), 'spe2')
			try:
				con.row_factory = sqlite3.Row
				cursor = con.cursor()
			except:
				LOG('YampScreen: getArtistMbid: con, cursor: EXCEPT', 'err')
			try:
				cursor.execute('SELECT * FROM Artists WHERE artist="%s";' %(self.artistFanartDl))
			except:
				LOG('YampScreen: getArtistMbid: SELECT: EXCEPT', 'err')
			try:
				rows = cursor.fetchone()
			except:
				LOG('YampScreen: getArtistMbid: fetchone: EXCEPT', 'err')
			try:
				artistId = rows['artist_id']
			except:
				LOG('YampScreen: getArtistMbid: No Aritst-ID: EXCEPT', 'all')
#				LOG('YampScreen: getArtistMbid: No Aritst-ID: EXCEPT', 'spe2')
				msg = _("Artist %s not in database,\nonline search for artist-art at fanart.tv not possible") %(self.artistFanartDl)
				self.session.open(MessageBox, msg, type = MessageBox.TYPE_INFO,timeout = 8 )
				self.artistFanartDl = ''
				return
			try:
				artistname = rows['artist']
				mbid = rows['mbid']
				picsloaded = rows['picsloaded']
				LOG('YampScreen: getArtistMbid: found artist in DB: artistname: %s mbid: %s picsloaded: %d artistId: %d' %(artistname,mbid,picsloaded,artistId), 'all')
#				LOG('YampScreen: getArtistMbid: found artist in DB: artistname: %s mbid: %s picsloaded: %d artistId: %d' %(artistname,mbid,picsloaded,artistId), 'spe2')
			except:
				LOG('YampScreen: getArtistMbid: get artistId, name, mbid: EXCEPT', 'err')
			try:
				cursor.close()  
				con.close()
			except:
				LOG('YampScreen: getArtistMbid: DB close: EXCEPT', 'err')
			try:
				if mbid != None:
					LOG('YampScreen: getArtistMbid: mbid: %s artistId: %d' %(mbid,artistId), 'all')
#					LOG('YampScreen: getArtistMbid: mbid: %s artistId: %d' %(mbid,artistId), 'spe2')
					self.getArtistPicsCB(artistId, mbid, picsloaded)
				else:
					LOG('YampScreen: getArtistMbid: mbid: None, try to get it online' , 'all')
#					LOG('YampScreen: getArtistMbid: mbid: None, try to get it online' , 'spe2')
					try:
						self.getMbidOnline(artistId)
					except:
						self.artistFanartDl = ''
						LOG('YampScreen: getArtistMbid: getMbidOnline: EXCEPT', 'err')
			except:
				self.artistFanartDl = ''
				LOG('YampScreen: getArtistMbid: End: EXCEPT', 'err')

	def getMbidOnline(self, artistId):
		mbid = None
		artistlist = []
		try:
			self.showFanartNumberDl(text = _('try to get\nMbID...'))
			searchArtist = self.artistFanartDl.replace('!', '\!')
			searchArtist = searchArtist.replace('/', '\/').strip()
#			searchArtist = searchArtist.replace('!', '\!').encode("utf-8", 'ignore')
			LOG('YampScreen: getMbidOnline: Artist: %s searchArtist: %s ' %(self.artistFanartDl,searchArtist), 'all')
#			LOG('YampScreen: getMbidOnline: Artist: %s searchArtist: %s ' %(self.artistFanartDl,searchArtist), 'spe2')

			if config.plugins.yampmusicplayer.searchMbidMode.value == 'standard':	
				url = "http://musicbrainz.org/ws/2/artist/?query=%s&dismax=true" %(quote(searchArtist))
####				url = "http://search.musicbrainz.org/ws/2/artist/?query=artist:%s&dismax=true" %(quote(searchArtist))
			else:	
				url = "http://musicbrainz.org/ws/2/artist/?query=%s" %(quote(searchArtist))
#####				url = "http://musicbrainz.org/ws/2/artist/?query=%s" %(quote(searchArtist))
#				url = "http://musicbrainz.org/ws/2/artist/?query=artist:%s" %(quote(searchArtist))
			LOG('YampScreen: getMbidOnline: url: %s' %(url), 'all')
#			LOG('YampScreen: getMbidOnline: type(url): %s url: %s' %(type(url),url), 'spe2')
			getPage(url, timeout=10).addCallback(self.getMbidParseXML,artistId).addErrback(self.getMbidFailed)
		except:
			self.artistFanartDl = ''
			LOG('YampScreen: getMbidOnline: result musicbrainz search: EXCEPT', 'err')

	def getMbidParseXML(self, xmlstring, artistId):
		LOG('YampScreen: getMbidParseXML: Start', 'all')
#		LOG('YampScreen: getMbidParseXML: Start', 'spe2')
		try:
			artistlist = self.mbidParse(xmlstring)
			LOG('YampScreen: getMbidParseXML: artistlist: %s' %(artistlist), 'all')
			if len(artistlist) == 0:
				msg = _("Music-Brainz ID for Artist %s not found,\nonline search for artist-art at fanart.tv not possible") %(self.artistFanartDl)
				self.session.open(MessageBox, msg, type = MessageBox.TYPE_INFO,timeout = 8 )
			else:	
				self.showArtistSelection(artistlist, artistId)	#maybe several artists found
		except:
			self.artistFanartDl = ''
			self.showFanartNumberDl(text = _('MbID\n download failed'), failed = True)
			msg = _("Music-Brainz ID for Artist %s not found,\nonline search for artist-art at fanart.tv not possible") %(self.artistFanartDl)
			self.session.open(MessageBox, msg, type = MessageBox.TYPE_INFO,timeout = 8 )
			
			LOG('YampScreen: getMbidParseXML: showArtistSelection: EXCEPT', 'err')
			
	def mbidParse(self, xmlstring):
		import re
		minscore = config.plugins.yampmusicplayer.searchMbidMinScore.value
		LOG('YampScreen: mbidParse: Start: minscore: %d xmlstring: %s' %(minscore, xmlstring), 'all')
#	LOG('YampScreen: mbidParse: Start: minscore: %d xmlstring: %s' %(minscore, xmlstring), 'spe2')
		artistlist = []
		try:
#			xmlstring = re.sub(' xmlns:ext=(")+.+"', '', xmlstring, count=1)
#			xmlstring = re.sub(r' xmlns:ext=(")+.+ext#-2.0"', "", xmlstring)
#			xmlstring = re.sub(r' xmlns=".+d-2.0#"', "", xmlstring)
#			xmlstring = re.sub(r' xmlns:ext=(")+.+ext#-2.0"', "", xmlstring)
#			xmlstring = re.sub(r' xmlns=".+d-2.0#"', "", xmlstring)
			xmlstring = xmlstring.replace('#','')
#			LOG('YampScreen: mbidParse: Start: xmlstring after1: %s' %(xmlstring), 'spe2')
			xmlstring = re.sub(' xmlns:ext=(")+.+ext-2.0', "", xmlstring)
#			LOG('YampScreen: mbidParse: Start: xmlstring after2: %s' %(xmlstring), 'spe2')
#			xmlstring = re.sub(" xmlns:ext=.+ext", "", xmlstring)
#			xmlstring = re.sub("#-2.0", "", xmlstring)
#			xmlstring = re.sub(" xmlns=.+d-2.0#", "", xmlstring)
#			xmlstring = re.sub(" xmlns=.+d-2.0#", "", xmlstring)
##			xmlstring = re.sub(' xmlns=".+d-2.0', "", xmlstring)
			xmlstring = re.sub(' xmlns=.+mmd-2.0', "", xmlstring)
#			LOG('YampScreen: mbidParse: Start: xmlstring after3: %s' %(xmlstring), 'spe2')
			xmlstring = re.sub("ns2:score", "score", xmlstring)
#			LOG('YampScreen: mbidParse: Start: xmlstring after4: %s' %(xmlstring), 'spe2')
			xmlstring = xmlstring.replace('\"\"\"','\"')
			xmlstring = xmlstring.replace('\"\"','\"')
##			LOG('YampScreen: mbidParse: xmlstring after: %s' %(xmlstring), 'all')
			LOG('YampScreen: mbidParse: xmlstring after: %s' %(xmlstring), 'spe2')
		except:
			LOG('YampScreen: mbidParse: xmlstring = re.sub: EXCEPT', 'err')
		try:
			root = xmlET.fromstring(xmlstring)
		except:
			self.artistFanartDl = ''
			LOG('YampScreen: mbidParse: xmlET.fromstring: EXCEPT', 'err')
#		try:
#			LOG('YampScreen: mbidParse: root.tag: %s root.attrib: %s' %(root.tag, root.attrib), 'err')
#		except:
#			LOG('YampScreen: mbidParse: xmlElement root.tag: EXCEPT', 'err')
		try:
			artistListXml = root[0]
		except:
			self.artistFanartDl = ''
			LOG('YampScreen: mbidParse: xmlElement root.tag: EXCEPT', 'err')
		try:
			for artists in artistListXml:
				score = artists.get('score')
#				LOG('YampScreen: mbidParse: for artists : Artist: %s score: %s' %(artists, score), 'spe2')
				if score == None or int(score) == 0: score = minscore
				if int(score) >= minscore:
					try:
						id = artists.get('id')
						type = artists.get('type')
						if type == None: type = ''
						name=''
						country=''
						disambiguation=''
						gender=''
						area=''
						nametag = artists.find('name')
						if nametag is not None: name = nametag.text
						countrytag = artists.find('country')
						if countrytag is not None: country = countrytag.text
						disambiguationtag = artists.find('disambiguation')
						if disambiguationtag is not None: disambiguation = disambiguationtag.text
						gendertag = artists.find('gender')
						if gendertag is not None: gender = gendertag.text
						elArea = artists.find('area')
					except:
						LOG('YampScreen: mbidParse: parser1: EXCEPT', 'err')
					try:
						if elArea is not None: 
							areaNametag = elArea.find('name')
							if areaNametag is not None: area = areaNametag.text
					except:
						LOG('YampScreen: mbidParse: parser2: EXCEPT', 'err')
					try:
##						LOG('YampScreen: mbidParse: score: %s name: %s type: %s gender: %s country: %s area: %s disamb: %s id: %s ' %(score,name,type,gender,country,area,disambiguation,id), 'all')
						LOG('YampScreen: mbidParse: score: %s name: %s type: %s gender: %s country: %s area: %s disamb: %s id: %s ' %(score,name,type,gender,country,area,disambiguation,id), 'spe2')
					except:
						LOG('YampScreen: mbidParse: parser3: EXCEPT', 'err')
					try:
						artistlist.append([name,type,gender,country,disambiguation,area,id])
					except:
						LOG('YampScreen: mbidParse: artistlist.append: EXCEPT', 'err')
		except:
			LOG('YampScreen: mbidParse: artistlist.append: EXCEPT', 'err')
		return artistlist
		

	def getMbidFailed(self,result):
		LOG('YampScreen: getMbidFailed: Start', 'err')
		self.artistFanartDl = ''
		self.showFanartNumberDl(text = _('MbID\n download failed'), failed = True)


	def writeMbidToDb(self, mbid, artistId):
		LOG('YampScreen: writeMbidToDb: Start: artistId: %d mbid: %s' %(artistId,mbid), 'all')
		if mbid is None: mbid = ''
		if mbid != '': 	
			con, result = ConnectDatabase()
			if con is not None:
				try:
					cursor = con.cursor()
				except:
					LOG('YampScreen: writeMbidToDb: cursor: EXCEPT', 'err')
					con.close()
					return
				try:
					mbid=unicode(mbid)
					cursor.execute("UPDATE Artists SET mbid='%s' WHERE artist_id=%d" %(mbid, artistId))        
#					LOG('YampScreen: writeMbidToDb: write new mbid: artistId: %d mbid: %s' %(artistId, mbid), 'all')
				except:
					LOG('YampScreen: writeMbidToDb: write new mbid: EXCEPT', 'err')
				try:
					con.commit()
				except:
					LOG('YampScreen: writeMbidToDb: commit: EXCEPT', 'err')
				try:
					cursor.close()  
					con.close()
				except:
					LOG('YampScreen: writeMbidToDb: close: EXCEPT', 'err')

	def getFanarttv(self, mbid, artistId):
		user_agent = 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_4; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.63 Safari/534.3'
		headers = { 'User-Agent' : user_agent }

#		LOG('YampScreen: getFanarttv: mbid: %s type: %s'  %(mbid,type(mbid)), 'all')
		mbid = str(mbid)
#		LOG('YampScreen: getFanarttv: mbid: %s type: %s'  %(mbid,type(mbid)), 'all')
	
		url="http://webservice.fanart.tv/v3/music/%s?api_key=%s" %(quote_plus(mbid),quote_plus(self.fanarttvAppApikey))
		if self.fanarttvPersonalApikey.strip() != '': url = url + '&client_key=%s' %(quote_plus(self.fanarttvPersonalApikey))
		url = str(url)
		LOG('YampScreen: getFanarttv: type(url): %s url %s' %(type(url), url), 'all')
#		LOG('YampScreen: getFanarttv: type(url): %s url %s' %(type(url), url), 'spe2')
#		LOG('YampScreen: getFanarttv: pathArtistPic: %s artist: %s artistId: %d'  %(self.pathArtistPic,self.artistFanartDl,artistId), 'err')
		try:	
			self.showFanartNumberDl(text = _('try to get\npicture list...'))
			getPage(url, timeout=30).addCallback(self.getFanartParseJson,artistId).addErrback(self.getFanartFailed)
		except:
			LOG('YampScreen: getFanarttv: getPage: EXCEPT', 'err')

	def getFanartParseJson(self, resultString, artistId):
		try:
			import json
#			LOG('YampScreen: getFanartParseJson: Start: artist: %s resultString: %s' %(self.artistFanartDl, resultString), 'all')
#			LOG('YampScreen: getFanartParseJson: Start: artist: %s resultString: %s' %(self.artistFanartDl, str(resultString)), 'spe2')
			LOG('YampScreen: getFanartParseJson: Start: artist: %s resultString > 100: %d' %(self.artistFanartDl, len(resultString) > 100), 'all')
#			LOG('YampScreen: getFanartParseJson: Start: artist: %s resultString > 100: %d' %(self.artistFanartDl, len(resultString) > 100), 'spe2')
			jsonDict = json.loads(resultString)
			
			if 'artistbackground' in jsonDict: numberPics = len(jsonDict["artistbackground"])	
			else: numberPics = 0
			LOG('YampScreen: getFanartParseJson: number of pictures: %d' %(numberPics), 'all')
#			LOG('YampScreen: getFanartParseJson: number of pictures: %d' %(numberPics), 'spe2')
			if numberPics == 0: 
				LOG('YampScreen: getFanartParseJson: no artistart pictures found for %s' %(self.artistFanartDl), 'all')
#				LOG('YampScreen: getFanartParseJson: no artistart pictures found for %s' %(self.artistFanartDl), 'spe2')
				self.artistFanartDl = ''
				self.showFanartNumberDl(text = _('no pictures\nfound'), failed = True)
				return

##			self.artistIdFanart = artistId	
			picNum = 0
			fulllist = []
			for pic in jsonDict["artistbackground"]:
				picNum += 1
				url = pic["url"]
				LOG('YampScreen: getFanartParseJson: No: %d  url: %s' %(picNum, url), 'all')
				fulllist.append(url)
#			LOG('YampScreen: getFanartParseJson: try1:\n', 'spe2')
#			LOG('YampScreen: getFanartParseJson: try2: len: %d %s\n' %(len(fulllist),fulllist), 'spe2')
#			LOG('YampScreen: getFanartParseJson: try3:\n', 'spe2')
				
			#check, if picture already existing
			tempList = []
			import os
			for pic in fulllist:
#				LOG('YampScreen: getFanartParseJson: try4:\n', 'spe2')
#				LOG('YampScreen: getFanartParseJson: try5: self.pathArtistPic: %s\n' %(self.pathArtistPic), 'spe2')
#				LOG('YampScreen: getFanartParseJson: try5: os.path.basename(pic): %s\n' %(os.path.basename(pic)), 'spe2')
				searchPath = os.path.join(self.pathArtistPic, os.path.basename(pic)).encode()	 
#				LOG('YampScreen: getFanartParseJson: try5: searchpath: %s\n' %(searchPath), 'spe2')
#				LOG('YampScreen: getFanartParseJson: try6:\n', 'spe2')

#				LOG('YampScreen: getFanartParseJson: try7: os.path.exists: %d\n' %(os.path.exists(searchPath)), 'spe2')
#				LOG('YampScreen: getFanartParseJson: try8:\n', 'spe2')


				if not os.path.exists(searchPath) or not os.path.isfile(searchPath):
					tempList.append(pic)
			self.artistBgPicsList = tempList	
			LOG('YampScreen: getFanartParseJson: BgListaftercheck: len: %d \n%s' %(len(self.artistBgPicsList),'\n'.join(self.artistBgPicsList)), 'all')
#			LOG('YampScreen: getFanartParseJson: BgListaftercheck: len: %d \n%s' %(len(self.artistBgPicsList),'\n'.join(self.artistBgPicsList)), 'spe2')
#			LOG('YampScreen: getFanartParseJson: BgListaftercheck: len: %d %s\n' %(len(self.artistBgPicsList),self.artistBgPicsList), 'spe2')
			try:
				self.artistIdFanart = artistId	
				if len(self.artistBgPicsList) > 0:	#pcitures existing for download
#					LOG('YampScreen: getFanartParseJson: BgListaftercheck: len: %d \n%s' %(len(self.artistBgPicsList),'\n'.join(self.artistBgPicsList)), 'spe2')
#					self.artistIdFanart = artistId	
					self.fanartNumberDl = len(self.artistBgPicsList)
					self.fanartSuccessDl = 0
					self.downloadPiclist()
				else: 
					LOG('YampScreen: getFanartParseJson: BgListaftercheck: len: %d' %(len(self.artistBgPicsList)), 'spe2')
					self.writePicsloadedToDb()				#no pictures
					LOG('YampScreen: getFanartParseJson: writePicsloadedToDb', 'spe2')
					self.artistFanartDl = ''
#					self.showFanartNumberDl()
			except:
				self.artistFanartDl = ''
#				self.showFanartNumberDl()
				LOG('YampScreen: getFanartParseJson: downloadPiclist: EXCEPT' , 'err')
		except:
			self.artistFanartDl = ''
#			self.showFanartNumberDl()
			try:
				LOG('YampScreen: getFanartParseJson: EXCEPT: artist: %s resultString > 100: %d' %(self.artistFanartDl, len(resultString) > 100), 'err')
			except:	
				LOG('YampScreen: getFanartParseJson: EXCEPT', 'err')
		self.showFanartNumberDl()

	def getFanartFailed(self,result):
		try:
			if '404 not found' in str(result).lower():
				self.showFanartNumberDl(text = _('fanart.tv\nartist not found'), failed = True)
				LOG('YampScreen: getFanartFailed: 404 not found' , 'all')
#				LOG('YampScreen: getFanartFailed: 404 not found' , 'spe2')
			else:
				self.showFanartNumberDl(text = _('picture list\ndownload failed'), failed = True)
				LOG('YampScreen: getFanartFailed: result: %s' %(str(result)), 'all')
#				LOG('YampScreen: getFanartFailed: result: %s' %(str(result)), 'spe2')
		except: LOG('YampScreen: getFanartFailed: EXCEPT' , 'err')
		self.artistFanartDl = ''
#		try: LOG('YampScreen: getFanartFailed: type: %s result: %s' %(type(result),result), 'spe2')
#		except: LOG('YampScreen: getFanartFailed: EXCEPT' , 'err')
#		try: LOG('YampScreen: getFanartFailed: type: %s result: %s' %(type(str(result)),str(result)), 'spe2')
#		except: LOG('YampScreen: getFanartFailed: EXCEPT2' , 'err')


		
	def downloadPiclist(self):
		LOG('YampScreen: downloadPiclist: Start: artist: %s len(artistBgPicsList): %d newArtistBg: %d' %(self.artistFanartDl,len(self.artistBgPicsList), self.newArtistBg), 'all')
#		LOG('YampScreen: downloadPiclist: Start: artist: %s len(artistBgPicsList): %d newArtistBg: %d' %(self.artistFanartDl,len(self.artistBgPicsList), self.newArtistBg), 'spe2')

		if self.fanartDlTime == FANARTDLTIME and (self.fanartNumberDl - len(self.artistBgPicsList)) == 5:
			LOG('YampScreen: downloadPiclist: limit personal api key', 'all')
#			LOG('YampScreen: downloadPiclist: limit personal api key', 'spe2')
			self.artistBgPicsList = []

		#download finished and artist is still the artist of download -> initScreensaver
		if len(self.artistBgPicsList) == 0:
			LOG('YampScreen: downloadPiclist: no (more) pictures' , 'all')
#			LOG('YampScreen: downloadPiclist: no (more) pictures' , 'spe2')
			if self.fanartSuccessDl == self.fanartNumberDl: 
				LOG('YampScreen: downloadPiclist: all successfully downlaoded, writePicsloadedToDb: success: %d total: %d' %(self.fanartSuccessDl,self.fanartNumberDl), 'all')
#				LOG('YampScreen: downloadPiclist: all successfully downlaoded, writePicsloadedToDb: success: %d total: %d' %(self.fanartSuccessDl,self.fanartNumberDl), 'spe2')
				self.writePicsloadedToDb()
			if self.newArtistBg:
				currartist = self.currArtist		
				if config.plugins.yampmusicplayer.capitalizeTitleAndArtist.value: currartist = titlecase(currartist)
#				try: LOG('YampScreen: downloadPiclist: artist: %s currArtist: %s' %(self.artistFanartDl,currartist), 'spe2')
#				except: LOG('YampScreen: downloadPiclist: artist: currArtist: EXCEPT', 'spe2')
				if currartist == self.artistFanartDl:
#					self.initScreenSaver(artist = currartist, newArtist = True)
					self.initScreenSaver(artist = currartist)
					LOG('YampScreen: downloadPiclist: new artistbgpictures: initScreenSaver', 'all')
#					LOG('YampScreen: downloadPiclist: new artistbgpictures: initScreenSaver', 'spe2')
			self.artistFanartDl = ''
			self.showFanartNumberDl()
			return
		
		LOG('YampScreen: downloadPiclist: len(list) > 0: artistBgPicsList: %s' %(self.artistBgPicsList), 'all')
		try: url = self.artistBgPicsList[0].encode()	
		except: LOG('YampScreen: downloadPiclist: url: EXCEPT' , 'err')
		try: filename = url.rsplit('/',1)[1]
		except: LOG('YampScreen: downloadPiclist: filename: EXCEPT' , 'err')
		try: fullname = os.path.join(self.pathArtistPic, filename)
		except: LOG('YampScreen: downloadPiclist: fullname: EXCEPT' , 'err')
		try: LOG("YampScreen: downloadPiclist: url: %s fullname: %s" %(url,fullname), 'all')
		except: LOG('YampScreen: downloadPiclist: url, filename: EXCEPT' , 'err')
		try:
			LOG("YampScreen: downloadPiclist: downloadPage: url: %s fullname: %s" %(url,fullname), 'all')
		except:
			LOG('YampScreen: downloadPiclist: downloadPage: url: EXCEPT' , 'err')
		self.showFanartNumberDl()
		try:
			LOG('YampScreen: downloadPiclist: fanartDlTimer start' , 'all')
#			LOG('YampScreen: downloadPiclist: fanartDlTimer start' , 'spe2')
			self.fanartDlTimer.start(self.fanartDlTime, True)
			self.artistFanartDlFinished = False
			downloadPage(url, fullname).addCallback(boundFunction(self.artistartDownloadFinished, fullname)).addErrback(self.artistartDownloadFailed)
		except:
			LOG('YampScreen: downloadPiclist: downloadPage: EXCEPT' , 'err')

		
	def fanartDlOk(self):		#vom Überwachungstimer
		LOG('YampScreen: fanartDlOk: DlFinished: %d' %(self.artistFanartDlFinished), 'all')
#		LOG('YampScreen: fanartDlOk: DlFinished: %d' %(self.artistFanartDlFinished), 'spe2')
		if not self.artistFanartDlFinished: return
#		if len(self.artistBgPicsList) > 0:
#			self.fanartDlTimer.start(self.fanartDlTime, True)
		self.downloadPiclist()	

			
	def artistartDownloadFinished(self, fullname, result):
		LOG("YampScreen: artistartDownloadFinished: Start: DlTimerActive: %d artist: %s" %(self.fanartDlTimer.isActive(),self.artistFanartDl), 'all')
#		LOG("YampScreen: artistartDownloadFinished: Start: DlTimerActive: %d artist: %s" %(self.fanartDlTimer.isActive(),self.artistFanartDl), 'err')
		self.artistFanartDlFinished = True
		self.fanartSuccessDl += 1
		try:
			LOG('YampScreen: artistartDownloadFinished: success: %d fullname: %s artist: %s' %(self.fanartSuccessDl,fullname, self.artistFanartDl), 'all')
#			LOG('YampScreen: artistartDownloadFinished: success: %d fullname: %s artist: %s' %(self.fanartSuccessDl,fullname, self.artistFanartDl), 'spe2')
			if len(self.artistBgPicsList) > 0:
				self.artistBgPicsList.remove(self.artistBgPicsList[0])
#				self.fanartDlTimer.start(self.fanartDlTime, True)
			self.newArtistBg = True
		except:
			LOG('YampScreen: artistartDownloadFinished: remove: EXCEPT' , 'err')
		try:
			if not self.fanartDlTimer.isActive():
#				if len(self.artistBgPicsList) > 0:
#					self.fanartDlTimer.start(self.fanartDlTime, True)
				self.downloadPiclist()	
		except:
			LOG('YampScreen: artistartDownloadFinished: downloadPiclist: EXCEPT' , 'err')

		
	def artistartDownloadFailed(self, result):
		artist = self.artistFanartDl
		LOG("YampScreen: artistartDownloadFailed: DlTimerActive: %d artist: %s" %(self.fanartDlTimer.isActive(),self.artistFanartDl), 'all')
#		LOG("YampScreen: artistartDownloadFailed: DlTimerActive: %d artist: %s" %(self.fanartDlTimer.isActive(),self.artistFanartDl), 'spe2')
		self.artistFanartDlFinished = True
		if len(self.artistBgPicsList) > 0:
			try:
				actDownload = self.artistBgPicsList[0]
#				LOG('YampScreen: artistartDownloadFailed', 'all')
				LOG('YampScreen: artistartDownloadFailed: %s' %(actDownload), 'err')	###Test
				self.artistBgPicsList.remove(actDownload)
			except:
				LOG('YampScreen: artistartDownloadFailed: remove: EXCEPT' , 'err')
		if not self.fanartDlTimer.isActive():
			if len(self.artistBgPicsList) > 0:
				self.fanartDlTimer.start(self.fanartDlTime, True)
			self.downloadPiclist()	
#			self.downloadPiclist(artist)	

	def writePicsloadedToDb(self):
		try:
			LOG('YampScreen: writePicsloadedToDb: Start: artistId: %d' %(self.artistIdFanart), 'all')
#			LOG('YampScreen: writePicsloadedToDb: Start: artistId: %d' %(self.artistIdFanart), 'spe2')
		except:	
			LOG('YampScreen: writePicsloadedToDb: Start: artistId: EXCEPT', 'err')
		con, result = ConnectDatabase()
		if con is not None:
			try:
				cursor = con.cursor()
			except:
				LOG('YampScreen: writePicsloadedToDb: cursor: EXCEPT', 'err')
				con.close()
				return
			try:
				cursor.execute("UPDATE Artists SET picsloaded='1' WHERE artist_id=%d" %(self.artistIdFanart))        
				LOG('YampScreen: writePicsloadedToDb: write picsloaded = 1 artistId: %d' %(self.artistIdFanart), 'all')
			except:
				LOG('YampScreen: writePicsloadedToDb: write picsloaded: EXCEPT', 'err')
			try:
				con.commit()
			except:
				LOG('YampScreen: writePicsloadedToDb: commit: EXCEPT', 'err')
			try:
				cursor.close()  
				con.close()
			except:
				LOG('YampScreen: writePicsloadedToDb: close: EXCEPT', 'err')


	def getUnixErrorString(self, s):
		# unicode error fix dirty....
		try:
			s=re_sub('\xe9','e',s)
		except:
			LOG('YampScreen: getUnixErrorString: EXCEPT' , 'err')
		return s

	def showArtistSelection(self, artistlist, artistId):			#select artist, if several with same name
		try:
#			LOG('YampScreen: showArtistSelection: artist: %s artistId: %d artistlist: %s' %(self.artistFanartDl, artistId, artistlist), 'err')
			if len(artistlist) > 1:
				self.hideScreenSaver()
				menu = []
				textLen = []
				idx = 0
				try:
					import re
					for artis in artistlist:
						text=', '.join(artis[:6]).rstrip(', ').encode("utf-8", 'ignore')
						text = re.sub('(, )+',', ',text)
						LOG('YampScreen: showArtistSelection: len: %d text: %s'  %(len(text),text), 'all')
						menu.append((text, str(idx)))
						textLen.append(len(text)+10)
						idx += 1
				except:
					LOG('YampScreen: showArtistSelection: menu append: EXCEPT', 'err')
				self.par1 = artistlist
				self.par2 = artistId
				title = _('Several Artists %s found at musicbrainz.org, please select Artist') %(self.artistFanartDl)
				LOG('YampScreen: showArtistSelection: len: %d title: %s'  %(len(title),title), 'all')
				textLen.append(len(title))
				title = title.ljust(max(textLen)) + '.'
#				LOG('YampScreen: showArtistSelection: maxlen: %d'  %(max(textLen)), 'err')
				try:
					self.session.openWithCallback(self.showArtistSelectionCB, ChoiceBox, title=title, list=menu)
				except:
					LOG('YampScreen: showArtistSelection: ChoiceBoxYamp: EXCEPT', 'err')
					self.getArtistPicsCB(artistId, None)
			elif len(artistlist) == 1:
				mbid = artistlist[0][6]
				self.writeMbidToDb(mbid, artistId)
				self.getArtistPicsCB(artistId, mbid)
			else:
				self.getArtistPicsCB(artistId, None)
		except:
			LOG('YampScreen: showArtistSelection: : EXCEPT', 'err')
			self.getArtistPicsCB(artistId, None)
		
	def showArtistSelectionCB(self, choice):
###		LOG('YampScreen: showArtistSelectionCB: call startSreenSaverTimer', 'spe')
		artistlist = self.par1
		artistId = self.par2
##		if choice is None or self.currentIsVideo: mbid = None
		if choice is None: mbid = None
		else:
			mbid = artistlist[int(choice[1])][6]
			self.writeMbidToDb(mbid, artistId)
#		self.startScreenSaverTimer()
		self.getArtistPicsCB(artistId, mbid)
		if not self.currentIsVideo or self.videoStartMode != "manual":
#			LOG('YampScreen: showArtistSelectionCB: resetScreenSaverTimer', 'all') 
			self.resetScreenSaverTimer()

			
#-----------------------------------			
#
# Possibly blocking database actions use their own thread and are called from their own screen (YampDatabaseScreen)
#
class ThreadQueue:
	def __init__(self):
		self.__list = [ ]
		self.__lock = Lock()

	def push(self, val):
		lock = self.__lock
		lock.acquire()
		self.__list.append(val)
		lock.release()

	def pop(self):
		lock = self.__lock
		lock.acquire()
		ret = self.__list.pop()
#		if self.__list:
#			ret = self.__list.pop()
#		else:
#			ret = (1, None, None)
		lock.release()
		return ret

class YampDbActions(Thread):
	def __init__(self):
		Thread.__init__(self)
		self.__running = False
		self.__cancel = False
		self.__path = None
		self.__messages = ThreadQueue()
		self.__messagePump = ePythonMessagePump()
#		self.__beginn = None
#		self.__end = None

	def __getMessagePump(self):
		return self.__messagePump
	MessagePump = property(__getMessagePump)

	def __getMessageQueue(self):
		return self.__messages
	Message = property(__getMessageQueue)

	def __getRunning(self):
		return self.__running
	isRunning = property(__getRunning)

	def Cancel(self):
#		try: LOG('YampDbActions: ++ Cancel received', 'spe')
#		except: LOG('YampDbActions: ++ Cancel received: EXCEPT', 'err')
		self.__cancel = True


	def Start(self, func, dir, recursive):
#		try: LOG('YampDbActions: ++ Start: func: %s dir: %s recursive: %d' %(func, dir, recursive), 'spe')
#		except: LOG('YampDbActions: ++ Start: func: dir: recursive: EXCEPT', 'err')
		if not self.__running:
#			self.__running = True
			self.__func = func
			self.__path = dir
			self.__recursive = recursive
			self.start()
#		else:
#			self.__list.append(item)


	def run(self):
		self.mp = self.__messagePump
		self.__running = True
		self.__cancel = False
		con, result = ConnectDatabase()
		if con is not None:
#			try: LOG('YampDbActions: ++ run: con is not None', 'spe')
#			except: LOG('YampDbActions: ++ run: con is not None: EXCEPT', 'err')
			con.text_factory = str
			cursor = con.cursor()
			self.counteradded = 0
			self.counterskipped = 0
			self.checkTime = time()
			
			if self.__func == "add":
#				try: LOG('YampDbActions: ++ run: func == add', 'spe')
#				except: LOG('YampDbActions: ++ run: func == add: EXCEPT', 'err')
				self.__messages.push((THREAD_WORKING, _("searching ... ")))
				self.mp.send(0)
				self.addDirToDatabase(con, cursor, self.__path, self.__recursive)
#				try: LOG('YampDbActions: ++ run: addDir finished: cancel: %d' %(self.__cancel), 'spe')
#				except: LOG('YampDbActions: ++ run: addDir finished: cancel: EXCEPT', 'err')
				if not self.__cancel:
					con.commit()
				cursor.close()  
				con.close()
				msgAdded = _('%d files added to database\n') %(self.counteradded)
				msgSkipped = _('%d files skipped\n\n') %(self.counterskipped)
				msgClose = _('Press OK to close.')
				if self.__cancel:
#					try: LOG('YampDbActions: ++ run: FINISHED cancel: push', 'spe')
#					except: LOG('YampDbActions: ++ run: FINISHED cancel: push: EXCEPT', 'err')
					msg1 = _('Process aborted.\n')
					self.__messages.push((THREAD_FINISHED, msg1 + msgAdded + msgSkipped + msgClose))
				else:	
#					try: LOG('YampDbActions: ++ run: FINISHED added: push', 'spe')
#					except: LOG('YampDbActions: ++ run: FINISHED added: push: EXCEPT', 'err')
					self.__messages.push((THREAD_FINISHED, msgAdded + msgSkipped + msgClose))
		else:
#			try: LOG('YampDbActions: ++ run: FINISHED cannot: push', 'spe')
#			except: LOG('YampDbActions: ++ run: FINISHED cannot: push: EXCEPT', 'err')
			self.__messages.push((THREAD_FINISHED, _("Error!\nCan not open database!\nCheck if database path is correct and writeable!\n\nPress OK to close.")))
		self.mp.send(0)
#		try: LOG('YampDbActions: ++ run: after send', 'spe')
#		except: LOG('YampDbActions: ++ run: after send: EXCEPT', 'err')
		self.__running = False
		Thread.__init__(self)

	def addDirToDatabase(self, con, cursor, dir, recursive = True):					
#		try: LOG('\nYampDbActions: addDirToDatabase: Start: dir: %s diff: %s time: %s checkTime: %s'  %(dir, time() - self.checkTime,time(),self.checkTime), 'spe')
#		except: LOG('\nYampDbActions: addDirToDatabase: Start: EXCEPT', 'err')
		filelist = YampFileList(dir, matchingPattern = "(?i)^.*\.(mp2|mp3|wav|flac|ogg|m4a|m3u|pls|e2pls|mp4)", useServiceRef = True, showMountpoints = False, isTop = True, additionalExtensions = "4098:m3u 4098:e2pls 4098:pls")
		for x in filelist.getFileList():
#			try: LOG('\nYampDbActions: addDirToDatabase: diff: %s time: %s checkTime: %s'  %(time() - self.checkTime,time(),self.checkTime), 'spe')
#			except: LOG('\nYampDbActions: addDirToDatabase: difftime: EXCEPT', 'err')
#			try: LOG('YampDbActions: addDirToDatabase: processing filelist x: %s' %(x), 'spe')
#			except: LOG('YampDbActions: addDirToDatabase: processing filelist x: EXCEPT', 'err')
#			try: LOG('YampDbActions: addDirToDatabase: processing filelist x[0][0]: %s  isDir: %s' %(x[0][0],x[0][1]), 'spe')
#			except: LOG('YampDbActions: addDirToDatabase: processing filelist x[0][0]: EXCEPT', 'err')
			if self.__cancel:	break
			if x[0][1] == True: #isDir
#				try: LOG('YampDbActions: addDirToDatabase: is directory', 'spe')
#				except:	LOG('YampDbActions: addDirToDatabase: is directory: EXCEPT', 'err')
				if recursive:
					if x[0][0] != dir:
#						try: LOG('YampDbActions: addDirToDatabase: x[0][0] != dir: x[0][0]: %s' %(x[0][0]), 'spe')
#						except:	LOG('YampDbActions: addDirToDatabase: x[0][0] != dir: x[0][0]: EXCEPT', 'err')
						self.addDirToDatabase(con, cursor, x[0][0])
						con.commit()
			else:
#				try: LOG('YampDbActions: addDirToDatabase: is file: path: %s' %(x[0][0].getPath()), 'spe')
#				except:	LOG('YampDbActions: addDirToDatabase: is file: path: EXCEPT', 'err')
				f = x[0][0].getPath()
				if f.lower().endswith(".m3u") or f.lower().endswith(".pls") or f.lower().endswith(".e2pls"):
					success = dbInsert(con, cursor, x[0][0], table="playlists", update=False)
				else: success = dbInsert(con, cursor, x[0][0], table="titles", update=False)
				if success:
					self.counteradded +=1
#					try: LOG('YampDbActions: addDirToDatabase: success: file added: %s' %(f), 'spe')
#					except:	LOG('YampDbActions: addDirToDatabase: success: file added: EXCEPT', 'err')
				else:
					self.counterskipped +=1
#					try: LOG('YampDbActions: addDirToDatabase: file already existing: %s' %(f), 'spe')
#					except:	LOG('YampDbActions: addDirToDatabase: file already existing: EXCEPT', 'err')
				if time() - self.checkTime >= 0.15: # update interval for GUI				
					if success:
#						try: LOG('YampDbActions: ++ addDirToDatabase: added: counteradded: %d counterskipped: %d' %(self.counteradded,self.counterskipped), 'spe')
#						except: LOG('YampDbActions: ++ addDirToDatabase: added: push: EXCEPT', 'err')
						msg1 = _("%s\n\n added to database ") %(f[-80:])
						msg2 = _("(%d)") %(self.counteradded)
						try: self.__messages.push((THREAD_WORKING, msg1 + msg2))
						except: LOG('YampDbActions: ++ simPump added: push: EXCEPT', 'err')
					else:
#						try: LOG('YampDbActions: ++ addDirToDatabase: not added: counteradded: %d counterskipped: %d' %(self.counteradded,self.counterskipped), 'spe')
#						except: LOG('YampDbActions: ++ addDirToDatabase: not added: push: EXCEPT', 'err')
						msg1 = _("%s\n\n already exists in database ") %(f[-80:])
						msg2 = _("(%d)") %(self.counterskipped)
						self.__messages.push((THREAD_WORKING, msg1 + msg2))
					self.mp.send(0)
#					try: LOG('YampDbActions: ++ addDirToDatabase: after send', 'spe')
#					except: LOG('YampDbActions: ++ addDirToDatabase: after send: EXCEPT', 'err')
					self.checkTime = time()

databaseActions = YampDbActions()	# Build instance of YampDbActions class

#
# More screens
#
			
	
class YampDatabaseScreenV32(Screen):
	def __init__(self, session, func, dir, recursive = False):
		Screen.__init__(self, session)
		with open(os.path.join(yampDir, "skins", config.plugins.yampmusicplayer.yampSkin.value, "YampDatabase.xml"), 'r') as f:
			self.skin = f.read()

		self.func = func
		self.dir = dir
		self.recursive = recursive
#		try: LOG('YampDatabaseScreen: init: func: %s self.dir: %s recursive: %d' %(self.func, self.dir, self.recursive), 'err')
#		except: LOG('YampDatabaseScreen: init: func: dir: recursive: EXCEPT', 'err')
		
		if self.func == "add":
			if self.recursive:
				self["title"] = Label(_("YAMP Music Player - Add all directories to Database"))
			else:
				self["title"] = Label(_("YAMP Music Player - Add directory to Database"))
		else:
			self["title"] = Label(_("YAMP Music Player - Database Actions"))
		self["key_red"] = Label(_("Cancel"))
		self["key_green"] = Label("")
		self["message"] = Label()
		self["actions"] = ActionMap(["YampActions", "YampOtherActions"], 
		{
			"exit": self.keyCancel,
			"red": self.keyCancel,
			"green": self.keyClose,
			"ok": self.keyClose,
		}, -1)
		self.onClose.append(self.__onClose)
		try: self["key_green"].setText('')
		except: LOG('YampDatabaseScreen: init: keygreen setText: EXCEPT', 'err')
		self.isDreamOS = os.path.exists("/var/lib/dpkg/status")
#		try: LOG('YampDatabaseScreen: init: isDreamOS: %d' %(self.isDreamOS), 'spe')
#		except: LOG('YampDatabaseScreen: init: isDreamOS: EXCEPT', 'err')
		
		if self.isDreamOS: self.databaseActions_conn=databaseActions.MessagePump.recv_msg.connect(self.gotThreadMsg)
		else:	databaseActions.MessagePump.recv_msg.get().append(self.gotThreadMsg)	
#		try: LOG('YampDatabaseScreen: init: isRunning: %d' %(databaseActions.isRunning), 'spe')
#		except: LOG('YampDatabaseScreen: init: isRunning: EXCEPT', 'err')
		if not databaseActions.isRunning:
			databaseActions.Start(func, dir, recursive)
#		try: LOG('YampDatabaseScreen: init: isRunning2: %d' %(databaseActions.isRunning), 'spe')
#		except: LOG('YampDatabaseScreen: init: isRunning2: EXCEPT', 'err')

	def keyCancel(self):
#		try: LOG('YampDatabaseScreen: ---- keyCancel: isRunning: %d isDreamOS: %d' %(databaseActions.isRunning,self.isDreamOS), 'spe')
#		except: LOG('YampDatabaseScreen: ---- keyCancel: isRunning: EXCEPT', 'err')
		try: 
			if databaseActions.isRunning: databaseActions.Cancel()
		except: LOG('YampDatabaseScreen: keyCancel: EXCEPT', 'err')
				
	def keyClose(self):
#		try: LOG('YampDatabaseScreen: ---- keyClose: isRunning: %d isDreamOS: %d' %(databaseActions.isRunning,self.isDreamOS), 'spe')
#		except: LOG('YampDatabaseScreen: ---- keyClose: isRunning: EXCEPT', 'err')
#		try: LOG('YampDatabaseScreen: ---- keyClose: keygreen.getText: *%s*' %(self["key_green"].getText()), 'spe')
#		except: LOG('YampDatabaseScreen: ---- keyClose: keygreen.getText: EXCEPT', 'err')
		try: 
			if self["key_green"].getText() != '': self.close()
		except:	LOG('YampDatabaseScreen: keyClose: EXCEPT', 'err')

	def gotThreadMsg(self, msg):
#		try: LOG('YampDatabaseScreen: -- gotThreadMsg: Start', 'spe')
#		except: LOG('YampDatabaseScreen: -- gotThreadMsg: Start: EXCEPT', 'err')
		msg = databaseActions.Message.pop()
#		try: LOG('YampDatabaseScreen: -- gotThreadMsg: msg: %s' %(str(msg)), 'spe')
#		except: LOG('YampDatabaseScreen: -- gotThreadMsg: msg', 'err')
		self["message"].setText(msg[1])
		if msg[0] == THREAD_FINISHED:
#			try: LOG('YampDatabaseScreen: -- gotThreadMsg: THREAD_FINISHED: %s' %(str(msg[0])), 'spe')
#			except: LOG('YampDatabaseScreen: -- gotThreadMsg: THREAD_FINISHED', 'err')
			self["key_red"].setText('')
			self["key_green"].setText(_('OK'))

	def __onClose(self):
		if self.isDreamOS: self.databaseActions_conn = None
		else:	databaseActions.MessagePump.recv_msg.get().remove(self.gotThreadMsg)

		
		
#
# General routines
#
def ConnectDatabase():
	dbPath = os.path.join(config.plugins.yampmusicplayer.databasePath.value ,"yamp.db")
	try:
		con = sqlite3.connect(dbPath)
		try:
			LOG('ConnectDatabase: dbPath: %s con: %s ' %(dbPath, con), 'all') ###Test
		except:
			LOG('ConnectDatabase: dbPath: con : EXCEPT' , 'err') ###Test
		if not os.access(dbPath, os.W_OK):
			LOG('ConnectDatabase: dbPath: no write access' , 'err') ###Test
			print "[YAMP] Can not open %s for writing..." %(dbPath)
			con.close()
			return None, 2
	except:
		LOG('ConnectDatabase: cannot open database dbPath: %s ' %(dbPath), 'err') ###Test
		print "[YAMP] Can not open database file: %s" %(dbPath)
		return None, 1
	cur = con.cursor()	#!!!
	cur.execute('CREATE TABLE IF NOT EXISTS Playlists (playlist_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, playlist_filename TEXT NOT NULL UNIQUE, playlist_title TEXT);')
#	cur.execute('CREATE TABLE IF NOT EXISTS Titles (title_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, filename TEXT NOT NULL UNIQUE, title TEXT, artist_id INTEGER, album_id INTEGER, genre_id INTEGER, date_id INTEGER, length TEXT, tracknr INTEGER, sref TEXT);')
	cur.execute('CREATE TABLE IF NOT EXISTS Titles (title_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, filename TEXT NOT NULL UNIQUE, title TEXT, artist_id INTEGER, album_id INTEGER, genre_id INTEGER, date_id INTEGER, date TEXT, length TEXT, tracknr INTEGER, sref TEXT);')
#	cur.execute('CREATE TABLE IF NOT EXISTS Artists (artist_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, artist TEXT NOT NULL UNIQUE);')
#	cur.execute('CREATE TABLE IF NOT EXISTS Artists (artist_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, artist TEXT NOT NULL UNIQUE, mbid TEXT);')
	cur.execute('CREATE TABLE IF NOT EXISTS Artists (artist_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, artist TEXT NOT NULL UNIQUE, mbid TEXT, picsloaded INT DEFAULT 0);')
	cur.execute('CREATE TABLE IF NOT EXISTS Albums (album_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, album TEXT NOT NULL UNIQUE);')
	cur.execute('CREATE TABLE IF NOT EXISTS Genres (genre_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, genre TEXT NOT NULL UNIQUE);')
	cur.execute('CREATE TABLE IF NOT EXISTS Dates (date_id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, date TEXT NOT NULL UNIQUE);')
	cur.close()
	return con, 0
	
def dbUpgradeV31():	
		con, result = ConnectDatabase()
		if con is not None:
			try:
				con.row_factory = sqlite3.Row
			except:
				LOG('dbUpgradeV31: row_factory: EXCEPT', 'err')
			try:
				cursor = con.cursor()
			except:
				LOG('dbUpgradeV31: cursor: EXCEPT', 'err')
			try:
				cursor.execute("SELECT * FROM Artists")
			except:
				LOG('dbUpgradeV31: SELECT: EXCEPT', 'err')
			try:
				rows = cursor.fetchone()
			except:
				LOG('dbUpgradeV31: fetchone: EXCEPT', 'err')
			try:
				a=rows['artist_id']
			except:
				LOG('dbUpgradeV31: a=rows[artist_id]: EXCEPT: most likely empty database', 'err')
				return 1
			try:
				b=rows['artist']
			except:
				LOG('dbUpgradeV31: b=rows[artist]: EXCEPT: most likely empty database', 'err')
				return 2
			try:
				c=rows['mbid']
#				LOG('dbUpgradeV31: c=rows[mbid] exists: already upgraded', 'err')
				return 0
			except:
				LOG('dbUpgradeV31: c=rows[mbid]: EXCEPT: must be upgraded', 'err')
			upgradeerror = 0	
			try:
				cursor.execute("ALTER TABLE Artists ADD COLUMN 'mbid' TEXT")
			except:		
				LOG('dbUpgradeV31: ALTER TABLE: mbid: EXCEPT', 'err')
				upgradeerror = 3
			try:
				cursor.execute("ALTER TABLE Artists ADD COLUMN 'picsloaded' INT DEFAULT 0")
			except:		
				LOG('dbUpgradeV31: ALTER TABLE: picsloaded: EXCEPT', 'err')
				upgradeerror = 4
			try:
				con.commit()
			except:		
				LOG('dbUpgradeV31: commit: EXCEPT', 'err')
				upgradeerror = 5
			try:
				cursor.close()  
				con.close()
			except:
				LOG('dbUpgradeV31: close: EXCEPT', 'err')
			return upgradeerror		

		
def dbInsert(con, cursor, ref, table="titles", update=False):
	res = False
	f = ref.getPath()
	sref = str(ServiceReference(ref))
	if table == "titles":
		cursor.execute('SELECT title_id FROM Titles WHERE filename = "%s";' % f)
		row = cursor.fetchone()
#		try: LOG('YampDbActions: dbInsert: Start titles: filename: %s' %(f), 'spe')
#		except:	LOG('YampDbActions: dbInsert: Start titles: EXCEPT', 'err')
		if row is None:
#			try: LOG('YampDbActions: dbInsert: filename not found, insert' , 'spe')
#			except:	LOG('YampDbActions: dbInsert: filename not found, insert: EXCEPT', 'err')
			try: title, album, genre, artist, date, length, tracknr, strBitrate = readID3Infos(f)
			except:	LOG('YampDbActions: dbInsert: readID3Infos: EXCEPT', 'err')
#			try: 	LOG('YampDbActions: dbInsert: title: %s album: %s genre: %s artist: %s date: %s length: %s tracknr: %d' %(title, album, genre, artist, date, length, tracknr), 'spe')
#			except:	LOG('YampDbActions: dbInsert: Log Id3Infos: EXCEPT', 'err')
			# 1. Artist
			if config.plugins.yampmusicplayer.capitalizeTitleAndArtist.value:
				artist = titlecase(artist)
			cursor.execute('SELECT artist_id FROM Artists WHERE artist = "%s";' % (artist.replace('"','""')))
			row = cursor.fetchone()
			if row is None:
				cursor.execute('INSERT INTO Artists (artist) VALUES("%s");' % (artist.replace('"','""')))
				artistID = cursor.lastrowid
			else:
				artistID = row[0]
			# 2. Album
			cursor.execute('SELECT album_id FROM Albums WHERE album = "%s";' % (album.replace('"','""')))
			row = cursor.fetchone()
			if row is None:
				cursor.execute('INSERT INTO Albums (album) VALUES("%s");' % (album.replace('"','""')))
				albumID = cursor.lastrowid
			else:
				albumID = row[0]
			# 3. Genre
			cursor.execute('SELECT genre_id FROM Genres WHERE genre = "%s";' % (genre.replace('"','""')))
			row = cursor.fetchone()
			if row is None:
				cursor.execute('INSERT INTO Genres (genre) VALUES("%s");' % (genre.replace('"','""')))
				genreID = cursor.lastrowid
			else:
				genreID = row[0]
			
			# 4. Date
			cursor.execute('SELECT date_id FROM Dates WHERE date = "%s";' % (date.replace('"','""')))
			row = cursor.fetchone()
			if row is None:
				cursor.execute('INSERT INTO Dates (date) VALUES("%s");' % (date.replace('"','""')))
				dateID = cursor.lastrowid
			else:
				dateID = row[0]

			# 5. Titles
			if config.plugins.yampmusicplayer.capitalizeTitleAndArtist.value:
				title = titlecase(title)
			try:
#				cursor.execute("INSERT INTO Titles (filename,title,artist_id,album_id,genre_id, length,date,tracknr, sref) VALUES(?,?,?,?,?,?,?,?,?);" , (f,title,artistID,albumID,genreID,length,date,tracknr,sref))
				cursor.execute("INSERT INTO Titles (filename,title,artist_id,album_id,genre_id,date_id, length,date,tracknr, sref) VALUES(?,?,?,?,?,?,?,?,?,?);" , (f,title,artistID,albumID,genreID,dateID,length,date,tracknr,sref))
				res = True
			except sqlite3.IntegrityError:
				pass  # update if True
	elif table == "playlists":
#		LOG('YampScreen: dbInsert: playlist: Start', 'spe')
#		try: LOG('YampScreen: dbInsert: playlist: ref.gethpath f: %s' %(f), 'spe')
#		except: LOG('YampScreen: dbInsert: playlist: ref.gethpath f: EXCEPT', 'err')
		cursor.execute('SELECT playlist_id FROM Playlists WHERE playlist_filename = "%s";' % f)
		row = cursor.fetchone()
		if row is None:
			text = ref.getName()
#			try: LOG('YampScreen: dbInsert: playlist: text: %s' %(text), 'spe')
#			except: LOG('YampScreen: dbInsert: playlist: text: EXCEPT', 'err')
			if text == "": text = os.path.basename(f)
#			try: LOG('YampScreen: dbInsert: playlist: text basename: %s' %(text), 'spe')
#			except: LOG('YampScreen: dbInsert: playlist: text basename: EXCEPT', 'err')
			try:
#				sql='INSERT INTO Playlists (playlist_filename,playlist_title) VALUES(\"%s\",\"%s\")' %(f,text)
#				try: LOG('YampScreen: dbInsert: playlist: sql: %s' %(sql), 'spe')
#				except: LOG('YampScreen: dbInsert: playlist: sql: EXCEPT', 'err')
#				try: cursor.execute(sql)
				try: cursor.execute("INSERT INTO Playlists (playlist_filename,playlist_title) VALUES(?,?);" , (f,text))
				except: LOG('YampScreen: dbInsert: playlist: execute sql: EXCEPT', 'err')
				res = True
			except sqlite3.IntegrityError:
				pass
	return res
	
def readID3Infos(pathname):
#	LOG('readID3Infos: Start', 'err')
	audio = None
	title = ""
	genre = "n/a"
	artist = "n/a"
	album = "n/a"
	date = "n/a"
	length = "n/a"
	tracknr = -1
	bitrate = 0.0
	strBitrate = 'n/a'
	if pathname.lower().endswith(".mp3"):
		try: audio = MP3(pathname, ID3 = EasyID3)
		except: audio = None
	elif pathname.lower().endswith(".flac"):
		try: 
			audio = FLAC(pathname)
			LOG('readID3Infos: FLAC', 'all')
#			try: LOG('readID3Infos: FLAC: audio: %s' %(audio), 'err') 
#			except: LOG('readID3Infos: FLAC: audio: EXCEPT', 'err')
				
			try:
				filesize = os.path.getsize(pathname)
				duration = audio.info.length
#				LOG('readID3Infos: getFlacBitrate: duration: %f' %(duration), 'err')
				if duration > 0:
					bitrate = filesize / duration * 8.0 / 1000.0
#				LOG('readID3Infos: getFlacBitrate: bitrate: %f' %(bitrate), 'err')
			except: LOG('readID3Infos: getFlacBitrate: EXCEPT', 'err')
		except: audio = None
	elif pathname.lower().endswith(".m4a") or pathname.lower().endswith(".mp4"):
#		LOG('readID3Infos: mp4, m4a: pathname: %s' %(pathname), 'err') 
			try: audio = EasyMP4(pathname)		#old method
			except: 
				audio = None
				try: audio = mutagen.File(pathname, easy=True)		#new method
				except: audio = None
			try: LOG('YampScreen:readID3Infos: m4a, mp4: audio: %s' %(audio), 'spe') 
#			try: LOG('readID3Infos: m4a, mp4: audio: %s' %(audio), 'spe2') 
			except: LOG('YampScreen:readID3Infos: m4a, mp4: audio: EXCEPT', 'err')
	elif pathname.lower().endswith(".ogg"):
		try: audio = OggVorbis(pathname)
		except: audio = None
#	try: LOG('\nreadID3Infos: audio: %s' %(audio), 'err') 
#	except: LOG('readID3Infos: audio: EXCEPT', 'err')
	filename = os.path.splitext(os.path.basename(pathname))[0]
#	try: LOG('readID3Infos: pathname: %s filename: %s' %(pathname,filename), 'spe') 
#	except: LOG('readID3Infos: readID3Infos: pathname, filename:: EXCEPT', 'err')
	if audio:
		try: title = getEncodedString(audio.get('title', [filename])[0])
		except: LOG('readID3Infos: get title: EXCEPT', 'err')
		try: album = getEncodedString(audio.get('album', ['n/a'])[0])
		except: LOG('readID3Infos: title: %s: get album: EXCEPT' %(title), 'err')
		try: artist = getEncodedString(audio.get('artist', ['n/a'])[0])
		except: LOG('readID3Infos: title: %s album: %s: get artist: EXCEPT' %(title, album), 'err')
		try: genre = getEncodedString(audio.get('genre', ['n/a'])[0])
		except: LOG('readID3Infos: title: %s artist: %s: get genre: EXCEPT' %(title, artist), 'err')
		try: date = getEncodedString(audio.get('date', ['n/a'])[0])
		except: LOG('readID3Infos: title: %s artist: %s: get date: EXCEPT' %(title, artist), 'err')
		try: tracknr = int(audio.get('tracknumber', ['-1'])[0].split("/")[0])
		except:	pass
		if bitrate == 0.0:
			try: bitrate = audio.info.bitrate / 1000.0
			except: pass
		try:
			len = int(audio.info.length)
			length = str(datetime_timedelta(seconds=len)).encode("utf-8", 'ignore')
			if len < 3600:
				length = length[-5:]
		except:
			length = "n/a"
#		try: LOG('readID3Infos: title: %s album: %s genre: %s artist: %s date: %s tracknr: %d length: %s ' %(title,album,genre,artist,date,tracknr,length), 'spe')
#		except: LOG('readID3Infos: title: album: genre: artist: date: tracknr: length: EXCEPT', 'err')
	else:
		title = filename
	if bitrate > 0.0: strBitrate = str(round(bitrate,1)) + ' kBit/s' 	
#	LOG('readID3Infos: title: %s  bitrate: %s' %(title, strBitrate), 'err')
	
	return title, album, genre, artist, date, length, tracknr, strBitrate

def getEncodedString(value):
	returnValue = ""
	try:
		returnValue = value.encode("utf-8", 'ignore')
	except UnicodeDecodeError:
		try:
			returnValue = value.encode("iso8859-1", 'ignore')
		except UnicodeDecodeError:
			try:
				returnValue = value.decode("cp1252").encode("utf-8")
			except UnicodeDecodeError:
				returnValue = "n/a"
	return returnValue

	
def htmlUnescape(text):		#ab hier #neuTest6
		text = text.replace('&amp;', '&')
		text = text.replace('&quot;', '"')
		text = text.replace('&gt;', ">")
		text = text.replace('&lt;', "<")
		return text

		
